package com.infobip.push;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;

import com.infobip.push.lib.InfobipNotificationHandlingUtil;
import com.infobip.push.lib.util.Consts;

/**
 * <p>Abstract receiver class to be extended if you want to be notified when the user is registered or unregistered, when the notification is received 
 * or opened, or when an error occurs.</p>
 * 
 */
public abstract class AbstractPushReceiver extends BroadcastReceiver {

	/**
	 * <p>Called when the intent with one of the following actions was received: </p>
	 * <code>com.infobip.push.intent.REGISTERED_FOR_NOTIFICATIONS</code> <br />
	 * <code>com.infobip.push.intent.REGISTRATION_REFRESHED</code> <br />
	 * <code>com.infobip.push.intent.NOTIFICATION_RECEIVED</code> <br />
	 * <code>com.infobip.push.intent.NOTIFICATION_OPENED</code> <br />
	 * <code>com.infobip.push.intent.UNREGISTERED_FROM_NOTIFICATIONS</code> <br />
	 * <code>com.infobip.push.intent.ERROR</code>
	 * 
	 * <p>Please ensure that these actions are added to the intent filter in the manifest.</p>
	 */
	@Override
	public final void onReceive(Context context, Intent intent) {
		
		String currentAction = intent.getAction();	
		// user registered
		if (Consts.RECEIVER_REGISTERED_ACTION.equals(currentAction)) {
			onRegistered(context);
			return;
		}
		
		// user registration refreshed
		if (Consts.RECEIVER_REGISTRATION_REFRESHED_ACTION.equals(currentAction)) {
			onRegistrationRefreshed(context);
			return;
		}
		
		// notification received
		if (Consts.RECEIVER_NOTIFICATION_RECEIVED_ACTION.equals(currentAction)) {
			PushNotification notification = (PushNotification) InfobipNotificationHandlingUtil.fetchParcelledNotificationFromIntent(intent);			
			onNotificationReceived(notification, context);
			return;
		}
		
		// notification opened
		if (Consts.RECEIVER_NOTIFICATION_OPENED_ACTION.equals(currentAction)) {			
			PushNotification notification = (PushNotification) InfobipNotificationHandlingUtil.fetchParcelledNotificationFromIntent(intent);			
			onNotificationOpened(notification, context);	
			return;
		}
		
		// user unregistered
		if (Consts.RECEIVER_UNREGISTERED_ACTION.equals(currentAction)) {
			onUnregistered(context);
			return;
		}
		
		// error occurred
		if (Consts.RECEIVER_ERROR_ACTION.equals(currentAction)) {
			int reason = intent.getIntExtra(Consts.EXTRA_REASON, -1);
			onError(reason, context);
			return;
		}
	}

	/**
	 * <p>Called when a user registers for push notifications. Invoked on <code>com.infobip.push.intent.REGISTERED_FOR_NOTIFICATIONS</code> action.</p>
	 * @param context Context initialized in <code>PushNotificationManager</code> constructor.
	 * @see PushNotificationManager
	 */
	public abstract void onRegistered(Context context);
	
	/**
	 * <p>Called when user's registration is refreshed. Invoked on <code>com.infobip.push.intent.REGISTRATION_REFRESHED</code> action.</p>
	 * @param context Context initialized in <code>PushNotificationManager</code> constructor.
	 * @see PushNotificationManager
	 */
	protected void onRegistrationRefreshed(Context context) {
		
	}
	
	/**
	 * <p>Called when a user receives a notification. Method provides an instance of the received <code>PushNotification</code> object to be managed.
	 * Invoked on <code>com.infobip.push.intent.NOTIFICATION_RECEIVED</code> action.</p>
	 * @param notification {@link PushNotification} instance.
	 * @param context Context initialized in <code>PushNotificationManager</code> constructor.
	 * @see PushNotificationManager
	 */
	public abstract void onNotificationReceived(PushNotification notification, Context context);
	
	/**
	 * <p>Called when a user opens a push notification. Method provides an instance of the opened <code>PushNotification</code> object to be managed.
	 * Invoked on <code>com.infobip.push.intent.NOTIFICATION_OPENED</code> action.</p>
	 * @param notification {@link PushNotification} instance.
	 * @param context Context initialized in <code>PushNotificationManager</code> constructor.
	 * @see PushNotificationManager
	 */
	protected void onNotificationOpened(PushNotification notification, Context context) {
		
	}
	
	/**
	 * <p>Called when a user unregisters from receiving push notifications. Invoked on <code>com.infobip.push.intent.UNREGISTERED_FROM_NOTIFICATIONS</code> 
	 * action.</p>
	 * @param context Context initialized with <code>PushNotificationManager</code> class.
	 * @see PushNotificationManager
	 */
	public abstract void onUnregistered(Context context);
	
	/**
	 * <p>Called when an error occurs during registration, unregistration, notification receiving or opening. Invoked on 
	 * <code>com.infobip.push.intent.ERROR</code> action.</p>
	 * @param context Context initialized with <code>PushNotificationManager</code> class.
	 * @param reason Reason why error occurred will be one of the following: <br />
	 * {@link PushNotificationManager#OPERATION_FAILED} - Operation failed.<br />
	 * {@link PushNotificationManager#INTERNET_NOT_AVAILABLE} - Internet access is unavailable.<br />
	 * {@link PushNotificationManager#PUSH_SERVICE_NOT_AVAILABLE} - Infobip Push service is unavailable.<br />
	 * {@link PushNotificationManager#USER_NOT_REGISTERED} - User is not registered.<br />
	 * {@link PushNotificationManager#GCM_ACCOUNT_MISSING} - User account is missing.<br />
	 * {@link PushNotificationManager#GCM_AUTHENTICATION_FAILED} - Authentication failed.<br />
	 * {@link PushNotificationManager#GCM_INVALID_SENDER} - Sender is invalid.<br />
	 * {@link PushNotificationManager#GCM_PHONE_REGISTRATION_ERROR}- Phone registration error occurred.<br />
	 * {@link PushNotificationManager#GCM_INVALID_PARAMETERS} - Invalid parameters are sent.<br />
	 * {@link PushNotificationManager#GCM_SERVICE_NOT_AVAILABLE} - GCM service is unavailable.<br />
	 * {@link PushNotificationManager#LIBRARY_NOT_INITIALIZED} - Library is not initialized. <br />
	 * {@link PushNotificationManager#USER_ALREADY_REGISTERED} - Error occurs while performing registration if user is already registered.
	 * @see PushNotificationManager
	 */
	public abstract void onError(int reason, Context context);
}
