package com.infobip.push;

import java.lang.ref.WeakReference;
import java.util.List;

import android.Manifest.permission;
import android.content.Context;
import android.location.Location;
import android.text.TextUtils;

import com.infobip.push.lib.InternalReceiver;
import com.infobip.push.lib.LocationBootReceiver;
import com.infobip.push.lib.LocationService;
import com.infobip.push.lib.LocationUpdateService;
import com.infobip.push.lib.livegeo.LiveGeoIntentService;
import com.infobip.push.lib.livegeo.LiveGeoReceiver;
import com.infobip.push.lib.livegeo.ReceiveTransitionsIntentService;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.LiveGeoUtil;
import com.infobip.push.lib.util.LocationManagerUtil;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;

/**
 * <p>This class manages location retrieving, sending and live geo monitoring.</p>
 *
 */
public class PushLocationManager {
	
	private WeakReference<Context> context;
	private Prefs prefs;
	
	/**
	 * <p>Creates a new <code>PushLocationManager</code> object using the given context.</p>
	 * 
	 */
	public PushLocationManager(Context context) {
		this.context = new WeakReference<Context>(context);
		this.prefs = new Prefs(context.getApplicationContext());
	}
	
	/**
	 * <p>Gets context which was used to instantiate this class.</p>
	 * @return Context
	 */
	public Context getContext() {
		return context.get();
	}
	
	/**
	 * <p>Enables location sending and sends location to the Infobip Push service.</p>
	 * <p>In case you're using Push location service for obtaining user's location, this method will start location retrieving.</p>
	 * @throws LocationConfigurationException Occurs when location feature isn't properly configured.
	 * @see #setLocationUpdateTimeInterval(int)
	 * @see #useCustomLocationService(boolean)
	 * @see #saveUserLocation(Location)
	 */
	public void enableLocation() {
		if (TextUtils.isEmpty(prefs.getRegistrationId())) {
			Util.LogWarn("Unable to start location service, user is not registered for push.");
			return;
		}
		
		if (isLocationEnabled()) {
			Util.LogDebug("Location service is already enabled.");
			return;
		}
		
		checkLocationManifest();
		
		Util.LogDebug("Enabling location service...");
		prefs.setLocationEnabled(true);
		prefs.save();
				
		LocationManagerUtil.getInstance(getContext()).startUpdateAlarm(prefs);
	}
	
	/**
	 * Checks for required location service configuration and throws {@link PushConfigurationException} if something is missing.
	 */
	private void checkLocationManifest() {
		Util.checkLocationRequiredPermission(permission.ACCESS_FINE_LOCATION, context.get());
		
		Util.checkLocationRequiredService(LocationUpdateService.class.getCanonicalName(), context.get());
		if (!prefs.isUsingCustomLocationService()) {
			Util.checkLocationRequiredService(LocationService.class.getCanonicalName(), context.get());
		}
		
		Util.checkLocationRequiredReceiver(LocationBootReceiver.class.getCanonicalName(), context.get());
		Util.checkLocationRequiredReceiver(InternalReceiver.class.getCanonicalName(), context.get());
	}

	/**
	 * <p>Disables Push location service from retrieving and sending location to the Infobip Push service.</p>
	 */
	public void disableLocation() {
		if (!isLocationEnabled()) {
			Util.LogDebug("Location service is already disabled.");
			return;
		}
		
		Util.LogDebug("Disabling location service...");
		prefs.setLocationEnabled(false);
		prefs.save();

		LocationManagerUtil.getInstance(getContext()).cancelUpdateAlarm(prefs);
	}
	
	/**
	 * <p>Checks if the location sending is enabled.</p>
	 * @return <code>true</code> if the location is enabled, <code>false</code> otherwise.
	 * @see #enableLocation()
	 * @see #disableLocation()
	 */
	public boolean isLocationEnabled() {
		return prefs.isLocationEnabled();
	}
	
	/**
	 * <p>Sets interval for sending location updates to the Infobip Push service in minutes. Call this method prior to calling {@link #enableLocation()}
	 * that starts location sending.</p>
	 * @param minutes Number of minutes to wait for the location update. 
	 */
	public void setLocationUpdateTimeInterval(int minutes) {
		if (minutes < Consts.MINIMUM_LOCATION_UPDATE_INTERVAL_MINUTES) {
			throw new IllegalArgumentException("Interval for sending location updates cannot be less than " + Consts.MINIMUM_LOCATION_UPDATE_INTERVAL_MINUTES);
		}
		
		prefs.setLocationUpdateTimeInterval(minutes);
		prefs.save();
	}
	
	/**
	 * <p>Gets interval for sending location updates in minutes.</p>
	 * @return Number of minutes to wait for the location update. 
	 * @see #setLocationUpdateTimeInterval(int)
	 */
	public int getLocationUpdateTimeInterval() {
		return prefs.getLocationUpdateTimeInterval();
	}
	
	/**
	 * <p>Overrides default location service usage, and specifies that you use your custom location service.</p>
	 * <p>For providing location to Infobip with your own service, use {@link #saveUserLocation(Location)} method each time you want the location to be saved.
	 * Push library will send your location periodically or in the interval you defined with {@link #setLocationUpdateTimeInterval(int)} method.</p>
	 * <p>Call this method prior to {@link #enableLocation()}.</p>
	 * @param useCustomService Set to <code>true</code> if you want to obtain location on your own with your own location service, 
	 * <code>false</code> if you want Push library to handle it.
	 * @throws LocationConfigurationException Occurs when library refuses to use your location service because retrieving location is already in use by Infobip
	 * location service. Requires {@link #isLocationEnabled()} to be <code>false</code>.
	 */
	public void useCustomLocationService(boolean useCustomService) {
		if (isLocationEnabled() && !isUsingCustomLocationService() && useCustomService) {
			throw new LocationConfigurationException("Unable to use your location service since using Push location service" +
					" is enabled. You have to disable it using disableLocation() method.");
		}
		
		prefs.setUsingCustomLocationService(useCustomService);
		prefs.save();
	}
	
	/**
	 * <p>Checks if you're using your own location service.</p>
	 * @return <code>true</code> if you're using your custom made location service
	 * @see #useCustomLocationService(boolean)
	 */
	public boolean isUsingCustomLocationService() {
		return prefs.isUsingCustomLocationService();
	}
	
	/**
	 * <p>Saves provided location acquired by your own location service. Location parameters which will be sent to Infobip in the default or custom interval are 
	 * latitude and longitude. To set the custom interval for location sending use {@link #setLocationUpdateTimeInterval(int)} method.</p>
	 * @param location Location to be saved. 
	 * @throws LocationConfigurationException Occurs when library refuses to save your location because retrieving location is already in use by Infobip
	 * location service. Requires {@link #useCustomLocationService(boolean)} to be <code>true</code>.
	 * @see #useCustomLocationService(boolean)
	 */
	public void saveUserLocation(Location location) {
		if (!isUsingCustomLocationService()) {
			throw new LocationConfigurationException("Unable to save location because retrieving location with push service is enabled. Call " +
					"useCustomLocationService(boolean) method for providing your own location.");
		}
		
		Util.LogDebug("Saving provided location to Push library");
		prefs.setLocationLatitude(String.valueOf(location.getLatitude()));
		prefs.setLocationLongitude(String.valueOf(location.getLongitude()));
		prefs.setLocationTimeSaved(System.currentTimeMillis());
		prefs.save();
	}
	
	/**
	 * <p>Enables live geo feature which is disabled by default and schedules daily updates of live geo notifications to sync Push service with Android library.</p>
	 * <p>{@link PushLocationManager#isLocationEnabled()} doesn't need to be true for live geo feature to function.</p>
	 * @throws LocationConfigurationException Occurs when live geo feature isn't properly configured.
	 * @see #stopMonitoringLiveGeoAreas()
	 * @see #getActiveLiveGeoAreasNumber()
	 * @since 1.3.0
	 */
	public void enableLiveGeo() {
		if (isLiveGeoEnabled()) {
			return;
		}
		
		checkLiveGeoManifest();
		
		Util.LogDebug("Live geo feature enabled");
		prefs.setLiveGeoEnabled(true);
		prefs.save();
		
		LocationManagerUtil.getInstance(getContext()).startRequestingLiveGeosAlarm();
	}
	
	/**
	 * Checks for required live geo configuration and throws {@link PushConfigurationException} if something is missing.
	 */
	private void checkLiveGeoManifest() {
		Util.checkLocationRequiredPermission(permission.ACCESS_FINE_LOCATION, context.get());
		
		Util.checkLocationRequiredService(LiveGeoIntentService.class.getCanonicalName(), context.get());
		Util.checkLocationRequiredService(ReceiveTransitionsIntentService.class.getCanonicalName(), context.get());
		
		Util.checkLocationRequiredReceiver(LiveGeoReceiver.class.getCanonicalName(), context.get());
		Util.checkLocationRequiredReceiver(LocationBootReceiver.class.getCanonicalName(), context.get());
		Util.checkLocationRequiredReceiver(InternalReceiver.class.getCanonicalName(), context.get());
	}

	/**
	 * <p>Disables live geo feature and cancels daily live geo updates. Once you disable it, your end user won't be able to receive live geo notifications.</p> 
	 * @since 1.3.0
	 */
	public void disableLiveGeo() {
		if (!isLiveGeoEnabled()) {
			return;
		}
		
		Util.LogDebug("Live geo feature disabled");
		prefs.setLiveGeoEnabled(false);
		prefs.save();
		
		stopMonitoringLiveGeoAreas();
	}
	
	/**
	 * <p>Checks if the live geo feature enabled.</p>
	 * @return <code>true</code> if live geo is enabled, <code>false</code> otherwise.
	 * @see #enableLiveGeo()
	 * @see #disableLiveGeo()
	 * @since 1.3.0
	 */
	public boolean isLiveGeoEnabled() {
		return prefs.isLiveGeoEnabled();
	}
	
	/**
	 * <p>Stops monitoring all live geo areas. Once you stop their monitoring, those areas are cleared and you cannot start monitoring them again.</p>
	 * @return The number of stopped live geo areas.
	 * @since 1.3.0
	 */
	public int stopMonitoringLiveGeoAreas() {
		List<String> geofencesToRemove = prefs.getActiveAreaIds();
		
		if (!geofencesToRemove.isEmpty()) {
			LiveGeoUtil.removeLiveGeosFromPrefs(prefs, geofencesToRemove);
			LiveGeoUtil.removeLiveGeosFromPlayServices(getContext(), geofencesToRemove);
			
			LocationManagerUtil.getInstance(getContext()).cancelRequestingLiveGeosAlarm();
		}
		
		return geofencesToRemove.size();
	}
	
	/**
	 * <p>Gets the number of active live geo areas. Active areas are ones monitored by your application.</p>
	 * @return The number of active live geo areas.
	 * @since 1.3.0
	 */
	public int getActiveLiveGeoAreasNumber() {
		return prefs.getActiveAreaIds().size();
	}
	
}
