package com.infobip.push;

import android.content.Intent;
import android.os.Parcel;
import android.os.Parcelable;

import com.infobip.push.lib.InfobipPushNotification;


/**
 * <p>Class that contains information about push notification object. Its instance is created in library when a notification is received,
 * and is {@link Parcelable} since 1.3.0 version, so it can be transmitted through the {@link Intent#putExtra(String, Parcelable)}.</p>
 * 
 * <p>PushNotification instance of MIME type <code>text/plain</code> contains id, mimeType, title, message, sound, vibrate, lights and additionalInfo
 * fields. Additionally, push with <code>text/html</code> MIME type will have a URL field sent via JSON.</p>
 */
public class PushNotification extends InfobipPushNotification {
	
	/**
	 * <p>Empty constructor.</p>
	 */
	public PushNotification() {
		
	}
	
	/**
	 * <p>Constructor for transmitting the {@link Parcelable} {@link PushNotification} object. It is invoked by {@link Parcelable.Creator#createFromParcel(Parcel)}.</p>
	 * @param in
	 */
	protected PushNotification(Parcel in) {
		super(in);
	}

	/**
	 * <p>Gets push notification identifier.</p>
	 * @return Identifier of the received notification.
	 */
	public String getId() {
		return id;
	}
	
	/**
	 * <p>Gets notification id used for displaying Notification object. <br />
	 * Using this id you can update or cancel notifications calling {@link NotificationManager#notify(int, Notification)}.</p>
	 * @return Notification id used for displaying Notification object. <br />-1 if the notification isn't handled by library.  
	 */
	public int getNotificationId() {
		return notificationId;
	}
	
	/**
	 * <p>Gets push notification MIME type.</p>
	 * @return 
	 * 		<code>text/plain</code> - Plain push notification with title and message sent via REST API. <br />
	 * 		<code>text/html</code> - Push notification with title, message and URL sent via REST API.
	 */
	public String getMimeType() {
		return mimeType;
	}

	/**
	 * @deprecated As of release 1.1.0, application/json MIME type isn't supported anymore. Notifications sent via REST API will be ignored. 
	 * Instead, use <code>additionalInfo</code> field from <code>text/plain</code> or <code>text/html</code> MIME type.
	 * <p>Gets push notification JSON message body.</p>
	 * @return JSON message of the received notification. This method will return an empty string.
	 */
	@Deprecated
	public String getJsonMessage() {
		return "";
	}
	
	/**
	 * <p>Gets push notification title.</p>
	 * @return Title of the received notification.
	 */
	public String getTitle() {
		return title;
	}
	
	/**
	 * <p>Gets push notification message body.</p>
	 * @return Message of the received notification.
	 */
	public String getMessage() {
		return message;
	}
	
	/**
	 * <p>Gets push notification URL.</p>
	 * @return URL of the received notification.
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * <p>Checks if a notification should make a sound.</p>
	 * @return <code>true</code> if sound will be played, <code>false</code> otherwise.
	 */
	public boolean makeSound() {
		return sound;
	}

	/**
	 * <p>Checks the vibration value.</p>
	 * <p>Requires permission: VIBRATE</p>
	 * @return <code>true</code> if vibration will be executed, <code>false</code> otherwise.
	 */
	public boolean vibrate() {
		return vibrate;
	}

	/**
	 * <p>Checks the lights value.</p>
	 * @return <code>true</code> if lights will be turned on, <code>false</code> otherwise.
	 */
	public boolean showLights() {
		return lights;
	}

	/**
	 * <p>Gets additional information sent in the received notification.</p>
	 * @return Additional information.
	 */
	public String getAdditionalInfo() {
		return additionalInfo;
	}
	
	/**
	 * <p>Gets media data sent in the received notification.</p>
	 * @return Media data.
	 * @since 1.2.0
	 */
	public String getMediaData() {
		return mediaData;
	}
	
	@Override
	public String toString() {		
		return "ID: " + id + " | " +
				"notification ID: " + notificationId + " | " +
				"MIME type: " + mimeType + " | " +
				"message: " + message + " | " +
				"title: " + title + " | " +
				"URL: " + url + " | " +
				"sound: " + sound + " | " +
				"vibrate: " + vibrate + " | " +
				"light: " + lights + " | " +
				"additionalInfo: " + additionalInfo + " | " +
				"media data: " + mediaData;
	}
	
	public static final Parcelable.Creator<PushNotification> CREATOR = new Parcelable.Creator<PushNotification>() {
		public PushNotification createFromParcel(Parcel in) {
			return new PushNotification(in);
		}

		public PushNotification[] newArray(int size) {
			return new PushNotification[size];
		}
	};
	
}
