package com.infobip.push;

import java.util.Calendar;

import android.app.Notification;
import android.content.Context;
import android.graphics.Color;
import android.text.TextUtils;
import android.widget.RemoteViews;

import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;


/**
 * <p>Use this class to customize the status bar notification, the remote view displayed in the notification drawer and default actions 
 * (sound, vibrate, light) performed when notification is received.</p>
 * 
 * @see Notification
 * @see RemoteViews
 */
public class PushNotificationBuilder {
	
	private Context context;
	private Prefs prefs;
	
	public static final int UNSET 		= -1;
	public static final int DISABLED 	= 0;
	public static final int ENABLED 	= 1;
	
	/**
	 * <p>Creates a new <code>PushNotificationBuilder</code> object using the given context.</p>
	 * 
	 */
	public PushNotificationBuilder(Context context) {
		this.context = context;
		this.prefs = new Prefs(context);
	}
	
	/**
	 * <p>Creates a new <code>PushNotificationBuilder</code> object using icon and ticker text displayed in the status bar.</p>
	 * @param iconDrawableId The resource id of a drawable to use as the icon in the status bar. This is required; notifications with an invalid icon resource 
	 * will not be shown.
	 * @param tickerText Text to scroll across the screen when this item is added to the status bar on large and smaller devices.
	 * @param context Given context.
	 * @see Notification
	 */
	public PushNotificationBuilder(int iconDrawableId, String tickerText, Context context) {
		this.context = context;
		this.prefs = new Prefs(context);
		setIconDrawableId(iconDrawableId);
		setTickerText(tickerText);
	}

	/**
	 * <p>Gets the notification icon displayed in the status bar.</p>
	 * @return The resource id of a drawable to use as the icon in the status bar. If {@link #UNSET}, -1 is returned.
	 * @see Notification#icon
	 */
	public int getIconDrawableId() {
		return prefs.getIconDrawableId();
	}

	/**
	 * <p>Sets notification icon. This icon will be displayed in the status bar.</p>
	 * @param iconDrawableId The resource id of a drawable to use as the icon in the status bar. This is required; notifications with an invalid icon resource 
	 * will not be shown.
	 * @see Notification#icon
	 */
	public void setIconDrawableId(int iconDrawableId) {	
		prefs.setIcon(iconDrawableId);
		prefs.save();
	}
	
	/**
	 * <p>Checks if the status bar notification icon is set.</p>
	 * @return <code>true</code> if notification icon is set, <code>false</code> if not.
	 * @see Notification#icon
	 */
	public boolean hasIconDrawableId() {
		return prefs.getIconDrawableId() != UNSET;
	}

	/**
	 * <p>Gets the notification ticker text displayed in the status bar.</p>
	 * @return Text to scroll across the screen when this item is added to the status bar on large and smaller devices.
	 * @see Notification#tickerText
	 */
	public String getTickerText() {
		return prefs.getTickerText();
	}

	/**
	 * <p>Sets ticker text. The text will be displayed in the status bar.</p>
	 * @param tickerText Text to scroll across the screen when this item is added to the status bar on large and smaller devices.
	 * @see Notification#tickerText
	 */
	public void setTickerText(String tickerText) {	
		prefs.setTickerText(tickerText);
		prefs.save();
	}

	/**
	 * <p>Gets the id of the layout resource file.</p>
	 * @return The resource id of the layout file to use for a remote view display in the notification drawer. If {@link #UNSET}, -1 is returned.
	 */
	public int getLayoutId() {
		return prefs.getLayoutId();
	}

	/**
	 * <p>Sets the id of the layout resource file.</p>
	 * @param layoutId The id of the layout resource to use for a remote view display in the notification drawer.
	 */
	public void setLayoutId(int layoutId) {	
		prefs.setLayoutId(layoutId);
		prefs.save();
	}
	
	/**
	 * <p>Checks if the id of the layout resource file to use for a remote view display is set.</p>
	 * @return <code>true</code> if the remote view layout id is set, <code>false</code> if not.
	 */
	public boolean hasLayoutId() {
		return prefs.getLayoutId() != UNSET;
	}

	/**
	 * <p>Gets the resource id of the 'title' view.</p>
	 * @return The resource id of the 'title' view to use for a remote view display in the notification drawer. If {@link #UNSET}, -1 is returned.
	 */
	public int getTitleId() {
		return prefs.getTitleId();
	}

	/**
	 * <p>Sets the resource id of the 'title' view.</p>
	 * @param titleId The resource id of the 'title' view to use for a remote view display in the notification drawer.
	 */
	public void setTitleId(int titleId) {	
		prefs.setTitleId(titleId);
		prefs.save();
	}

	/**
	 * <p>Checks if the resource id of the 'title' view is set.</p>
	 * @return <code>true</code> if the remote view 'title' id is set, <code>false</code> if not.
	 */
	public boolean hasTitleId() {
		return prefs.getTitleId() != UNSET;
	}

	/**
	 * <p>Gets application name from your <code>AndroidManifest.xml</code> file. This value will be overridden if you {@link #setApplicationName(String)} 
	 * to the desired value.</p>
	 * @return Application name shown in the notification drawer as a title if the title isn't sent via push notification.
	 */
	public String getApplicationName() {
		String appNamePrefs = prefs.getApplicationName();
		
		if (!TextUtils.isEmpty(appNamePrefs)) {
			return appNamePrefs;
		}
		
		return Util.getAppName(context);
	}

	/**
	 * <p>Sets application name.</p>
	 * @param applicationName Application name shown in the notification drawer as a title if the title isn't sent via push notification.
	 * @throws IllegalArgumentException Application name cannot be an empty string or null.
	 */
	public void setApplicationName(String applicationName) {
		
		if (TextUtils.isEmpty(applicationName)) {
			throw new IllegalArgumentException("Application name cannot be an empty string or null."); 
		}
		
		prefs.setApplicationName(applicationName);
		prefs.save();
	}
	
	/**
	 * <p>Gets the resource id of the 'text' view.</p>
	 * @return The resource id of the 'text' view to use for a remote view display in the notification drawer. If {@link #UNSET}, -1 is returned.
	 */
	public int getTextId() {
		return prefs.getTextId();
	}

	/**
	 * <p>Sets the resource id of the 'text' view.</p>
	 * @param textId The resource id of the 'text' view to use for a remote view display in the notification drawer.
	 */
	public void setTextId(int textId) {	
		prefs.setTextId(textId);
		prefs.save();
	}
	
	/**
	 * <p>Checks if the resource id of the 'text' view is set.</p>
	 * @return <code>true</code> if the remote view 'text' id is set, <code>false</code> if not.
	 */
	public boolean hasTextId() {
		return prefs.getTextId() != UNSET;
	}

	/**
	 * <p>Gets the resource id of the image view.</p>
	 * @return The resource id of the image view to use for a remote view display in the notification drawer. If {@link #UNSET}, -1 is returned.
	 */
	public int getImageId() {
		return prefs.getImageId();
	}

	/**
	 * <p>Sets the resource id of the image view.</p>
	 * @param imageId The resource id of the image view to use for a remote view display in the notification drawer.
	 */
	public void setImageId(int imageId) {	
		prefs.setImageId(imageId);
		prefs.save();
	}
	
	/**
	 * <p>Checks if the id of the view used for displaying image in the remote view is set.</p>
	 * @return <code>true</code> if the remote view image id is set, <code>false</code> if not.
	 */
	public boolean hasImageId() {
		return prefs.getImageId() != UNSET;
	}
	
	/**
	 * <p>Gets the resource id of a drawable image.</p>
	 * @return The resource id of a drawable to use as the image in the remote view. If {@link #UNSET}, -1 is returned.
	 */
	public int getImageDrawableId() {
		return prefs.getImageDrawableId();
	}

	/**
	 * <p>Sets the resource id of a drawable image.</p>
	 * @param imageDrawableId The resource id of a drawable to use as the image in the remote view.
	 */
	public void setImageDrawableId(int imageDrawableId) {	
		prefs.setImageDrawableId(imageDrawableId);
		prefs.save();
	}
	
	/**
	 * <p>Checks if the resource id of a drawable to use as the image in the remote view is set.</p>
	 * @return <code>true</code> if the remote view image id is set, <code>false</code> if not.
	 */
	public boolean hasImageDrawableId() {
		return prefs.getImageDrawableId() != UNSET;
	}
	
	/**
	 * <p>Gets the resource id of the 'date' view.</p>
	 * @return The resource id of the 'date' view to use for a remote view display in the notification drawer. If {@link #UNSET}, -1 is returned.
	 */
	public int getDateId() {
		return prefs.getDateId();
	}
	
	/**
	 * <p>Sets the resource id of the 'date' view.</p>
	 * @param dateId The resource id of the 'date' view to use for a remote view display in the notification drawer.
	 */
	public void setDateId(int dateId) {	
		prefs.setDateId(dateId);
		prefs.save();
	}
	
	/**
	 * <p>Checks if the id of the view used for displaying date in the remote view is set.</p>
	 * @return <code>true</code> if the remote view date id is set, <code>false</code> if not.
	 */
	public boolean hasDateId() {
		return prefs.getDateId() != UNSET;
	}

	/**
	 * <p>Gets sound value.</p>
	 * @return <ul><li>-1 if the sound is {@link #UNSET}</li>
	 * <li>0 if the sound is {@link #DISABLED}</li>
	 * <li>1 if the sound is {@link #ENABLED}</li> </ul>
	 */
	public int getSound() {
		return prefs.getSound();
	}
	
	/**
	 * <p>Sets sound.</p>
	 * <p>Override the default ringtone by using {@link #setSoundResourceId(int)}.</p>
	 * <p><b>Note:</b> User's phone will never make sound if the default ringtone is silent or if the notification is received in the quiet time.<br />
	 * @param sound Can be one of the following values: <br />
	 * {@link #UNSET} - takes sound value from the received {@link PushNotification}, <br />
	 * {@link #ENABLED} - every push notification will make sound, <br /> 
	 * {@link #DISABLED} - no push notification will make sound.
	 * 
	 * @see #setQuietTimeEnabled(boolean)
	 * @see #setQuietTime(int, int, int, int)
	 */
	public void setSound(int sound) {
		prefs.setSound(sound);
		prefs.save();
	}

	/**
	 * <p>Checks if the sound value is set.</p>
	 * @return <code>true</code> if sounds are {@link #ENABLED} or {@link #DISABLED}, <code>false</code> if {@link #UNSET}
	 */
	public boolean hasSound() {
		return prefs.getSound() != UNSET;
	}

	/**
	 * <p>Sets sound to play.</p>
	 * @param resourceId The resource ID of your sound to play when the notification is received.
	 * @see Notification#sound
	 */
	public void setSoundResourceId(int resourceId) {
		prefs.setSoundResourceId(resourceId);
		prefs.save();
	}
	
	/**
	 * <p>Gets the resource ID of the sound to play.</p>
	 * @return Resource ID of the sound.
	 */
	public int getSoundResourceId() {		
		return prefs.getSoundResourceId();
	}
	
	
	/**
	 * <p>Gets the vibration value.</p>
	 * @return <ul><li>-1 if vibrating is {@link #UNSET}</li>
	 * <li>0 if vibrating is {@link #DISABLED}</li>
	 * <li>1 if vibrating is {@link #ENABLED}</li> </ul>
	 */
	public int getVibration() {
		return prefs.getVibration();
	}
	
	/**
	 * <p>Sets vibration.</p>
	 * <p>Customize the vibration pattern by using {@link #setVibrationPattern(long[])}.</p>
	 * <p><b>Note:</b> User's phone will never vibrate if he/she disabled vibration in system settings or if the notification is received 
	 * in the quiet time. </p>
	 * <p>Requires permission: VIBRATE</p>
	 * @param vibration Can be one of the following values: <br />
	 * {@link #UNSET} - takes vibration value from the received {@link PushNotification}, <br />
	 * {@link #ENABLED} - every push notification will vibrate, <br /> 
	 * {@link #DISABLED} - no push notification will vibrate.
	 * 
	 * @see #setQuietTimeEnabled(boolean)
	 * @see #setQuietTime(int, int, int, int)
	 */
	public void setVibration(int vibration) {		
		prefs.setVibration(vibration);
		prefs.save();
	}

	/**
	 * <p>Checks if the vibration is set.</p>
	 * @return <code>true</code> if vibrating is {@link #ENABLED} or {@link #DISABLED}, <code>false</code> if {@link #UNSET}
	 */
	public boolean hasVibration() {
		return prefs.getVibration() != UNSET;
	}

	/**
	 * <p>Sets the vibration pattern.</p>
	 * <p>Pass in an array of longs that are the durations for which to turn on or off the vibrator in milliseconds. The first value indicates 
	 * the number of milliseconds to wait before turning the vibrator on. The next value indicates the number of milliseconds for which to keep 
	 * the vibrator on before turning it off. Subsequent values alternate between durations in milliseconds to turn the vibrator off or to turn 
	 * the vibrator on.</p>
	 * @param pattern The pattern with which to vibrate.
	 * @see #setVibration(int)
	 * @see Notification#vibrate
	 */
	public void setVibrationPattern(long[] pattern) {
		prefs.setVibrationPattern(pattern);
		prefs.save();
	}

	/**
	 * <p>Gets the vibration pattern.</p>
	 * @return An empty array if the pattern is unset.
	 */
	public long[] getVibrationPattern() {		
		return prefs.getVibrationPattern();
	}

	/**
	 * <p>Gets the light value.</p>
	 * @return <ul><li>-1 if lights are {@link #UNSET}</li>
	 * <li>0 if lights are {@link #DISABLED}</li>
	 * <li>1 if lights are {@link #ENABLED}</li></ul>
	 */
	public int getLight() {
		return prefs.getLight();
	}
	
	/**
	 * <p>Sets lights.</p>
	 * <p>For managing LED light color use {@link #setLightsColor(int)}. Customize the number of milliseconds for 
	 * the LED to be on and off while it's flashing by using {@link #setLightsOnOffMS(int, int)}.</p>
	 * <p><b>Note:</b> Mobile phones without LED lights won't make lights. Mobile phones with LED lights won't make lights if the notification 
	 * is received in the quiet time.</p>
	 * @param lights Can be one of the following values: <br />
	 * {@link #UNSET} - takes lights value from the received {@link PushNotification}, <br />
	 * {@link #ENABLED} - every push notification will flash lights, <br /> 
	 * {@link #DISABLED} - no push notification will flash lights.
	 * 
	 * @see #setQuietTimeEnabled(boolean)
	 * @see #setQuietTime(int, int, int, int)
	 */
	public void setLight(int lights) {
		prefs.setLight(lights);
		prefs.save();
	}
	
	/**
	 * <p>Checks if the light value is set.</p>
	 * @return <code>true</code> if lights are {@link #ENABLED} or {@link #DISABLED}, <code>false</code> if {@link #UNSET}
	 */
	public boolean hasLight() {
		return prefs.getLight() != UNSET;
	}
	
	/**
	 * Sets the color of the LED light. The hardware will do its best approximation.
	 * @param lightsColor LED color value in integer. You can use {@link Color} class to choose one of the colors.
	 * @see Notification#ledARGB
	 */
	public void setLightsColor(int lightsColor) {		
		prefs.setLightsColor(lightsColor);
		prefs.save();
	}

	/**
	 * Gets the color of the LED light.
	 * @return The color of the LED in integer. -1 if the color is {@link #UNSET}.
	 */
	public int getLightsColor() {		
		return prefs.getLightsColor();
	}
	
	/**
	 * Sets the number of milliseconds for the LED to be on and off while it's flashing. The hardware will do its best approximation.
	 * @param lightsOnMS The number of milliseconds for the LED to be on while it's flashing.
	 * @param lightsOffMS The number of milliseconds for the LED to be off while it's flashing.
	 * @see Notification#ledOnMS
	 * @see Notification#ledOffMS
	 */
	public void setLightsOnOffMS(int lightsOnMS, int lightsOffMS) {	
		prefs.setLightsOnMS(lightsOnMS);
		prefs.setLightsOffMS(lightsOffMS);
		prefs.save();
	}
	
	/**
	 * Gets the number of milliseconds for the LED to be on while it's flashing.
	 * @return The number of milliseconds for the LED to be on while it's flashing. -1 if the number is {@link #UNSET}.
	 */
	public int getLightsOnMS() {
		return prefs.getLightsOnMS();
	}
	
	/**
	 * Gets the number of milliseconds for the LED to be off while it's flashing.
	 * @return The number of milliseconds for the LED to be off while it's flashing. -1 if the number is {@link #UNSET}.
	 */
	public int getLightsOffMS() {		
		return prefs.getLightsOffMS();
	}

	/**
	 * <p>Clears saved preferences used in this class.</p>
	 */
	public void removeSavedData() {
		prefs.removeNotificationBuilderData();
	}
	
	/**
	 * <p>Enables/disables quiet time.</p> 
	 * <p>Quiet time is an interval set with {@link #setQuietTime(int startHour, int startMinute, int endHour, int endMinute)} that determines the time
	 * when the sound, vibration and flashing lights won't perform on the notification receival.</p>
	 * 
	 * @param enabled <code>true</code> if quiet time is enabled, <code>false</code> if quiet time is disabled.
	 * @see #setQuietTime(int, int, int, int)
	 */
	public void setQuietTimeEnabled(boolean enabled) {
		prefs.setQuietTime(enabled);
		prefs.save();
	}
	
	/**
	 * <p>Checks if quiet time is enabled.</p>
	 * @return <code>true</code> if quiet time is enabled, <code>false</code> if quiet time is disabled or not set.
	 * @see #setQuietTimeEnabled(boolean)
	 */
	public boolean isQuietTimeEnabled() {
		return prefs.isQuietTime();
	}
	
	/**
	 * <p>Sets start hour and minute, end hour and minute of quiet time.</p>
	 * @throws IllegalArgumentException Hours must be in 24 hour time format, minutes in 0 to 59 interval.
	 * @param startHour Hour that determines the start of quiet time.
	 * @param startMinute Minute of hour that determines the start of quiet time.
	 * @param endHour Hour that determines the end of quiet time.
	 * @param endMinute Minute of hour that determines the end of quiet time.
	 * @see #setQuietTimeEnabled(boolean)
	 * @see #isInQuietTime()
	 */
	public void setQuietTime(int startHour, int startMinute, int endHour, int endMinute) {
		
		if (	startHour < 0 	|| startHour >= 24 	||
				endHour < 0 	|| endHour >= 24 ||
				startMinute < 0 || startMinute >= 60 ||
				endMinute < 0 	|| endMinute >= 60) {
			throw new IllegalArgumentException("Invalid date parameters entered. Please enter hours in 24 hour time format, and minutes in 0 to 59 interval.");
		}

		prefs.setStartQuietHour(startHour);
		prefs.setStartQuietMinute(startMinute);
		prefs.setEndQuietHour(endHour);
		prefs.setEndQuietMinute(endMinute);
		prefs.save();
	}
	
	/**
	 * <p>Gets hour that defines the start of quiet time.</p>
	 * @return Start quiet hour.
	 * @see #setQuietTime(int, int, int, int)
	 */
	public int getStartHour() {
		return prefs.getStartQuietHour();
	}
	
	/**
	 * <p>Gets minute that defines the start of quiet time.</p>
	 * @return Start quiet minute.
	 * @see #setQuietTime(int, int, int, int)
	 */
	public int getStartMinute() {
		return prefs.getStartQuietMinute();
	}
	
	/**
	 * <p>Gets hour that defines the end of quiet time.</p>
	 * @return End quiet hour.
	 * @see #setQuietTime(int, int, int, int)
	 */
	public int getEndHour() {
		return prefs.getEndQuietHour();
	}
	
	/**
	 * <p>Gets minute that defines the end of quiet time.</p>
	 * @return End quiet minute.
	 * @see #setQuietTime(int, int, int, int)
	 */
	public int getEndMinute() {
		return prefs.getEndQuietMinute();
	}
	
	/**
	 * <p>Checks if the application is in quiet time interval. Quiet time interval is the time between your start hour/minute and end hour/minute.</p>
	 * @return <code>true</code> if application is in quiet time interval, <code>false</code> otherwise.
	 * @see #setQuietTime(int, int, int, int)
	 */
	public boolean isInQuietTime() {
		
		boolean isInQuietTime = false;
		
		if (prefs.isQuietTime()){
			int startHour = prefs.getStartQuietHour();
			int startMinute = prefs.getStartQuietMinute();			
			int endHour = prefs.getEndQuietHour();
			int endMinute = prefs.getEndQuietMinute();
			
			Calendar startTime = Calendar.getInstance();
			startTime.set(Calendar.HOUR_OF_DAY, startHour);
			startTime.set(Calendar.MINUTE, startMinute);
			
			Calendar endTime = Calendar.getInstance();
			endTime.set(Calendar.HOUR_OF_DAY, endHour);
			endTime.set(Calendar.MINUTE, endMinute);	
			
			Calendar currentTime = Calendar.getInstance();
			
			if (startTime.after(currentTime) && endTime.before(startTime)) {
				startTime.add(Calendar.DAY_OF_YEAR, -1);
			}
			
			if (endTime.before(startTime)) {
				endTime.add(Calendar.DAY_OF_YEAR, 1);
			}
			
			if (currentTime.after(startTime) && currentTime.before(endTime)) {
				isInQuietTime = true;
			}
		}
		
		return isInQuietTime;
	}
}
