package com.infobip.push;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.Manifest.permission;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.text.TextUtils;

import com.infobip.push.lib.InfobipNotificationHandlingUtil;
import com.infobip.push.lib.InfobipPushReceiver;
import com.infobip.push.lib.InfobipPushService;
import com.infobip.push.lib.InternalReceiver;
import com.infobip.push.lib.SendPostMessage;
import com.infobip.push.lib.SendPostMessage.MessageSentListener;
import com.infobip.push.lib.model.ChannelRegistrationModel;
import com.infobip.push.lib.model.NotificationOpenedModel;
import com.infobip.push.lib.model.UserDataModel;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.LocationManagerUtil;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;

/**
 * <p>Methods for interacting with the Android Push library are located in this class.</p>
 * 
 */
public class PushNotificationManager {

	//---------------------------------- CONSTANTS ----------------------------------//
	
	private static final String URL_CHANNEL_REGISTRATION 	= Consts.URL_CHANNEL_REGISTRATION; 	// POST
	private static final String URL_REGISTERED_CHANNELS 	= Consts.URL_REGISTERED_CHANNELS; 	// GET
	private static final String URL_UNREAD_MESSAGES 		= Consts.URL_UNREAD_MESSAGES; 	// GET
	private static final String URL_NOTIFICATION_OPENED 	= Consts.URL_NOTIFICATION_OPENED; 	// PUT
	private static final String URL_SAVE_NEW_USER_DATA 		= Consts.URL_SAVE_NEW_USER_DATA; 	// PUT
	
	/**
	 * <p>Push library is by default configured to check manifest prior to calling methods for connecting with the Infobip Push service. 
	 * It is recommended to change this value to <code>false</code>, or disable debug mode with {@link #setDebugModeEnabled(boolean)} method before 
	 * sending your application to production.</p>
	 */
	public static boolean checkManifest = true;
	
	/**
	 * <p>Used as a reason when performed operation fails.<br /> For more details check your LogCat. Make sure that you've 
	 * {@link #setDebugModeEnabled(boolean)} to true.</p>
	 * Constant value: {@value}
	 */
	public static final int OPERATION_FAILED			= 12;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that the Internet access is unavailable.</p>
	 * Constant value: {@value}
	 */
	public static final int INTERNET_NOT_AVAILABLE 		= 1;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that the Infobip Push service is currently unavailable.</p>
	 * Constant value: {@value}
	 */
	public static final int PUSH_SERVICE_NOT_AVAILABLE 	= 2; 
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that the user is not registered.</p>
	 * Constant value: {@value}
	 */
	public static final int USER_NOT_REGISTERED 		= 4;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that the user account is missing.</p>
	 * Constant value: {@value}
	 */
	public static final int GCM_ACCOUNT_MISSING 		= 8;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that authentication failed.</p>
	 * Constant value: {@value}
	 */
	public static final int GCM_AUTHENTICATION_FAILED 	= 16;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that sender is invalid.</p>
	 * Constant value: {@value}
	 */
	public static final int GCM_INVALID_SENDER 			= 32;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that phone registration error occurred.</p>
	 * Constant value: {@value}
	 */
	public static final int GCM_PHONE_REGISTRATION_ERROR = 64;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that invalid parameters are sent.</p>
	 * Constant value: {@value}
	 */
	public static final int GCM_INVALID_PARAMETERS 		= 128;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that GCM service is unavailable.</p>
	 * Constant value: {@value}
	 */
	public static final int GCM_SERVICE_NOT_AVAILABLE 	= 256;
	
	/**
	 * <p>Used as a reason when performed operation fails. Means that the library is not initialized.</p>
	 * Constant value: {@value}
	 */
	public static final int LIBRARY_NOT_INITIALIZED 	= 512;
	
	/**
	 * <p>Used as a reason when the registration fails. Means that the user is already registered.</p>
	 * Constant value: {@value}
	 */
	public static final int USER_ALREADY_REGISTERED 	= 1024;
	
	//---------------------------------- CONSTANTS ----------------------------------//
	
	private WeakReference<Context> context;
	private Prefs prefs;

	/**
	 * <p>Creates a new <code>PushNotificationManager</code> object using the given context.</p>
	 * 
	 */
	public PushNotificationManager(Context context) {
		this.context = new WeakReference<Context>(context);
		this.prefs = new Prefs(context);
	}

	/**
	 * <p>Gets context which was used to instantiate this class.</p>
	 * @return Context
	 */
	public Context getContext() {
		return context.get();
	}

	/**
	 * <p>Initializes sender ID, application ID and application secret. It is mandatory <strong>before</strong> registration. Call it in your application's entry
	 * {@link Activity#onCreate()} method.</p>
	 * <p>This method generates user's own device ID that uniquely identifies him/her in the Infobip Push system. Device ID can be used to 
	 * send push notifications to a specific user. It is created only once (unless user clears application data - then it recreates).</p>
	 * <p>As of 1.2.0 release, this method performs following:</p>
	 * <ul>
	 * <li>in case your application is updated it refreshes existing registration on GCM. When refreshing registration succeeds 
	 * {@link AbstractPushReceiver#onRegistrationRefreshed(Context)} method will be called if you registered 
	 * <code>com.infobip.push.intent.REGISTRATION_REFRESHED</code> action in AndroidManifest.xml.</li>
	 * <li>if enabled, location updates are rescheduled in case they've been stopped by Android OS.</li>
	 * <li>checks if the time zone has been changed and updates it on the Infobip Push service. To disable automatic time zone updates, 
	 * call {@link #setTimeZoneAutomaticUpdateEnabled(boolean)} with the value <code>false</code>.</li>
	 * </ul>
	 * @param senderId Unique sender identifier provided by Google (your Project number).
	 * @param applicationId Unique application identifier provided by Infobip Push service.
	 * @param applicationSecret Application secret provided by Infobip Push service.
	 * @throws IllegalArgumentException Sender ID, application ID and application secret cannot be an empty string or null.
	 */
	public void initialize(String senderId, String applicationId, String applicationSecret) {
		
		if (	TextUtils.isEmpty(senderId) || 
				TextUtils.isEmpty(applicationId) || 
				TextUtils.isEmpty(applicationSecret)) {
			throw new IllegalArgumentException("Sender ID, application ID and application secret cannot be an empty string or null.");
		}
		
		prefs.setSenderId(senderId);
		prefs.setApplicationId(applicationId);
		prefs.setApplicationSecret(applicationSecret);
		
		if (TextUtils.isEmpty(prefs.getDeviceId())) {
			String deviceId = Util.generateDeviceId();
			prefs.setDeviceId(deviceId);
			
			Util.LogDebug("device ID: " + deviceId);
		}
		
		if (isRegistered()) {
			
			if (isRegistrationExpired()) {
				Util.LogDebug("Refreshing expired GCM registration");
				refreshGcmRegistration();
			}
			
			if (prefs.isLocationEnabled()) {
				LocationManagerUtil.getInstance(getContext()).rescheduleUpdateAlarmIfNecessary(prefs);
			}
		}

		int timeZoneOffset = prefs.getTimeZoneOffset();
		boolean timeZoneChanged = prefs.isTimeZoneChanged();
			
		if (prefs.isCheckingTimeZoneAutomatically()) {
			int automaticTimeZoneOffset = Util.getTimeZoneOffset(getContext());
			timeZoneChanged = timeZoneOffset != automaticTimeZoneOffset;
				
			timeZoneOffset = automaticTimeZoneOffset;
		}
			
		if (timeZoneChanged && isRegistered()) {
			saveNewTimeZoneOffset(timeZoneOffset);		
		} else if (timeZoneChanged) {
			prefs.setTimeZoneOffset(timeZoneOffset);
			prefs.setTimeZoneChanged(false);
		}
		
		if (TextUtils.isEmpty(prefs.getPackageName())) {
			prefs.setPackageName(getContext().getPackageName());
		}
		
		prefs.save();
	}

	/**
	 * Saves new time zone on the Infobip Push service.
	 * @param timeZoneOffset
	 */
	private void saveNewTimeZoneOffset(final int timeZoneOffset) {
		Util.LogDebug("Saving time zone offset (" + timeZoneOffset + " minutes) on the Infobip Push service.");
		UserDataModel userDataModel = new UserDataModel();
		userDataModel.timeZoneOffset = timeZoneOffset;
		
		SendPostMessage sendPost = new SendPostMessage(String.format(Consts.URL_SAVE_NEW_USER_DATA, prefs.getDeviceId()), 
				userDataModel.toJsonObject(), Consts.PUT);
		sendPost.doInBackground(context.get(), new MessageSentListener() {
			
			@Override
			public void onSendSuccess() {
				Util.LogDebug("Saved time zone offset: " + timeZoneOffset);
				prefs.setTimeZoneOffset(timeZoneOffset);
				prefs.setTimeZoneChanged(false);
				prefs.save();
			}
			
			@Override
			public void onSendFailed(int reason) {
				Util.LogError("Saving new time zone on Infobip Push service failed.");
			}				
		});
	}
	
	/**
	 * <p>This method allows you to manually set an offset from UTC for the time zone defined in minutes.</p>
	 * <p>Time zone is updated by default in {@link #initialize(String, String, String)} method when the time zone changes. 
	 * If you manually set the time zone, automatic checking for time zone changes will be disabled.</p>
	 * @param minutes Offset from UTC for the time zone. If you are, e.g., in Germany, this offset is 60 minutes (1 hour) without daylight savings time.
	 * @since 1.2.0
	 * @see #setTimeZoneAutomaticUpdateEnabled(boolean)
	 * @see #initialize(String, String, String)
	 */
	public void setTimeZoneOffset(int minutes) {
		if (prefs.getTimeZoneOffset() != minutes) {
			prefs.setTimeZoneChanged(true);
		} else {
			prefs.setTimeZoneChanged(false);
		}
		prefs.setCheckTimeZoneAutomatically(false);
		prefs.setTimeZoneOffset(minutes);
		prefs.save();
	}
	
	/**
	 * <p>Sets automatic time zone updates to be enabled or disabled according to time zone changes.</p>
	 * <p>Automatic updates of user's time zone are enabled by default. If you manually set the time zone offset with method {@link #setTimeZoneOffset(int)}, 
	 * automatic checking for time zone changes will be disabled.</p>
	 * @param isEnabled Indicates whether time zone automatic updates are enabled or disabled. This value is <code>true</code> by default.
	 * @since 1.2.0
	 * @see #setTimeZoneOffset(int)
	 * @see #initialize(String, String, String)
	 */
	public void setTimeZoneAutomaticUpdateEnabled(boolean isEnabled) {
		prefs.setCheckTimeZoneAutomatically(isEnabled);
		prefs.save();
	}

	/**
	 * <p>Gets sender ID, i.e., your Google project number, provided by Google.</p>
	 * @return Sender ID if the {@link #initialize(String, String, String)} method was called, if it wasn't called returns an empty string.
	 */
	public String getSenderId() {
		return prefs.getSenderId();
	}
	
	/**
	 * <p>Gets application ID provided by Infobip Push service.</p>
	 * @return Application ID if the {@link #initialize(String, String, String)} method was called, if it wasn't called returns an empty string.
	 */
	public String getApplicationId() {
		return prefs.getApplicationId();
	}

	/**
	 * <p>Gets application secret provided by Infobip Push service.</p>
	 * @return Application secret if the {@link #initialize(String, String, String)} method was called, if it wasn't called returns an empty string.
	 */
	public String getApplicationSecret() {
		return prefs.getApplicationSecret();
	}
	
	/**
	 * <p>Gets device ID generated by the library.</p>
	 * @return Device ID if the {@link #initialize(String, String, String)} method was called, if it wasn't called returns an empty string.
	 */
	public String getDeviceId() {
		return prefs.getDeviceId();
	}

	private void refreshGcmRegistration() {
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to refresh registration on GCM.");
			return;
		}		
		
		Intent refreshIntent = new Intent();
		refreshIntent.setAction(Consts.INFOBIP_REFRESH_REGISTRATION_ACTION);
		InfobipPushService.runIntentInService(context.get(), refreshIntent);		//call service class with com.push.REFRESH_REG action
	}

	/**
	 * <p>Registers to the Infobip Push service for receiving push notifications.</p>
	 * <p>When the registration succeeds intent with the <code>com.infobip.push.intent.REGISTERED_FOR_NOTIFICATIONS</code> action will be invoked and 
	 * {@link AbstractPushReceiver#onRegistered(Context)} will be executed. If the registration fails <code>com.infobip.push.intent.ERROR</code> action 
	 * will be invoked after which you can check the reason why the registration failed by extending {@link AbstractPushReceiver#onError(int, Context)} 
	 * method.</p>
	 * <p>Check if the user is registered by calling {@link #isRegistered()} method.</p>
	 * <p><strong>Note:</strong> Initialize your data with the {@link #initialize(String, String, String)} method prior to calling this method because 
	 * you won't be able to register for Infobip Push without your credentials being previously stored.</p>
	 * @see AbstractPushReceiver
	 */
	public void register() {
		
		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (!isLibraryInitialized()) {
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			Util.sendCustomBroadcast(context.get(), Consts.RECEIVER_ERROR_ACTION, LIBRARY_NOT_INITIALIZED);
			return;
		}
		
		if (isRegistered()) {
			Util.LogWarn("Not trying to register again, user is already registered.");
			Util.sendCustomBroadcast(context.get(), Consts.RECEIVER_ERROR_ACTION, USER_ALREADY_REGISTERED);
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to register.");
			Util.sendCustomBroadcast(context.get(), Consts.RECEIVER_ERROR_ACTION, INTERNET_NOT_AVAILABLE);
			return;
		}
		
		Intent registerIntent = new Intent();
		registerIntent.setAction(Consts.INFOBIP_REGISTER_ACTION);
		InfobipPushService.runIntentInService(context.get(), registerIntent);		//call service class with com.push.REGISTER action
	}
	
	/**
	 * <p>Registers to the Infobip Push service for receiving push notifications by providing {@link RegistrationData}. Registration data can be one of the 
	 * following: userId, channels, additionalInfo.</p>
	 * <p>When the registration succeeds intent with the <code>com.infobip.push.intent.REGISTERED_FOR_NOTIFICATIONS</code> action will be invoked and 
	 * {@link AbstractPushReceiver#onRegistered(Context)} will be executed. If the registration fails <code>com.infobip.push.intent.ERROR</code> action 
	 * will be invoked after which you can check the reason why the registration failed by extending {@link AbstractPushReceiver#onError(int, Context)} 
	 * method.</p>
	 * <p>Check if the user is registered by calling {@link #isRegistered()} method.</p>
	 * <p><strong>Note:</strong> Initialize your data with the {@link #initialize(String, String, String)} method prior to calling this method because 
	 * you won't be able to register for Infobip Push without your credentials being previously stored.</p>
	 * @param registrationData Registration data.
	 * @throws IllegalArgumentException Registration data cannot be null.
	 * @see RegistrationData
	 * @see AbstractPushReceiver
	 */
	public void register(RegistrationData registrationData) {
		if (registrationData == null || registrationData.getAdditionalInfo() == null || registrationData.getUserId() == null || registrationData.getChannels() == null) {
			throw new IllegalArgumentException("Registration data cannot be null.");
		}
		
		prefs.registrationDataToPrefs(registrationData);
		register();
	}

	/**
	 * <p>Gets registration data.</p>
	 * @return Registration data.
	 * @see RegistrationData
	 */
	public RegistrationData getRegistrationData() {
		return prefs.prefsToUserRelatedData();
	}

	/**
	 * <p>Unregisters from the Infobip Push service to stop receiving push notifications.</p>
	 * <p>When the unregistration succeeds intent with the <code>com.infobip.push.intent.UNREGISTERED_FROM_NOTIFICATIONS</code> action will be invoked and 
	 * {@link AbstractPushReceiver#onUnregistered(Context)} will be executed. If the unregistration fails <code>com.infobip.push.intent.ERROR</code> action 
	 * will be invoked after which you can check the reason why it failed by extending {@link AbstractPushReceiver#onError(int, Context)} method.</p>
	 * @see AbstractPushReceiver
	 */
	public void unregister() {
		
		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (!isLibraryInitialized()) {
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			Util.sendCustomBroadcast(context.get(), Consts.RECEIVER_ERROR_ACTION, LIBRARY_NOT_INITIALIZED);
			return;
		}
		
		if (!isRegistered()) {
			Util.LogWarn("User is not registered.");
			Util.sendCustomBroadcast(context.get(), Consts.RECEIVER_ERROR_ACTION, USER_NOT_REGISTERED);
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to unregister.");
			Util.sendCustomBroadcast(context.get(), Consts.RECEIVER_ERROR_ACTION, INTERNET_NOT_AVAILABLE);
			return;
		}
		
		Intent unregisterIntent = new Intent();
		unregisterIntent.setAction(Consts.INFOBIP_UNREGISTER_ACTION);
		InfobipPushService.runIntentInService(context.get(), unregisterIntent);		//call service class with com.push.UNREGISTER action
	}
	
	/**
	 * <p>Checks if the library is initialized.</p>
	 * @return <code>true</code> if the library is initialized, <code>false</code> if not.</p>
	 * @see #initialize(String, String, String)
	 */
	public boolean isLibraryInitialized() {

		if (TextUtils.isEmpty(prefs.getSenderId()) ||
				TextUtils.isEmpty(prefs.getApplicationId()) ||
				TextUtils.isEmpty(prefs.getApplicationSecret())) {
			return false;
		}
		return true;
	}
	
	/**
	 * <p>Checks whether user is registered for receiving push notifications.</p>
	 * <p>Calling this method right after {@link #unregister()} will probably return <code>true</code> (meaning your user is still registered)
	 * because unregistration hasn't finished yet. </p>
	 * <p>If you want to check when the unregistration (or registration) process has been finished, register {@link AbstractPushReceiver} dynamically 
	 * in your activity like this:</p>
	 * <pre>
	 * {@code BroadcastReceiver receiver = new AbstractPushReceiver() {
	 * 
	 * 	public void onUnregistered(Context context) {
	 * 		//implement some code here
	 * 	}
	 * 
	 * 	public void onRegistered(Context context) { }
	 * 
	 * 	public void onNotificationReceived(PushNotification notification, Context context) { }
	 * 
	 * 	public void onError(int reason, Context context) { }
	 * }
	 * 
	 * IntentFilter filter = new IntentFilter();
	 * filter.addAction("com.infobip.push.intent.REGISTERED_FOR_NOTIFICATIONS");
	 * filter.addAction("com.infobip.push.intent.REGISTRATION_REFRESHED");
	 * filter.addAction("com.infobip.push.intent.UNREGISTERED_FROM_NOTIFICATIONS");
	 * filter.addAction("com.infobip.push.intent.ERROR");
	 * registerReceiver(receiver, filter);
	 * </pre>
	 * 
	 * @return <code>true</code> if the user is registered, <code>false</code> if not.</p>
	 */
	public boolean isRegistered() {
		String registrationId = prefs.getRegistrationId();		
		return !TextUtils.isEmpty(registrationId);
	}
	
	/**
	 * @deprecated As of release 1.2.0, calling this method is unnecessary because {@link #initialize(String, String, String)} method performs 
	 * registration expired check for you.
	 * <p>Checks if the registration has expired. Registration expires when you change your application version.</p>
	 * <p>This method only ensures you that the GCM registration hasn't expired. When an application is updated, it should invalidate its existing 
	 * registration, as it is not guaranteed to work with the new version.</p>
	 * @return <code>true</code> if the registration has expired, <code>false</code> if not.
	 */
	public boolean isRegistrationExpired() {
		
		if (isRegistered()) {
			int oldAppVersion = prefs.getApplicationVersion();
	        int newAppVersion = Util.getAppVersion(context.get());
	        if (oldAppVersion != Integer.MIN_VALUE && oldAppVersion != newAppVersion) {
	            Util.LogVerbose("Application version changed from " + oldAppVersion + " to " + newAppVersion + ".");            
	            return true;
	        }
		}
		return false;
	}
	
	/**
	 * @deprecated As of release 1.2.0, this method isn't supported anymore. Instead, calling {@link #initialize(String, String, String)} method from your 
	 * {@link Activity#onCreate()} will cover this scenario. 
	 * <p>Refreshes user's current registration.</p>
	 * <p>When the registration succeeds intent with the <code>com.infobip.push.intent.REGISTRATION_REFRESHED</code> action will be invoked and 
	 * {@link AbstractPushReceiver#onRegistrationRefreshed(Context)} will be executed.</p>
	 * <p>Check if you need to refresh your registration by calling {@link #isRegistrationExpired()} method.</p>
	 * @see AbstractPushReceiver
	 */
	public void refreshRegistration() {
		//do nothing
	}
	
	/**
	 * Gets the phone number.
	 * @return
	 */
	public String getPhoneNumber() {
		return prefs.getPhoneNumber();
	}

	/**
	 * <p>Sets the phone number and saves it instantly on the Infobip Push service. Use listener to know whether saving succeeded or not. 
	 * Phone number examples: 
	 * <ul>
	 * <li><b>Valid:</b> 44751234567 (UK), 385951234567 (Croatia), </li>
	 * <li><b>Invalid:</b> +44-75-123-4567 (UK), 00 385 95 123 4567 (Croatia) 
	 * </ul></p>
	 * 
	 * @param phoneNumber Phone number.
	 * @param userDataListener Listener that will be notified if the phone number saving succeeded. Pass null if you don't want 
	 * to listen for server response.
	 * @throws IllegalArgumentException Phone number should contain at least 8 numbers. No 00 prefix, spaces, pluses, dashes or parentheses are allowed.
	 * @see UserDataListener
	 */
	public void setPhoneNumber(String phoneNumber, final UserDataListener userDataListener) {
		
		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (!isLibraryInitialized()) {			
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			
			if (userDataListener != null) {
				userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.LIBRARY_NOT_INITIALIZED);
			}
			return;
		}
		
		if (!isRegistered()) {
			Util.LogWarn("User is not registered.");
			
			if (userDataListener != null) {
				userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.USER_NOT_REGISTERED);
			}
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to proceed.");
			
			if (userDataListener != null) {
				userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.INTERNET_NOT_AVAILABLE);
			}
			return;
		}
		
		if (!phoneNumber.matches("[0-9]+")) {
			throw new IllegalArgumentException("Phone number can contain only numbers.");
		}
		
		if (phoneNumber.length() <= 8) {
			throw new IllegalArgumentException("Phone number should have at least 8 characters.");
		}
		
		if (phoneNumber.startsWith("00")) {
			throw new IllegalArgumentException("00 prefix isn't allowed. Please, remove it.");
		}
		
		prefs.setPhoneNumber(phoneNumber);
		prefs.save();
		
		UserDataModel userDataModel = new UserDataModel();
		userDataModel.phoneNumber = phoneNumber;
		
		SendPostMessage sendPost = new SendPostMessage(String.format(URL_SAVE_NEW_USER_DATA, prefs.getDeviceId()), 
				userDataModel.toJsonObject(), Consts.PUT);
		sendPost.doInBackground(context.get(), new MessageSentListener() {
			
			@Override
			public void onSendSuccess() {
				Util.LogDebug("New phone number is saved on Infobip Push service.");
				
				if (userDataListener != null) {
					userDataListener.onUserDataSaved();
				}
			}
			
			@Override
			public void onSendFailed(int reason) {
				Util.LogError("Saving new phone number on Infobip Push service failed.");				
				
				if (userDataListener != null) {
					
					switch (reason) {					
					case MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE:
						userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.PUSH_SERVICE_NOT_AVAILABLE);
						break;
						
					default:
						userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.OPERATION_FAILED);
						break;
					}	
				}
			}
			
		});
	}

	/**
	 * <p>Overrides previously defined userID with the new one and saves the new userID on the Infobip Push service. 
	 * Use listener to know whether saving succeeded or not.</p>
	 * @param newUserId New userId to be saved.
	 * @param userDataListener Listener that will be notified if the saving new userID succeeded. Pass null if you don't want 
	 * to listen for server response.
	 * @throws IllegalArgumentException User ID cannot be null.
	 * @see UserDataListener
	 * @since 1.1.1
	 */
	public void saveNewUserId(String newUserId, final UserDataListener userDataListener) {
		
		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (!isLibraryInitialized()) {			
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			
			if (userDataListener != null) {
				userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.LIBRARY_NOT_INITIALIZED);
			}
			return;
		}
		
		if (!isRegistered()) {
			Util.LogWarn("User is not registered.");
			
			if (userDataListener != null) {
				userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.USER_NOT_REGISTERED);
			}
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to proceed.");
			
			if (userDataListener != null) {
				userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.INTERNET_NOT_AVAILABLE);
			}
			return;
		}
		
		if (newUserId == null) {
			throw new IllegalArgumentException("User ID cannot be null.");
		}

		UserDataModel userDataModel = new UserDataModel();
		userDataModel.userId = newUserId;
		
		SendPostMessage sendPost = new SendPostMessage(String.format(URL_SAVE_NEW_USER_DATA, prefs.getDeviceId()), 
				userDataModel.toJsonObject(), Consts.PUT);
		sendPost.doInBackground(context.get(), new MessageSentListener() {
			
			@Override
			public void onSendSuccess() {
				Util.LogDebug("New userID is saved on Infobip Push service.");
				
				if (userDataListener != null) {
					userDataListener.onUserDataSaved();
				}
			}
			
			@Override
			public void onSendFailed(int reason) {
				Util.LogError("Saving new userID on Infobip Push service failed.");				
				
				if (userDataListener != null) {
					
					switch (reason) {					
					case MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE:
						userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.PUSH_SERVICE_NOT_AVAILABLE);
						break;
						
					default:
						userDataListener.onUserDataSavingFailed(ChannelRegistrationListener.OPERATION_FAILED);
						break;
					}	
				}
			}
			
		});
	}
	
	/**
	 * <p>Registers to channels with an option to remove channels user was previously registered to. Use listener to know whether registration succeeded 
	 * or not.</p>
	 * @param channels List of channels to register to.
	 * @param removeExistingChannels <code>true</code> if you want to remove existing channels, <code>false</code> if your want to keep them and 
	 * register to additional ones.
	 * @param channelRegistrationListener Listener that will be notified if the registration to channels succeeded. Pass null if you don't want 
	 * to listen for server response.
	 * @throws IllegalArgumentException List of channel names cannot be null.
	 * @see ChannelRegistrationListener 
	 */
	public void registerToChannels(List<String> channels, boolean removeExistingChannels, final ChannelRegistrationListener channelRegistrationListener) {

		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (!isLibraryInitialized()) {			
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			
			if (channelRegistrationListener != null) {
				channelRegistrationListener.onChannelRegistrationFailed(ChannelRegistrationListener.LIBRARY_NOT_INITIALIZED);
			}
			return;
		}
		
		if (!isRegistered()) {
			Util.LogWarn("User is not registered.");
			
			if (channelRegistrationListener != null) {
				channelRegistrationListener.onChannelRegistrationFailed(ChannelRegistrationListener.USER_NOT_REGISTERED);
			}
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to proceed.");
			
			if (channelRegistrationListener != null) {
				channelRegistrationListener.onChannelRegistrationFailed(ChannelRegistrationListener.INTERNET_NOT_AVAILABLE);
			}
			return;
		}
		
		if (channels == null) {
			throw new IllegalArgumentException("List of channels cannot be null.");
		}

		if (removeExistingChannels) {
			Util.LogDebug("Registering user to new channels by removing existing channels...");
		
		} else {
			Util.LogDebug("Registering user to additional channels...");
		}
		
		ChannelRegistrationModel channelsRegistrationModel = new ChannelRegistrationModel();
		channelsRegistrationModel.channelNames = channels;
		channelsRegistrationModel.deviceId = prefs.getDeviceId();

		SendPostMessage sendPost = new SendPostMessage(URL_CHANNEL_REGISTRATION + String.valueOf(removeExistingChannels), 
				channelsRegistrationModel.toJsonObject());
		sendPost.doInBackground(context.get(), new MessageSentListener() {
			
			@Override
			public void onSendSuccess() {
				Util.LogDebug("User is registered to the new list of channels.");
				
				if (channelRegistrationListener != null) {
					channelRegistrationListener.onChannelsRegistered();
				}
			}
			
			@Override
			public void onSendFailed(int reason) {
				Util.LogError("Registration to channels failed.");				
				
				if (channelRegistrationListener != null) {
					
					switch (reason) {					
					case MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE:
						channelRegistrationListener.onChannelRegistrationFailed(ChannelRegistrationListener.PUSH_SERVICE_NOT_AVAILABLE);
						break;
						
					default:
						channelRegistrationListener.onChannelRegistrationFailed(ChannelRegistrationListener.OPERATION_FAILED);
						break;
					}	
				}
			}
			
		});
	}
	
	/**
	 * <p>Obtains channels user is registered to.</p>
	 * @param channelObtainListener Listener that will be notified of obtained channels.
	 * @throws IllegalArgumentException ChannelObtainListener instance cannot be null.
	 * @see ChannelObtainListener
	 */
	public void getRegisteredChannels(final ChannelObtainListener channelObtainListener) {
		
		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (channelObtainListener == null) {
			throw new IllegalArgumentException("ChannelObtainListener cannot be null.");
		}
		
		if (!isLibraryInitialized()) {
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			channelObtainListener.onChannelObtainFailed(ChannelObtainListener.LIBRARY_NOT_INITIALIZED);
			return;
		}
		
		if (!isRegistered()) {
			Util.LogWarn("User is not registered.");
			channelObtainListener.onChannelObtainFailed(ChannelObtainListener.USER_NOT_REGISTERED);
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to proceed.");
			channelObtainListener.onChannelObtainFailed(ChannelObtainListener.INTERNET_NOT_AVAILABLE);
			return;
		}
		
		Util.LogDebug("Obtaining channels user is registered to...");
		
		final SendPostMessage sendPost = new SendPostMessage(String.format(URL_REGISTERED_CHANNELS, prefs.getDeviceId()), SendPostMessage.GET);
		sendPost.doInBackground(context.get(), new MessageSentListener() {
			
			@Override
			public void onSendSuccess() {
				// Read channels from the server
				String channelsJsonResponse = sendPost.getResponse();	// response example: {"channels":["channel one","channel two"]}
				String[] channels = new String[0];
				
				try {
					JSONObject channelNames = new JSONObject(channelsJsonResponse);
					String channelsString = channelNames.optString("channels");
					JSONArray channelsJsonArray = new JSONArray(channelsString);
					
					channels = new String[channelsJsonArray.length()];
					
					for (int i = 0; i < channelsJsonArray.length(); i++) {
						channels[i] = channelsJsonArray.getString(i);
					}
					
				} catch (JSONException e) {
					Util.LogError(e.toString());
				}
				
				Util.LogVerbose("Found " + channels.length + " channels.");
				channelObtainListener.onChannelsObtained(channels);
			}
			
			@Override
			public void onSendFailed(int reason) {
				Util.LogError("Obtaining channels failed.");
								
				switch (reason) {					
				case MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE:
					channelObtainListener.onChannelObtainFailed(ChannelObtainListener.PUSH_SERVICE_NOT_AVAILABLE);
					break;
					
				default:
					channelObtainListener.onChannelObtainFailed(ChannelObtainListener.OPERATION_FAILED);
					break;
				}
			}
		});
	}
	
	/**
	 * <p>Obtains user's unreceived messages. </p> 
	 * @param unreceivedNotificationsListener Listener that will be notified of obtained messages.
	 * @throws IllegalArgumentException UnreceivedNotificationsListener instance cannot be null.
	 * @see UnreceivedNotificationsListener
	 */
	public void getUnreceivedNotifications(final UnreceivedNotificationsListener unreceivedNotificationsListener) {
		
		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (unreceivedNotificationsListener == null) {
			throw new IllegalArgumentException("UnreceivedNotificationsListener cannot be null.");
		}
		
		if (!isLibraryInitialized()) {
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			unreceivedNotificationsListener.onUnreceivedNotificationsObtainFailed(UnreceivedNotificationsListener.LIBRARY_NOT_INITIALIZED);
			return;
		}
		
		if (!isRegistered()) {
			Util.LogWarn("User is not registered.");
			unreceivedNotificationsListener.onUnreceivedNotificationsObtainFailed(UnreceivedNotificationsListener.USER_NOT_REGISTERED);
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to proceed.");
			unreceivedNotificationsListener.onUnreceivedNotificationsObtainFailed(UnreceivedNotificationsListener.INTERNET_NOT_AVAILABLE);
			return;
		}
		
		Util.LogDebug("Obtaining unreceived notifications and confirming delivery...");
		
		final SendPostMessage sendPost = new SendPostMessage(String.format(URL_UNREAD_MESSAGES, prefs.getDeviceId()), SendPostMessage.GET);
		sendPost.doInBackground(context.get(), new MessageSentListener() {
			
			@Override
			public void onSendSuccess() {				
				// Read notifications from the server
				String messagesJsonResponse = sendPost.getResponse();
				
				try {					
					JSONArray messagesJsonArray = new JSONArray(messagesJsonResponse);
					
					List<PushNotification> messages = new ArrayList<PushNotification>();
					
					for (int i = 0; i < messagesJsonArray.length(); i++) {
						JSONObject messageJsonObj = new JSONObject(messagesJsonArray.getString(i));
						
						String messageId =  messageJsonObj.optString(Consts.PUSH_MESSAGE_ID);
						String mimeType = messageJsonObj.optString(Consts.PUSH_MIME_TYPE);
						String jsonData = messageJsonObj.optString(Consts.PUSH_JSON_DATA);
						
						if (TextUtils.isEmpty(jsonData)) {
							Util.LogWarn("Missing \"" + Consts.PUSH_JSON_DATA + "\" parameter. Ignoring received notification...");
							return;
						}
						
						// check MIME type
						if (TextUtils.isEmpty(mimeType)) {
							Util.LogWarn("Missing \"" + Consts.PUSH_MIME_TYPE + "\" parameter. Ignoring received notification..."); 
						
						} else if (mimeType.equals(Consts.MIME_TYPE_TEXT_PLAIN) || mimeType.equals(Consts.MIME_TYPE_TEXT_HTML)) {
							int notificationId = -1;  //-1 since message isn't displayed						
							
							PushNotification notification = (PushNotification) InfobipNotificationHandlingUtil.fetchNotificationFromJson(messageId, notificationId, messageJsonObj);
							
							messages.add(notification);
						}
					}
					
					Util.LogVerbose("Found " + messages.size() + " notifications.");
					unreceivedNotificationsListener.onUnreceivedNotificationsObtained(messages);
					
				} catch (JSONException e) {
					Util.LogError("Failed to parse received JSON message.");
					unreceivedNotificationsListener.onUnreceivedNotificationsObtainFailed(UnreceivedNotificationsListener.OPERATION_FAILED);
				}
			}
			
			@Override
			public void onSendFailed(int reason) {
				Util.LogError("Obtaining unreceived notifications failed.");
				
				switch (reason) {					
				case MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE:
					unreceivedNotificationsListener.onUnreceivedNotificationsObtainFailed(UnreceivedNotificationsListener.PUSH_SERVICE_NOT_AVAILABLE);
					break;
					
				default:
					unreceivedNotificationsListener.onUnreceivedNotificationsObtainFailed(UnreceivedNotificationsListener.OPERATION_FAILED);
					break;
				}
			}
		});
	}

	/**
	 * <p>Notifies the Infobip Push service about an opened notification. Use listener to know whether notifying succeeded or not.</p>
	 * <p>Use this method only if you don't want to use default message handling but you still want to notify the Infobip Push service about a notification
	 * opening. </p>
	 * @param pushId Unique identifier of the opened notification.
	 * @param notificationOpenedListener Listener that will be notified if the notification opened request succeeded. Pass null if you don't want 
	 * to listen for server response.
	 * @throws IllegalArgumentException Push ID cannot be an empty string or null.
	 */
	public void notifyNotificationOpened(String pushId, final NotificationOpenedListener notificationOpenedListener) {
		
		if (checkManifest && isDebugModeEnabled()) {
			checkManifest();
		}
		
		if (!isLibraryInitialized()) {
			Util.LogError("You haven't initialized the library. Please, call initialize(String senderId, String applicationId, " +
					"String applicationSecret) method before calling this method.");
			
			if (notificationOpenedListener != null) {
				notificationOpenedListener.onNotifyNotificationOpenedFailed(NotificationOpenedListener.LIBRARY_NOT_INITIALIZED);
			}
			return;
		}
		
		if (!isRegistered()) {
			Util.LogWarn("User is not registered.");
			
			if (notificationOpenedListener != null) {
				notificationOpenedListener.onNotifyNotificationOpenedFailed(NotificationOpenedListener.USER_NOT_REGISTERED);
			}
			return;
		}
		
		if (!Util.isNetworkConnected(getContext())) {
			Util.LogError("Internet connection is not available, unable to forward notification opened event.");
			
			if (notificationOpenedListener != null) {
				notificationOpenedListener.onNotifyNotificationOpenedFailed(NotificationOpenedListener.INTERNET_NOT_AVAILABLE);
			}
			return;
		}
		
		if (TextUtils.isEmpty(pushId)) {
			throw new IllegalArgumentException("Push ID cannot be an empty string or null.");
		}

		Util.LogDebug("Notification opened event forwarded to the server");

		NotificationOpenedModel notificationOpenedModel = new NotificationOpenedModel();
		notificationOpenedModel.deviceId = prefs.getDeviceId();
		notificationOpenedModel.messageId = pushId;
		
		SendPostMessage sendPost = new SendPostMessage(URL_NOTIFICATION_OPENED, notificationOpenedModel.toJsonObject(), SendPostMessage.PUT);
		sendPost.doInBackground(context.get(), new MessageSentListener() {
			
			@Override
			public void onSendSuccess() {
				Util.LogDebug("User opened notification.");
				
				if (notificationOpenedListener != null) {
					notificationOpenedListener.onNotifyNotificationOpenedSuccess();
				}
			}
			
			@Override
			public void onSendFailed(int reason) {
				Util.LogError("Notification opened request failed.");				
				
				if (notificationOpenedListener != null) {
				
					switch (reason) {						
					case MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE:
						notificationOpenedListener.onNotifyNotificationOpenedFailed(NotificationOpenedListener.PUSH_SERVICE_NOT_AVAILABLE);
						break;
						
					default:
						notificationOpenedListener.onNotifyNotificationOpenedFailed(NotificationOpenedListener.OPERATION_FAILED);
						break;
					}	
				}
			}
		});
	}
	
	/**
	 * @deprecated As of release 1.1.1, sending location instantly in foreground to the Infobip Push service isn't supported. Instead, use 
	 * {@link PushLocationManager#enableLocation()} for enabling and {@link PushLocationManager#disableLocation()} for disabling background location updates.
	 * <p>Enables location sending and sends location to the Infobip Push service.</p>
	 * @param enabled
	 */
	public void setGeoLocationEnabled(boolean enabled) {
		// do nothing
	}
	
	/**
	 * @deprecated As of release 1.1.1, this method isn't supported. Instead, use {@link PushLocationManager#isLocationEnabled()}.
	 * <p>Checks if the location sending is enabled.</p>
	 * @return <code>true</code> if the location is enabled, <code>false</code> otherwise. Since deprecation, this method will always return false.
	 */
	public boolean isGeoLocationEnabled() {
		return false;  //do nothing
	}

	/**
	 * @deprecated As of release 1.1.1, sending your own location instantly to the Infobip Push service isn't supported. Instead, use 
	 * {@link PushLocationManager} class for managing background location updates. Call {@link PushLocationManager#useCustomLocationService(boolean)} for
	 * providing your own location, and after enabling location, save new location periodically using {@link PushLocationManager#saveUserLocation(Location)}.
	 * <p>Sets given location and sends it to the Infobip Push service.</p>
	 * <p>If the request for sending location fails <code>com.infobip.push.intent.ERROR</code> action will be invoked after which you can check the 
	 * reason why it failed by extending {@link AbstractPushReceiver#onError(int, Context)} method.</p>
	 * @param location Location containing latitude and longitude obtained by your own location service.
	 */
	public void setUserLocation(Location location) {
		// do nothing
	}

	/**
	 * <p>Enables or disables Push library debug mode.</p>
	 * <p>Debug mode should be used during application debugging. When it is enabled, library will print a lot of debug info to the log output. 
	 * By default, debug mode is disabled.</p>
	 * @param debugMode Set to <code>true</code> to enable debug mode, or set to <code>false</code> to disable debug mode.
	 */
	public void setDebugModeEnabled(boolean debugMode) {
		Util.setDebugModeEnabled(debugMode);
	}
	
	/**
	 * <p>Checks if the Push library is in debug mode.</p>
	 * @return <code>true</code> if Push library is in debug mode, or <code>false</code> otherwise. By default, Push library is not in debug mode.
	 * @see #setDebugModeEnabled(boolean)
	 */
	public boolean isDebugModeEnabled() {
		return Util.isDebugModeEnabled();
	}
	
	/**
	 * <p>Used to override default message handling/displaying mechanism.</p>
	 * @param developerHandleMessage Set to <code>true</code> if you want to handle messages on your own, <code>false</code> if you want Push library 
	 * to handle them.
	 */
	public void overrideDefaultMessageHandling(boolean developerHandleMessage) {		
		prefs.overrideDefaultMessageHandling(developerHandleMessage);
		prefs.save();
	}
	
	/**
	 * <p>Checks whether you handle push notifications on your own or via Push library (default).</p>
	 * @return <code>true</code> if you handle messages, <code>false</code> if Push library handles them.
	 * @see #overrideDefaultMessageHandling(boolean)
	 */
	public boolean isDefaultMessageHandlingOverriden() {
		return prefs.isDefaultMessageHandlingOverriden();
	}
	
	/**
	 * <p>Checks whether required permissions, receivers and services from AndroidManifest.xml file are defined and logs missing ones.</p>
	 * @throws PushConfigurationException If Push configuration exception occurred (wrong Manifest).
	 */
	public void checkManifest() {
		// GCM permissions		
		Util.checkPushRequiredPermission(context.get().getPackageName() + Consts.GCM_PERMISSION_C2D_MESSAGE, context.get());		
		Util.checkPushRequiredPermission(Consts.GCM_PERMISSION_RECEIVE, context.get());	
		Util.checkPushRequiredPermission(permission.WAKE_LOCK, context.get());
		Util.checkPushRequiredPermission(permission.GET_ACCOUNTS, context.get());
		
		// Infobip permissions
		Util.checkPushRequiredPermission(permission.ACCESS_NETWORK_STATE, context.get());
		Util.checkPushRequiredPermission(permission.INTERNET, context.get());
		
		// services
		Util.checkPushRequiredService(InfobipPushService.class.getCanonicalName(), context.get());
		
		// receivers
		Util.checkPushRequiredReceiver(InternalReceiver.class.getCanonicalName(), context.get());
		Util.checkPushRequiredReceiver(InfobipPushReceiver.class.getCanonicalName(), context.get());
	}
	
}