package com.infobip.push;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * <p>Class that contains information about additional data being sent during registration. This data is stored on the Infobip Push service and it is 
 * deleted when you unregister the user.</p>
 * <p>After registration, this data can be updated with method from {@link PushNotificationManager} class. You can update:
 * <ul>
 * <li>channels user is registered to by using {@link PushNotificationManager#registerToChannels(List, boolean, ChannelRegistrationListener)} 
 * method,</li>
 * <li>user ID by using {@link PushNotificationManager#saveNewUserId(String, UserDataListener)},</li>
 * <li>phone number by using {@link PushNotificationManager#setPhoneNumber(String, UserDataListener)}</li>
 * </ul>
 * </p>
 * @see PushNotificationManager#register(RegistrationData)
 */
public class RegistrationData {

	private String phoneNumber = "";
	private String userId = "";
	private List<String> channels = new ArrayList<String>();
	private HashMap<String, String> additionalInfo = new HashMap<String, String>();

	/**
	 * <p>Empty constructor.</p>
	 */
	public RegistrationData() {
		
	}
	
	/**
	 * <p>Create <code>RegistrationData</code> instance by providing user identifier.</p>
	 * @param userId User identifier.
	 * @throws IllegalArgumentException User ID cannot be null.
	 */
	public RegistrationData(String userId) {
		setUserId(userId);
	}
	
	/**
	 * <p>Create <code>RegistrationData</code> instance by providing user identifier and channels.</p>
	 * @param userId User identifier.
	 * @param channels List of channel names.
	 * @throws IllegalArgumentException User ID or channels cannot be null.
	 */
	public RegistrationData(String userId, List<String> channels) {
		setUserId(userId);
		setChannels(channels);
	}
	
	/**
	 * <p>Create <code>RegistrationData</code> instance by providing user identifier and additional information.</p>
	 * @param userId User identifier.
	 * @param additionalInfo Additional information set as a &lt;key, value&gt; pair.
	 */
	public RegistrationData(String userId, HashMap<String, String> additionalInfo) {
		setUserId(userId);
		this.additionalInfo = additionalInfo;
	}
	
	/**
	 * <p>Create <code>RegistrationData</code> instance by providing user identifier, channels and additional information.</p>
	 * @param userId User identifier.
	 * @param channels List of channel names.
	 * @param additionalInfo Additional information set as a &lt;key, value&gt; pair.
	 */
	public RegistrationData(String userId, List<String> channels, HashMap<String, String> additionalInfo) {
		setUserId(userId);
		setChannels(channels);
		this.additionalInfo = additionalInfo;
	}
	
	/**
	 * <p>Gets user identifier.</p>
	 * @return User identifier.
	 */
	public String getUserId() {
		return userId;
	}

	/**
	 * <p>Sets user identifier.</p>
	 * @param userId User identifier.
	 * @throws IllegalArgumentException User ID cannot be null.
	 */
	public void setUserId(String userId) {
		if (userId == null) {
			throw new IllegalArgumentException("User ID cannot be null.");
		}
		
		this.userId = userId;
	}
	
	/**
	 * Gets the phone number.
	 * @return
	 */
	public String getPhoneNumber() {
		return phoneNumber;
	}
	
	/**
	 * <p>Sets the phone number. Phone number examples: 
	 * <ul>
	 * <li><b>Valid:</b> 44751234567 (UK), 385951234567 (Croatia), </li>
	 * <li><b>Invalid:</b> +44-75-123-4567 (UK), 00 385 95 123 4567 (Croatia) 
	 * </ul></p>
	 * 
	 * @param phoneNumber Phone number.
	 * @throws IllegalArgumentException Phone number should contain at least 8 numbers. No 00 prefix, spaces, pluses, dashes or parentheses are allowed.
	 */
	public void setPhoneNumber(String phoneNumber) {
		if (!phoneNumber.matches("[0-9]+")) {
			throw new IllegalArgumentException("Phone number can contain only numbers.");
		}
		
		if (phoneNumber.length() <= 8) {
			throw new IllegalArgumentException("Phone number should have at least 8 characters.");
		}
		
		if (phoneNumber.startsWith("00")) {
			throw new IllegalArgumentException("00 prefix isn't allowed. Please, remove it.");
		}
		
		this.phoneNumber = phoneNumber;
	}
	
	/**
	 * <p>Gets channel names.</p>
	 * @return Channel names.
	 */
	public List<String> getChannels() {
		return channels;
	}

	/**
	 * <p>Sets channel names.</p>
	 * @param channels List of channel names.
	 * @throws IllegalArgumentException List of channel names cannot be null.
	 */
	public void setChannels(List<String> channels) {
		if (channels == null) {
			throw new IllegalArgumentException("List of channels cannot be null.");
		}
		
		this.channels = channels;
	}
	
	/**
	 * <p>Gets additional information.</p>
	 * @return Additional information set as a &lt;key, value&gt; pair.
	 */
	public HashMap<String, String> getAdditionalInfo() {
		return additionalInfo;
	}
	
	/**
	 * <p>Adds additional information.</p>
	 * @param key Specified key.
	 * @param value Specified value.
	 */
	public void addAdditionalInfo(String key, String value) {
		additionalInfo.put(key, value);
	}
}
