package com.infobip.push;

import java.util.List;

import com.infobip.push.lib.SendPostMessage.MessageSentListener;

/**
 * <p>Callback interface invoked when getting previously unreceived notifications.</p>
 * <p>When notifications are obtained {@link #onUnreceivedNotificationsObtained(List)} will be called, giving you the list of the unreceived notifications, 
 * each being passed through {@link PushNotification} instance. 
 * If the obtaining fails {@link #onUnreceivedNotificationsObtainFailed(int)} will be called with one of the following reasons: </p>
 * <ul><li>{@link #PUSH_SERVICE_NOT_AVAILABLE}</li>
 * <li>{@link #OPERATION_FAILED}</li>
 * <li>{@link #LIBRARY_NOT_INITIALIZED}</li>
 * <li>{@link #USER_NOT_REGISTERED}</li>
 * <li>{@link #INTERNET_NOT_AVAILABLE}</li></ul>
 * 
 * @see PushNotificationManager#getUnreceivedNotifications(UnreceivedNotificationsListener)
 */
public interface UnreceivedNotificationsListener {

	//---------------------------------- CONSTANTS ----------------------------------//
	
	/**
	 * <p>Used as a reason in {@link UnreceivedNotificationsListener#onUnreceivedNotificationsObtainFailed(int)}. Means that the Infobip Push 
	 * service is currently unavailable.</p>
	 * Constant value: {@value}
	 */
	public static final int PUSH_SERVICE_NOT_AVAILABLE	= MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE;
	
	/**
	 * <p>Used as a reason in {@link UnreceivedNotificationsListener#onUnreceivedNotificationsObtainFailed(int)}. Means that operation failed.</p>
	 * Constant value: {@value}
	 */
	public static final int OPERATION_FAILED			= MessageSentListener.OPERATION_FAILED;
	
	/**
	 * <p>Used as a reason in {@link UnreceivedNotificationsListener#onUnreceivedNotificationsObtainFailed(int)}. Means that the library 
	 * is not initialized.</p>
	 * Constant value: {@value}
	 */
	public static final int LIBRARY_NOT_INITIALIZED		= PushNotificationManager.LIBRARY_NOT_INITIALIZED;
	
	/**
	 * <p>Used as a reason in {@link UnreceivedNotificationsListener#onUnreceivedNotificationsObtainFailed(int)}. Means that the user is 
	 * not registered.</p>
	 * Constant value: {@value}
	 */
	public static final int USER_NOT_REGISTERED			= PushNotificationManager.USER_NOT_REGISTERED;
	
	/**
	 * <p>Used as a reason in {@link UnreceivedNotificationsListener#onUnreceivedNotificationsObtainFailed(int)}. Means that the Internet 
	 * access is unavailable.</p>
	 * Constant value: {@value}
	 */
	public static final int INTERNET_NOT_AVAILABLE		= PushNotificationManager.INTERNET_NOT_AVAILABLE;
	
	//---------------------------------- CONSTANTS ----------------------------------//
	
	/**
	 * <p>Called when unreceived notifications are obtained from Push service.</p>
	 * 
	 * @param notifications List of obtained notifications.
	 */
	public void onUnreceivedNotificationsObtained(List<PushNotification> notifications);
	
	/**
	 * <p>Called when notification obtaining fails.</p>
	 * 
	 * @param reason Reason why obtaining failed will be one of the following: <br />
	 * 		<ul><li>{@link #PUSH_SERVICE_NOT_AVAILABLE} - Push service unavailable.</li>
	 * 		<li>{@link #OPERATION_FAILED} - Communication error.</li>
	 * 		<li>{@link #LIBRARY_NOT_INITIALIZED} - Library is not initialized.</li>
	 * 		<li>{@link #USER_NOT_REGISTERED} - User is not registered.</li>
	 * 		<li>{@link #INTERNET_NOT_AVAILABLE} - Internet access is unavailable.</li></ul>
	 */
	public void onUnreceivedNotificationsObtainFailed(int reason);
	
}
