package com.infobip.push;

import com.infobip.push.lib.SendPostMessage.MessageSentListener;

/**
 * <p>Callback interface invoked when the user is saving new userID.</p>
 * <p>When the new userID is saved {@link #onUserDataSaved()} will be called, otherwise if it fails
 * {@link #onUserDataSavingFailed(int)} will be called with one of the following reasons: </p>
 * <ul><li>{@link #PUSH_SERVICE_NOT_AVAILABLE}</li>
 * <li>{@link #OPERATION_FAILED}</li>
 * <li>{@link #LIBRARY_NOT_INITIALIZED}</li>
 * <li>{@link #USER_NOT_REGISTERED}</li>
 * <li>{@link #INTERNET_NOT_AVAILABLE}</li></ul>
 * 
 * @see PushNotificationManager#saveNewUserId(String, UserDataListener)
 */
public interface UserDataListener {
	
	//---------------------------------- CONSTANTS ----------------------------------//
	
	/**
	 * <p>Used as a reason in {@link UserDataListener#onUserDataSavingFailed(int)}. Means that the Infobip Push service is currently unavailable.</p>
	 * Constant value: {@value}
	 */
	public static final int PUSH_SERVICE_NOT_AVAILABLE	= MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE;
	
	/**
	 * <p>Used as a reason in {@link UserDataListener#onUserDataSavingFailed(int)}. Means that operation failed.</p>
	 * Constant value: {@value}
	 */
	public static final int OPERATION_FAILED			= MessageSentListener.OPERATION_FAILED;
	
	/**
	 * <p>Used as a reason in {@link UserDataListener#onUserDataSavingFailed(int)}. Means that the library is not initialized.</p>
	 * Constant value: {@value}
	 */
	public static final int LIBRARY_NOT_INITIALIZED		= PushNotificationManager.LIBRARY_NOT_INITIALIZED;
	
	/**
	 * <p>Used as a reason in {@link UserDataListener#onUserDataSavingFailed(int)}. Means that the user is not registered.</p>
	 * Constant value: {@value}
	 */
	public static final int USER_NOT_REGISTERED			= PushNotificationManager.USER_NOT_REGISTERED;
	
	/**
	 * <p>Used as a reason in {@link UserDataListener#onUserDataSavingFailed(int)}. Means that the Internet access is unavailable.</p>
	 * Constant value: {@value}
	 */
	public static final int INTERNET_NOT_AVAILABLE		= PushNotificationManager.INTERNET_NOT_AVAILABLE;
	
	//---------------------------------- CONSTANTS ----------------------------------//
	
	/**
	 * <p>Called when the new userID is successfully saved.</p>
	 * 
	 */
	public void onUserDataSaved();
	
	/**
	 * <p>Called when saving new userID to the Infobip Push service fails.</p>
	 * 
	 * @param reason Reason why registering failed will be one of the following: <br />
	 * 		<ul><li>{@link #PUSH_SERVICE_NOT_AVAILABLE} - Push service unavailable.</li>
	 * 		<li>{@link #OPERATION_FAILED} - Communication error.</li>
	 * 		<li>{@link #LIBRARY_NOT_INITIALIZED} - Library is not initialized.</li>
	 * 		<li>{@link #USER_NOT_REGISTERED} - User is not registered.</li>
	 * 		<li>{@link #INTERNET_NOT_AVAILABLE} - Internet access is unavailable.</li></ul>
	 */
	public void onUserDataSavingFailed(int reason);
}
