package com.infobip.push.lib;

import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.UUID;

import android.Manifest.permission;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.text.TextUtils;
import android.widget.RemoteViews;

import com.infobip.push.PushNotification;
import com.infobip.push.PushNotificationBuilder;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.Util;

class DefaultActionBuilder {

	Context context;
	PushNotification pushMessage;
	
	Notification notification;
	NotificationManager mNotificationManager;
	PushNotificationBuilder builder;
	
	public DefaultActionBuilder(Context context, PushNotification pushMessage) {
		this.context = context;
		this.pushMessage = pushMessage;
		
		this.builder = new PushNotificationBuilder(context);
		this.notification = new Notification();
		this.mNotificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
	}
	
	/**
	 * Displays notifications with <code>text/plain</code> and <code>text/html</code> MIME type in phones status bar and sends notification opened request.
	 */
	void executeDefaultAction() {		
		setStatusBarNotification();
		
		PendingIntent contentIntent = generateNotificationOpenedPendingIntent();
		
		if (builder.hasLayoutId()) {
			setRemoteViewContentLayout(contentIntent);
		} else {
			setStandardContentViewLayout(contentIntent);			
		}

		if (!builder.isInQuietTime()) {
            setSound();
            setVibration();
            setLights();
		}
		
        notification.flags |= Notification.FLAG_AUTO_CANCEL;
		
		// show notification with hashed message id
		mNotificationManager.notify(pushMessage.notificationId, notification);
	}
	
	/**
	 * Sets status bar notification on default action.
	 */
	private void setStatusBarNotification() {
		
		if (builder.hasIconDrawableId()) {
			notification.icon = builder.getIconDrawableId();		
		} else {
			notification.icon = android.R.drawable.ic_dialog_email;
		}
		
		if (!TextUtils.isEmpty(builder.getTickerText())) {
			notification.tickerText = builder.getTickerText();			
		
		} else {	
			if (TextUtils.isEmpty(pushMessage.title)) {
				notification.tickerText = pushMessage.message;
			} else {
				notification.tickerText = pushMessage.title;
			}
		}
		
		notification.when = System.currentTimeMillis();
	}

	private PendingIntent generateNotificationOpenedPendingIntent() {
		Intent openedIntent = new Intent(Consts.NOTIFICATION_OPENED_ACTION + UUID.randomUUID().toString());	
		openedIntent.setClass(context, InternalReceiver.class);
		openedIntent.putExtra("com.infobip.push", pushMessage);		
		return PendingIntent.getBroadcast(context, 0, openedIntent, 0);
	}

	/**
	 * Sets remote view resources as set in PushNotificationBuilder.
	 */
	private void setRemoteViewContentLayout(PendingIntent contentIntent) {
		RemoteViews contentView = new RemoteViews(context.getPackageName(), builder.getLayoutId());
		
		if (builder.hasImageId()) {
			contentView.setImageViewResource(builder.getImageId(), builder.getImageDrawableId());
		}
		
		if (builder.hasTitleId()) {
			
			if (!TextUtils.isEmpty(pushMessage.title)) {
				contentView.setTextViewText(builder.getTitleId(), pushMessage.title);
			} else {
				contentView.setTextViewText(builder.getTitleId(), builder.getApplicationName());
			}
		}
		
		if (builder.hasTextId()) {
			contentView.setTextViewText(builder.getTextId(), pushMessage.message);
		}
		
		if (builder.hasDateId()) {
			String date = new SimpleDateFormat("HH:mm", Locale.getDefault()).format(new Date(notification.when));
			contentView.setTextViewText(builder.getDateId(), date);
		}
		notification.contentView = contentView;
		notification.contentIntent = contentIntent;
	}

	@SuppressWarnings("deprecation")
	private void setStandardContentViewLayout(PendingIntent contentIntent) {
		
		if (TextUtils.isEmpty(pushMessage.title)) {
			notification.setLatestEventInfo(context, builder.getApplicationName(), pushMessage.message, contentIntent);
		} else {
			notification.setLatestEventInfo(context, pushMessage.title, pushMessage.message, contentIntent);
		}
	}

	private void setSound() {

        if ((builder.getSound() == PushNotificationBuilder.UNSET && pushMessage.sound) || (builder.getSound() == PushNotificationBuilder.ENABLED)) {
        	
            if (builder.getSoundResourceId() != PushNotificationBuilder.UNSET) {
                notification.sound = Uri.parse(ContentResolver.SCHEME_ANDROID_RESOURCE + "://" + context.getPackageName() + "/" + builder.getSoundResourceId());
            } else {
                notification.defaults |= Notification.DEFAULT_SOUND;
            }
        }
    }

    private void setVibration() {

        if ((builder.getVibration() == PushNotificationBuilder.UNSET && pushMessage.vibrate) || (builder.getVibration() == PushNotificationBuilder.ENABLED)) {

            if (Util.isPermissionGranted(permission.VIBRATE, context)) {

                if (builder.getVibrationPattern().length != 0) {
                    notification.vibrate = builder.getVibrationPattern();
                } else {
                    notification.defaults |= Notification.DEFAULT_VIBRATE;
                }
            } else {
            	Util.LogError("Please add the following required permission to the AndroidManifest.xml: " + permission.VIBRATE);
            }
        }
    }

    /**
     * Takes flag data from intent and sets notification flags correspondingly.
     */
    private void setLights() {

        if ((builder.getLight() == PushNotificationBuilder.UNSET) && (pushMessage.lights || builder.getLight() == PushNotificationBuilder.ENABLED)) {

            if (builder.getLightsColor() != PushNotificationBuilder.UNSET) {
                notification.ledARGB = builder.getLightsColor();
            } else {
                notification.defaults |= Notification.DEFAULT_LIGHTS;
            }

            if ((builder.getLightsOnMS() != PushNotificationBuilder.UNSET) && (builder.getLightsOffMS() != PushNotificationBuilder.UNSET)) {
                notification.ledOnMS = builder.getLightsOnMS();
                notification.ledOffMS = builder.getLightsOffMS();
            }
        	
            notification.flags |= Notification.FLAG_SHOW_LIGHTS;
        }
    }
	
}
