package com.infobip.push.lib;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;

import com.infobip.push.AbstractPushReceiver;
import com.infobip.push.PushNotification;
import com.infobip.push.PushNotificationManager;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.Util;

public final class InfobipNotificationHandlingUtil {
	
	/**
	 * <p>Creates <code>PushNotification</code> instance out of data received in intent.</p>
	 * @param intent Intent with data received from Push server about push notification.
	 * @return <code>PushNotification</code> object.
	 * @see PushNotification
	 */
	public static InfobipPushNotification fetchParcelledNotificationFromIntent(Intent intent) {
		InfobipPushNotification notification = (PushNotification) intent.getParcelableExtra("com.infobip.push");	
		return notification;
	}
	
	/**
	 * <p>Creates <code>PushNotification</code> instance out of data received in intent.</p>
	 * @param intent Intent with data received from Push server about push notification.
	 * @return <code>PushNotification</code> object.
	 * @see PushNotification
	 */
	public static InfobipPushNotification fetchNotificationFromJson(String messageId, int notificationId, JSONObject data) {
		InfobipPushNotification notification = new PushNotification();
		
		JSONObject jsonData = data.optJSONObject(Consts.PUSH_JSON_DATA);
		
		String mimeType = data.optString(Consts.PUSH_MIME_TYPE);
		String message = jsonData.optString(Consts.PUSH_MESSAGE);
		String title = jsonData.optString(Consts.PUSH_TITLE);
		String url = jsonData.optString(Consts.PUSH_URL);
		String soundStr = jsonData.optString(Consts.PUSH_SOUND);
		String vibrateStr = jsonData.optString(Consts.PUSH_VIBRATE);
		String lightStr = jsonData.optString(Consts.PUSH_LIGHT);
		String additionalInfo = jsonData.optString(Consts.PUSH_ADD_INFO);
		String mediaData = jsonData.optString(Consts.PUSH_MEDIA);
		
		notification.id = messageId;
		notification.notificationId = notificationId;
		notification.mimeType = mimeType;
		notification.message = message;
		notification.title = title;
		notification.url = url;
		notification.sound = Boolean.valueOf(soundStr);
		notification.vibrate = Boolean.valueOf(vibrateStr);
		notification.lights = Boolean.valueOf(lightStr);
		notification.additionalInfo = additionalInfo;
		notification.mediaData = mediaData;
			
		return notification;
	}
	
	public static void handleInfobipMessage(Context context, String messageId, String message) {
		PushNotificationManager manager =  new PushNotificationManager(context);
		boolean defaultAction = !manager.isDefaultMessageHandlingOverriden();
		
		try {
			JSONObject messageJson = new JSONObject(message);
			String mimeType = messageJson.optString(Consts.PUSH_MIME_TYPE);
			String jsonData = messageJson.optString(Consts.PUSH_JSON_DATA);
			
			if (TextUtils.isEmpty(jsonData)) {
				Util.LogWarn("Missing \"" + Consts.PUSH_JSON_DATA + "\" parameter. Ignoring received notification...");
				return;
			}
			
			// check MIME type
			if (TextUtils.isEmpty(mimeType)) { 
				//don't show notification
				Util.LogWarn("Missing \"" + Consts.PUSH_MIME_TYPE + "\" parameter. Ignoring received notification..."); 
			
			} else if (mimeType.equals(Consts.MIME_TYPE_TEXT_HTML) || mimeType.equals(Consts.MIME_TYPE_TEXT_PLAIN)) {
				int notificationId = -1;
				PushNotification notification = (PushNotification) InfobipNotificationHandlingUtil.fetchNotificationFromJson(messageId, notificationId, messageJson);
				
				if (defaultAction) {
					//show notification
					notificationId = messageId.hashCode();
					notification.notificationId = notificationId;
					executeDefaultAction(context, notification);
				}
				sendNotificationReceivedBroadcast(context, notification);
				
			} else { 
				//don't show notification
				Util.LogWarn("Unknown MIME type. Default action for requested MIME type is not defined. Ignoring received notification..."); 
			}
			
		} catch (JSONException e) {
			Util.LogError("Failed to parse received JSON message.");
		}
		
	}
	
	/**
	 * Displays notification with <code>text/plain</code> and <code>text/html</code> MIME type in phones status bar and attaches notification opened 
	 * request to pending intent.
	 */
	private static void executeDefaultAction(Context context, PushNotification notification) {		
		DefaultActionBuilder dab = new DefaultActionBuilder(context, notification);
		dab.executeDefaultAction();
	}	
	
	/**
	 * Sends notification received broadcast with extras defining <code>PushNotification</code> object to notify the {@link AbstractPushReceiver} 
	 * class about the received notification for text/plain and text/html MIME types. The intent action being set is 
	 * <code>your_package_name.NOTIFICATION_RECEIVED</code>.
	 * @see #handleInfobipMessage(String, String)
	 */
	private static void sendNotificationReceivedBroadcast(Context context, PushNotification notification) {		
		Intent notificationReceivedIntent = new Intent(Consts.RECEIVER_NOTIFICATION_RECEIVED_ACTION);
		notificationReceivedIntent.putExtra("com.infobip.push", notification);
		notificationReceivedIntent.addCategory(context.getPackageName());
		
		context.sendBroadcast(notificationReceivedIntent, context.getPackageName() + Consts.GCM_PERMISSION_C2D_MESSAGE);
	}
	
	/**
	 * Sends notification received broadcast with extras defining <code>PushNotification</code> object to notify the {@link AbstractPushReceiver} 
	 * class about the received notification for text/plain and text/html MIME types. The intent action being set is 
	 * <code>your_package_name.NOTIFICATION_RECEIVED</code>.
	 * @see #handleInfobipMessage(String, String)
	 */
	public static void sendNotificationOpenedBroadcast(Context context, Intent intent) {
		Intent notificationOpenedIntent = new Intent(Consts.RECEIVER_NOTIFICATION_OPENED_ACTION);
		notificationOpenedIntent.putExtras(intent);
		notificationOpenedIntent.addCategory(context.getPackageName());
		
		context.sendBroadcast(notificationOpenedIntent, context.getPackageName() + Consts.GCM_PERMISSION_C2D_MESSAGE);
	}
	
}
