package com.infobip.push.lib;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Random;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.AlarmManager;
import android.app.IntentService;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.PowerManager;
import android.os.SystemClock;
import android.text.TextUtils;

import com.infobip.push.PushNotification;
import com.infobip.push.PushNotificationManager;
import com.infobip.push.lib.livegeo.LiveGeoIntentService;
import com.infobip.push.lib.model.NotificationOpenedModel;
import com.infobip.push.lib.model.RegistrationModel;
import com.infobip.push.lib.model.UnregistrationModel;
import com.infobip.push.lib.model.UserDataModel;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.LiveGeoUtil;
import com.infobip.push.lib.util.LocationManagerUtil;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;

/**
 * <p>Service class that handles operations with GCM and Infobip Push service and implements default action for displaying received notification. </p>
 * <p>GCM operations are: registration, unregistration and receiving the notification. </p>
 * Infobip Push service operations are: 
 * <ul><li>registration performed after GCM registration succeeds, </li>
 * <li>unregistration performed after GCM unregistration succeeds, </li>
 * <li>confirmation performed when the notification is received from GCM, </li>
 * <li>notifying service about the opened push notification. </li></ul>
 *
 */
public class InfobipPushService extends IntentService {
	
	/*------------------------------- INTERN ACTIONS --------------------------------*/	
	// GCM and Infobip registration actions
	private static final String GCM_REGISTRATION 				= Consts.GCM_REGISTRATION_ACTION;
	private static final String GCM_REGISTER 					= Consts.GCM_REGISTER_ACTION;
	private static final String GCM_UNREGISTER 					= Consts.GCM_UNREGISTER_ACTION;
	private static final String GCM_RETRY 						= Consts.GCM_RETRY_ACTION;
	private static final String INFOBIP_REGISTER 				= Consts.INFOBIP_REGISTER_ACTION;
	private static final String INFOBIP_REFRESH_REGISTRATION 	= Consts.INFOBIP_REFRESH_REGISTRATION_ACTION;
	private static final String INFOBIP_UNREGISTER 				= Consts.INFOBIP_UNREGISTER_ACTION;
	
	// notification actions
	private static final String GCM_NOTIFICATION_RECEIVED 		= Consts.GCM_NOTIFICATION_RECEIVED;
	private static final String NOTIFICATION_OPENED 			= Consts.NOTIFICATION_OPENED_ACTION;
	/*------------------------------- INTERN ACTIONS --------------------------------*/	
	
	
	private static String registrationOperation = "";	// operations: register, refresh registration, unregister 
	
	private static final int MAX_RETRY_COUNT = 3;
	private static int retryCount = 0;
	
	private static final String TOKEN = Long.toBinaryString(new Random().nextLong());
	private static final Object LOCK = InfobipPushService.class;
	private static PowerManager.WakeLock sWakeLock;

	private Prefs prefs;
	
	public InfobipPushService() {
		super("InfobipPushService");
	}
	
	public static void runIntentInService(Context context, Intent intent) {
		acquireLock(context);
		intent.setClassName(context, InfobipPushService.class.getName());
		context.startService(intent);
	}
	    
	private static void acquireLock(Context context) {		
		synchronized(LOCK) {
			if (sWakeLock == null) {
				PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
				sWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, Consts.WAKE_LOCK_PUSH_INTENT_SERVICE);
			}
		}
		if (!sWakeLock.isHeld()) {
			sWakeLock.acquire();
			Util.LogVerbose("Aquiring wakelock");
		}
	}
	
	@Override
	public final void onHandleIntent(Intent intent) {
		try {
			String action = intent.getAction();
			prefs = new Prefs(getApplicationContext());
			
			// register to the Infobip Push service or refresh registration
			if (INFOBIP_REGISTER.equals(action) || INFOBIP_REFRESH_REGISTRATION.equals(action)) {
				resetRetryCount();
				registrationOperation = action;

				String senderId = prefs.getSenderId();
				registerToGcm(senderId);
			}
			
			// unregister from the Infobip Push service
			else if (INFOBIP_UNREGISTER.equals(action)) {
				resetRetryCount();
				registrationOperation = INFOBIP_UNREGISTER;

				unregisterFromInfobipPush(getApplicationContext());
			}
			
			// retry GCM registration or unregistration
			else if (GCM_RETRY.equals(action)) {
				
				if (registrationOperation.equals(INFOBIP_REGISTER) || registrationOperation.equals(INFOBIP_REFRESH_REGISTRATION)) {
					String senderId = prefs.getSenderId();
					registerToGcm(senderId);
				
				} else if (registrationOperation.equals(INFOBIP_UNREGISTER)) {
					unregisterFromGcm();
				}
			}
				
			// handle GCM registration
			else if (GCM_REGISTRATION.equals(action)) {				
				handleGcmRegistration(intent);
			}
				
			// handle message received from GCM
			else if (GCM_NOTIFICATION_RECEIVED.equals(action)) {				
				handleGcmMessage(intent);
			} 
			
			// notify notification opened
			else if (NOTIFICATION_OPENED.equals(action)) {
				String pushId = intent.getStringExtra(Consts.PUSH_MESSAGE_ID);
				notifyNotificationOpened(pushId);
			}
			
		} finally {
            // Release the power lock, so phone can get back to sleep. The lock is reference-counted by default, so multiple
            // messages are ok.
			synchronized(LOCK) {
				// sanity check for null as this is a public method
				if (sWakeLock != null && sWakeLock.isHeld()) {
                    Util.LogVerbose("Releasing wakelock");
                    sWakeLock.release();
                } else {
                    // should never happen during normal workflow
                    Util.LogError("Wakelock reference is null");
                }
			}
		}
	}
	
	/**
	 * Resets retry count to zero.
	 */
	private void resetRetryCount() {
		retryCount = 0;
	}
	
	/**
	 * Tries to register to GCM with obtained sender id max retry number of times. 
	 * @see #onHandleIntent(Intent)
	 * @see #retryGcmRegistration()
	 * 
	 */
	private void registerToGcm(String senderId) {		
		Util.LogDebug("GCM register try #" + String.valueOf(retryCount + 1));
		Intent localIntent = new Intent(GCM_REGISTER);
		localIntent.putExtra("app", PendingIntent.getBroadcast(getApplicationContext(), 0, new Intent(), 0));
		localIntent.putExtra("sender", senderId);
		getApplicationContext().startService(localIntent);
		retryCount++;
	}

	/**
	 * Tries to unregister from GCM max retry number of times.
	 * @see #onHandleIntent(Intent)
	 * @see #retryGcmRegistration()
	 * @see #registerForInfobipPush(Context, String, String)
	 */
	private void unregisterFromGcm() {		
		Util.LogDebug("GCM unregister try #" + String.valueOf(retryCount + 1));
		Intent localIntent = new Intent(GCM_UNREGISTER);
		localIntent.putExtra("app", PendingIntent.getBroadcast(getApplicationContext(), 0, new Intent(), 0));
		getApplicationContext().startService(localIntent);
		retryCount++;
	}

	/**
	 * Handles GCM registration intent. Depending on the received values, method calls {@link #onGcmRegistered(String)}, 
	 * {@link #onGcmUnregistered()} or {@link #onGcmError(String)}.
	 */
	private void handleGcmRegistration(Intent intent) {
		String registrationId = intent.getStringExtra("registration_id"); 
		String error = intent.getStringExtra("error");
		String unregistered = intent.getStringExtra("unregistered");		
		Util.LogDebug(String.format("registrationID: %s, error: %s, unregistered: %s", registrationId, error, unregistered));
		
		if (registrationId != null) {
			onGcmRegistered(registrationId);
		}
		if (unregistered != null) {
			onGcmUnregistered();
		}
		if (error != null) {
			onGcmError(error);
		}
	}
	
	/**
	 * Saves registration id and application version to shared preferences, and calls {@link #registerForInfobipPush(Context, String, String)}
	 * if users phone is connected to the Internet.
	 * @see #handleGcmRegistration(Intent)
	 * @see Prefs
	 */
	private void onGcmRegistered(String newRegId) {		
		String currentRegId = prefs.getRegistrationId();
		
		if (INFOBIP_REFRESH_REGISTRATION.equals(registrationOperation) && !TextUtils.isEmpty(currentRegId)) {
			onRefreshingRegistration(getApplicationContext(), currentRegId, newRegId);
			return;
		}
		
		try {
			int timeZoneOffset = prefs.getTimeZoneOffset();
			
			if (timeZoneOffset != Consts.TIME_ZONE_OFFSET_UNSET) {
				prefs.setTimeZoneOffset(timeZoneOffset);
			}
			prefs.setRegistrationId(newRegId);
			prefs.setApplicationVersion(Util.getAppVersion(getApplicationContext()));
			prefs.save();
			
			registerForInfobipPush(getApplicationContext());
			
		} catch (JSONException e) {
			Util.LogError(e.toString() + " Registration failed.");			
			onInfobipRegistrationFailure(SendPostMessage.OPERATION_FAILED);
		}		
	}
	
	/**
	 * Sends POST request with <code>RegistrationModel</code> data to register to Infobip Push service. Invoked on {@link #onGcmRegistered(String)}
	 * @throws JSONException
	 * @see RegistrationModel
	 * @see SendPostMessage
	 */
	private void registerForInfobipPush(Context context) throws JSONException {
		Util.LogDebug("Library proccesses registration request");
		
		RegistrationModel registrationModel = prefs.prefsToRegistrationModel();
		registrationModel.mcc = Util.getMcc(context);
		registrationModel.mnc = Util.getMnc(context);
		registrationModel.operatorName = Util.getNetworkOperatorName(context);
		
		SendPostMessage sendPost = new SendPostMessage(Consts.URL_REGISTRATION, registrationModel.toJsonObject());
		int operationResponse = sendPost.run(context);
		
		switch (operationResponse) {
		case SendPostMessage.OPERATION_COMPLETED:
			onInfobipRegistrationSuccess(sendPost.getResponse());
			break;
			
		default:
			onInfobipRegistrationFailure(operationResponse);
			break;
		}
	}
	
	private void onInfobipRegistrationSuccess(String response) {
		Util.LogDebug("User is registered.");
		Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_REGISTERED_ACTION, -1);
		
		restartLocationAlarms(response);
	}
	
	private void onInfobipRegistrationFailure(int operationResponse) {
		Util.LogError("Registration failed.");
		
		resetRetryCount();
		prefs.removeRegistrationId();
		unregisterFromGcm();
		Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, operationResponse);
	}
	
	private void onRefreshingRegistration(Context context, String currentRegId, String newRegId) {
		
		if (currentRegId.equals(newRegId)) {
			Util.LogDebug("Registration is refreshed.");			
			prefs.setApplicationVersion(Util.getAppVersion(getApplicationContext()));
			prefs.save();
			
			Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_REGISTRATION_REFRESHED_ACTION, -1);
			return;
		}
		
		try {
			refreshInfobipPushRegistration(context, newRegId, currentRegId);
		} catch (JSONException e) {
			Util.LogError(e.toString() + " Refreshing registration failed.");
		}	
	}
	
	/**
	 * Sends POST request with <code>RegistrationModel</code> data to register to Infobip Push service. Invoked on {@link #onGcmRegistered(String)}
	 * @throws JSONException
	 * @see RegistrationModel
	 * @see SendPostMessage
	 */
	private void refreshInfobipPushRegistration(Context context, String newRegId, String currentRegId) throws JSONException {
		Util.LogDebug("Library proccesses registration request");	
		
		UserDataModel userDataModel = new UserDataModel();
		userDataModel.phoneRegistrationId = newRegId;
		
		SendPostMessage sendPost = new SendPostMessage(String.format(Consts.URL_SAVE_NEW_USER_DATA, prefs.getDeviceId()), 
				userDataModel.toJsonObject(), Consts.PUT);
		int operationResponse = sendPost.run(context);
		
		switch (operationResponse) {
		case SendPostMessage.OPERATION_COMPLETED:
			onRefreshRegistrationSuccess(sendPost.getResponse(), newRegId);
			break;
			
		default:
			Util.LogError("Refreshing registration failed.");
			break;
		}
	}
	
	private void onRefreshRegistrationSuccess(String response, String newRegId) {	
		restartLocationAlarms(response);

		Util.LogDebug("Registration is refreshed.");
		prefs.setRegistrationId(newRegId);
		prefs.setApplicationVersion(Util.getAppVersion(getApplicationContext()));
		prefs.save();
		
		Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_REGISTRATION_REFRESHED_ACTION, -1);
	}

	private void restartLocationAlarms(String response) {
			
		if (prefs.isLocationEnabled()) {
			Util.LogDebug("Rescheduling location updates");
			LocationManagerUtil.getInstance(getApplicationContext()).rescheduleUpdateAlarmIfNecessary(prefs);
		}
	
		if (prefs.isLiveGeoEnabled()) {
			LocationManagerUtil.getInstance(getApplicationContext()).startRequestingLiveGeosAlarm();
			
			JSONObject emptyResponse = new JSONObject();			
			if (!response.trim().equals(emptyResponse.toString())) {
				
				if (LiveGeoUtil.gPlayServicesConnected(getApplicationContext())) {
					Intent geoIntent = new Intent(LiveGeoUtil.ADD_AREAS_ACTION);
					geoIntent.putExtra(LiveGeoUtil.KEY_LIVE_GEOS_EXTRA, response);
					LiveGeoIntentService.runIntentInService(getApplicationContext(), geoIntent);
				}
			}
		}
	}

	/**
	 * Calls {@link #unregisterFromInfobipPush(Context)} if users phone is connected to the Internet.
	 * @see #handleGcmRegistration(Intent)
	 */
	private void onGcmUnregistered() {
		Util.LogDebug("Unregistered from GCM.");
	}

	/**
	 * Called when last operation (registration or unregistration) returned an error. Method calls {@link #retryGcmRegistration()} 
	 * if service isn't available and sends intent with reason as an additional data defining why other errors occurred.
	 * If the GCM service is not available 3 times, {@link #onGcmServiceNotAvailable()} is called.
	 * @see #handleGcmRegistration(Intent)
	 */
	private void onGcmError(String error) {
		
		if (Consts.ERROR_SERVICE_NOT_AVAILABLE.equals(error)) {
			
			if (retryCount < MAX_RETRY_COUNT) {
				retryGcmRegistration();
			} else {
				onGcmServiceNotAvailable();
			}
			
		} else if (Consts.ERROR_ACCOUNT_MISSING.equals(error)) {
			Util.LogError(String.format("Error: %s. Please use Android emulator with Google APIs.", error));
			
			if (INFOBIP_REGISTER.equals(registrationOperation)) {
				Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, PushNotificationManager.GCM_ACCOUNT_MISSING);
			}
		
		} else if (Consts.ERROR_AUTHENTICATION_FAILED.equals(error)) {
			Util.LogError(String.format("Error: %s. Your Google account password is invalid. Please fix Google account settings.", error));
			
			if (INFOBIP_REGISTER.equals(registrationOperation)) {
				Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, PushNotificationManager.GCM_AUTHENTICATION_FAILED);
			}
		
		} else if (Consts.ERROR_INVALID_SENDER.equals(error)) {
			Util.LogError(String.format("Error: %s. Your GCM sender ID is invalid. Please check " +
	    			"http://developer.android.com/guide/google/gcm/gs.html for instructions how to obtain sender ID.", error));
			
			if (INFOBIP_REGISTER.equals(registrationOperation)) {
				Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, PushNotificationManager.GCM_INVALID_SENDER);
			}
		
		} else if (Consts.ERROR_PHONE_REGISTRATION_ERROR.equals(error)) {
			Util.LogError(String.format("Error: %s. Sorry, your phone doesn't support GCM.", error));
			
			if (INFOBIP_REGISTER.equals(registrationOperation)) {
				Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, PushNotificationManager.GCM_PHONE_REGISTRATION_ERROR);
			}
		
		} else if (Consts.ERROR_INVALID_PARAMETERS.equals(error)) {
			Util.LogError(String.format("Error: %s. Sorry, your phone doesn't support GCM.", error));
			
			if (INFOBIP_REGISTER.equals(registrationOperation)) {
				Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, PushNotificationManager.GCM_INVALID_PARAMETERS);
			}
		}		
	}
	
	/**
	 * Retries to {@link #registerToGcm(String)} to or {@link #unregisterFromGcm()} from GCM with defined back-off time.
	 * @see #onGcmError(String)
	 */
	private void retryGcmRegistration() {
		// optionally retry using exponential back-off
		long backoffTimeMs = 3000;
		long nextAttempt = SystemClock.elapsedRealtime() + backoffTimeMs;
		Intent retryIntent = new Intent(GCM_RETRY);
		retryIntent.putExtra("token", TOKEN);
		retryIntent.setClass(getApplicationContext(), InfobipPushReceiver.class);
		PendingIntent retryPendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0, retryIntent, 0);
		AlarmManager am = (AlarmManager) getApplicationContext().getSystemService(Context.ALARM_SERVICE);
		am.set(AlarmManager.ELAPSED_REALTIME, nextAttempt, retryPendingIntent);
		backoffTimeMs *= 2;
	}
	
	/**
	 * Resets retry count number to zero for future usage and sends intent with reason why the error occurred (GCM service wasn't available).
	 * @see #registerToGcm(String)
	 * @see #unregisterFromGcm()
	 */
	private void onGcmServiceNotAvailable() {
		resetRetryCount();
		Util.LogError(String.format("Error: %s. GCM service is not available.", Consts.ERROR_SERVICE_NOT_AVAILABLE));
		
		if (INFOBIP_REGISTER.equals(registrationOperation)) {
			Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, PushNotificationManager.GCM_SERVICE_NOT_AVAILABLE);
		}
	}
	
	/**
	 * Sends POST request with <code>UnregistrationModel</code> data to unregister from Infobip Push service. 
	 * Invoked on {@link #onGcmUnregistered()}.
	 * @see UnregistrationModel
	 * @see SendPostMessage
	 */
	private void unregisterFromInfobipPush(Context context) {
		Util.LogDebug("Library proccesses unregistration request");
		
		UnregistrationModel unregistrationModel = prefs.prefsToUnregistrationModel();
		SendPostMessage sendPost = new SendPostMessage(Consts.URL_UNREGISTRATION, unregistrationModel.toJsonObject());
		int operationResponse = sendPost.run(context);
		
		switch (operationResponse) {
		case SendPostMessage.OPERATION_COMPLETED:
			onInfobipUnegistrationSuccess();
			break;
		
		default:		
			onInfobipUnregistrationFailure(operationResponse);
			break;
		}
	}

	private void onInfobipUnegistrationSuccess() {
		Util.LogDebug("User is unregistered.");
		
		prefs.removeRegistrationId();
		prefs.removeUserRelatedData();
		
		Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_UNREGISTERED_ACTION, -1);

		cancelLocationAlarms();
		
		unregisterFromGcm();
	}

	private void cancelLocationAlarms() {
		if (prefs.isLocationEnabled()) {
			LocationManagerUtil.getInstance(getApplicationContext()).cancelUpdateAlarm(prefs);
			Util.LogDebug("Location updates are canceled");
		}
		
		if (prefs.isLiveGeoEnabled()) {
			LocationManagerUtil.getInstance(getApplicationContext()).cancelRequestingLiveGeosAlarm();
			
			if (LiveGeoUtil.gPlayServicesConnected(getApplicationContext())) {
				LiveGeoUtil.removeLiveGeosFromPlayServices(getApplicationContext(), prefs.getActiveAreaIds());
			}
			LiveGeoUtil.removeAllLiveGeosFromPrefs(getApplicationContext(), prefs);
		}
	}

	private void onInfobipUnregistrationFailure(int operationResponse) {
		Util.LogError("Unregistrating user failed.");
		
		resetRetryCount();
		Util.sendCustomBroadcast(getApplicationContext(), Consts.RECEIVER_ERROR_ACTION, operationResponse);
	}

	/**
	 * Manages actions that are performed when the notification is received. <br />
	 * Calls {@link #getAndConfirmPushMessage(Intent)} for sending notification confirmation, and {@link #handleInfobipMessage(String, String)} for 
	 * sending notification received broadcast and displaying the notification.
	 */
	private void handleGcmMessage(Intent intent) {		
		String messageId = intent.getStringExtra(Consts.PUSH_MESSAGE_ID);
		String type = intent.getStringExtra(Consts.PUSH_TYPE);
		String locationValidityTime = intent.getStringExtra(Consts.PUSH_LOCATION_VALIDITY);
		
		if (TextUtils.isEmpty(messageId)) {
			Util.LogWarn("Missing \"" + Consts.PUSH_MESSAGE_ID + "\" parameter. Ignoring received notification...");
			return;
		}		
		
		if (TextUtils.isEmpty(type)) {
			onPushMessageReceived(messageId, locationValidityTime);
		
		} else if (Consts.TYPE_LIVE_GEO.equals(type)) {
			
			if (prefs.isLiveGeoEnabled() && LiveGeoUtil.gPlayServicesConnected(getApplicationContext())) {
				onLiveGeoMessageReceived(intent);
			}
			
		} else if (Consts.TYPE_LIVE_GEO_CANCEL.equals(type)) {
			
			if (prefs.isLiveGeoEnabled() && LiveGeoUtil.gPlayServicesConnected(getApplicationContext())) {
				onLiveGeoMessageCanceled(intent);
			}
			
		} else {
			Util.LogWarn("Unknown type: " + type + ". Ignoring received notification...");	
		}	
	}

	private void onPushMessageReceived(String messageId, String locationValidityTimeString) {
		Util.LogDebug("Application received push notification");
		
		if (isMessageSentOnLocation(locationValidityTimeString)) {
			long locationValidityTime = Long.valueOf(locationValidityTimeString);
			
			if (!isLocationStillValid(locationValidityTime)) {				
				String locationValidityDate = new SimpleDateFormat("yyyy.MM.dd 'at' HH:mm:ss", Locale.getDefault()).format(new Date(locationValidityTime));
				Util.LogWarn("Invalid location validity time. Location was valid until " + locationValidityDate + ". Ignoring received notification...");
				return;
			}
		}
		
		String message = getAndConfirmPushMessage(messageId);
		if (!TextUtils.isEmpty(message)) {
			InfobipNotificationHandlingUtil.handleInfobipMessage(getApplicationContext(), messageId, message);
			
		} else {
			Util.LogWarn("Received an empty message. Check JSON parameters sent via REST API. Ignoring received notification...");
		}
	}
	
	private boolean isMessageSentOnLocation(String locationValidityTime) {
		return !TextUtils.isEmpty(locationValidityTime);
	}

	private boolean isLocationStillValid(long locationValidityTime) {
		return locationValidityTime >= System.currentTimeMillis();
	}

	/**
	 * Gets JSON message (that defines {@link PushNotification} object) from the Infobip Push service and confirms the notification receival.
	 * @return Message received from Infobip in JSON string format. Empty string will be returned if the message wasn't obtained.
	 */
	private String getAndConfirmPushMessage(String pushId) {
		Util.LogDebug("Confirming push notification arrival and getting message from Infobip Push service");
		
		String deviceId = prefs.getDeviceId();
		String urlMessage = String.format(Consts.URL_MESSAGE, pushId, deviceId);
		
		SendPostMessage sendPost = new SendPostMessage(urlMessage, SendPostMessage.GET);
		int operationResponse = sendPost.run(getApplicationContext());
		
		String messageJsonString = null;
		
		switch (operationResponse) {
		case SendPostMessage.OPERATION_COMPLETED:
			Util.LogDebug("Confirmation sent");
			messageJsonString = sendPost.getResponse();			
			break;
		
		default:
			Util.LogError("Sending confirmation and getting message failed.");
			break;
		}
		
		return messageJsonString;
	}

	private void onLiveGeoMessageReceived(Intent intent) {
		Util.LogDebug("Application received live geo message");		
		String event = intent.getStringExtra(LiveGeoUtil.KEY_AREA_EVENT);
		String expiry = intent.getStringExtra(LiveGeoUtil.KEY_AREA_EXPIRY);
		String area = intent.getStringExtra(LiveGeoUtil.KEY_AREA);
		
		if (!TextUtils.isEmpty(event) && !TextUtils.isEmpty(expiry) && !TextUtils.isEmpty(area)) {
			Intent geoIntent = new Intent(LiveGeoUtil.ADD_AREA_ACTION);
			geoIntent.putExtras(intent);
			LiveGeoIntentService.runIntentInService(getApplicationContext(), geoIntent);
		
		} else {
			Util.LogWarn("Received live geo notification is missing one of the fields: event, expiry, area. Ignoring received notification...");
		}
	}

	private void onLiveGeoMessageCanceled(Intent intent) {
		Util.LogDebug("Application canceling live geo message");
		Intent geoIntent = new Intent(LiveGeoUtil.REMOVE_AREA_ACTION);
		geoIntent.putExtras(intent);
		LiveGeoIntentService.runIntentInService(getApplicationContext(), geoIntent);
	}
	
	/**
	 * Notifies Infobip Push service about opened notification.
	 * @param messageId Unique identifier of the opened notification.
	 */
	private void notifyNotificationOpened(String messageId) {
		Util.LogDebug("Notification opened event forwarded to the server");
		
		NotificationOpenedModel notificationOpenedModel = new NotificationOpenedModel();
		notificationOpenedModel.deviceId = prefs.getDeviceId();
		notificationOpenedModel.messageId = messageId;
		
		SendPostMessage sendPost = new SendPostMessage(Consts.URL_NOTIFICATION_OPENED, notificationOpenedModel.toJsonObject(), SendPostMessage.PUT);
		int response = sendPost.run(getApplicationContext());
		
		switch (response) {
		case SendPostMessage.OPERATION_COMPLETED:
			Util.LogDebug("User opened notification.");
			break;
			
		default:
			Util.LogError("Sending notification opened request failed.");
			break;
		}
	}
	
}