package com.infobip.push.lib;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.location.LocationManager;
import android.text.TextUtils;

import com.google.android.gms.location.LocationClient;
import com.infobip.push.PushNotification;
import com.infobip.push.lib.livegeo.ReceiveTransitionsIntentService;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.LocationManagerUtil;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;

/**
 * <p>
 * Internal library receiver invoked on default action when notification is opened. It sends notification opened broadcast to notify developer when the notification is opened. Supports only
 * text/plain and text/html MIME types.
 * </p>
 * 
 * <p>
 * Also, It can be invoked from {@link LocationManagerUtil#startUpdateAlarm(Prefs)} method which starts location updates with {@link Consts#LOCATION_RETRIEVING_ACTION} or
 * {@link Consts#LOCATION_SENDING_ACTION} actions. Each time receiver is called, it checks if the location services should run and cancels updates if they shouldn't.
 * </p>
 * <p>
 * When {@link Consts#LOCATION_RETRIEVING_ACTION} action invokes, {@link LocationService} starts acquiring location. On {@link Consts#LOCATION_SENDING_ACTION} action, receiver checks if the new location
 * is available and starts {@link LocationUpdateService} if it is.
 * </p>
 * 
 */
public class InternalReceiver extends BroadcastReceiver {

	@Override
	public void onReceive(Context context, Intent intent) {
		String action = intent.getAction();
		
		if (TextUtils.isEmpty(action)) {
			return;
		}
		
		if (action.startsWith(Consts.NOTIFICATION_OPENED_ACTION)) {
			InfobipNotificationHandlingUtil.sendNotificationOpenedBroadcast(context, intent);
			
			PushNotification notification = (PushNotification) InfobipNotificationHandlingUtil.fetchParcelledNotificationFromIntent(intent);
			notifyNotificationOpened(notification.getId(), context);

		} else if (Consts.LOCATION_RETRIEVING_ACTION.equals(action) || Consts.LOCATION_SENDING_ACTION.equals(action)) {
			Prefs prefs = new Prefs(context);

			if (LocationManagerUtil.getInstance(context).shouldLocationUpdatesRun(prefs)) {
				double latitude = Double.valueOf(prefs.getLocationLatitude());
				double longitude = Double.valueOf(prefs.getLocationLongitude());
				long timeSaved = prefs.getLocationTimeSaved();
				long updateInterval = prefs.getLocationUpdateTimeInterval();
				long timeSent = prefs.getLocationTimeSent();
				String deviceId = prefs.getDeviceId();

				if (isLocationAlreadyUpdated(timeSent, updateInterval)) {
					Util.LogWarn("Location is already updated. Stopping current location update.");
					return;
				}

				if (Consts.LOCATION_RETRIEVING_ACTION.equals(action)) {
					LocationManager locMgr = (LocationManager) context.getSystemService(Context.LOCATION_SERVICE);
					
					if (!locMgr.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && !locMgr.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
						Util.LogDebug("All location providers are disabled. Next location update is scheduled in " + updateInterval + " minutes.");
						return;
					}
					
					intent.setClass(context, LocationService.class);
					context.startService(intent);
				
				} else if (Consts.LOCATION_SENDING_ACTION.equals(action)) {

					if (!isNewLocationAvailable(latitude, longitude, timeSaved, updateInterval)) {
						Util.LogWarn("There is no new location to be saved. Stopping current location update.");
						return;
					}
					intent.putExtra("lat", latitude);
					intent.putExtra("long", longitude);
					intent.putExtra("deviceId", deviceId);
					LocationUpdateService.runIntentInService(context, intent);
				}

			} else {
				LocationManagerUtil.getInstance(context).cancelUpdateAlarm(prefs);
				Util.LogDebug("User is unregistered from Push or location is disabled, canceling location updates.");
			}

		} else if (Consts.LIVE_GEO_TRANSITION_ACTION.equals(action)) {
			Prefs prefs = new Prefs(context);
			
			if (!TextUtils.isEmpty(prefs.getRegistrationId())) {
				if (!LocationClient.hasError(intent)) {
					ReceiveTransitionsIntentService.runIntentInService(context, intent);
				}
			}
				
		}
	}

	/**
	 * Starts InfobipPushService to send notification opened analytics for a given message ID.
	 */
	private void notifyNotificationOpened(String messageId, Context context) {
		Intent opened = new Intent(Consts.NOTIFICATION_OPENED_ACTION);
		opened.putExtra(Consts.PUSH_MESSAGE_ID, messageId);
		InfobipPushService.runIntentInService(context, opened); // call service class with com.push.NOTIFICATION_OPENED action
	}

	private boolean isNewLocationAvailable(double latitude, double longitude, long timeSaved, long updateInterval) {

		if (latitude != 0 && longitude != 0 && timeSaved != 0) {
			long lastLocationSavedInterval = System.currentTimeMillis() - timeSaved;
			long locationUpdateTimeInterval = updateInterval * 60 * 1000;

			if (lastLocationSavedInterval < locationUpdateTimeInterval) {
				return true;
			}
		}
		return false;
	}

	private boolean isLocationAlreadyUpdated(long timeSent, long updateInterval) {

		if (timeSent != 0) {
			long now = System.currentTimeMillis();
			long lastLocationSentInterval = now - (timeSent - (60 * 1000)); //minus 60000 millis for not monitoring the last minute
			long updateIntervalMillis = updateInterval * 60 * 1000;

			if (lastLocationSentInterval < updateIntervalMillis) {
				return true;
			}
		}
		return false;
	}

}
