package com.infobip.push.lib;

import java.util.List;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.PowerManager;

import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;
import com.infobip.push.lib.util.WakefulThread;

/**
 * Service providing the guts of the location polling engine. Uses a WakeLock to
 * ensure the CPU stays on while the location lookup is going on. Handles both
 * successful and timeout conditions.
 * 
 * Those wishing to leverage this service should do so via the LocationReceiver class.
 */
public class LocationService extends Service {
	
	private static final String LOCK_NAME_STATIC = "com.infobip.push.lib.LocationService";
	private static final int DEFAULT_TIMEOUT = Consts.DEFAULT_LOCATION_POLLING_TIMEOUT_MILLIS;
	private static volatile PowerManager.WakeLock lockStatic = null;	
	
	private LocationManager locManager = null;
	

	/**
	 * Lazy-initializes the WakeLock when we first use it. We use a partial
	 * WakeLock since we only need the CPU on, not the screen.
	 */
	synchronized private static PowerManager.WakeLock getLock(Context context) {
		if (lockStatic == null) {
			PowerManager pm = (PowerManager) context.getApplicationContext().getSystemService(Context.POWER_SERVICE);
			
			lockStatic = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, LOCK_NAME_STATIC);
			lockStatic.setReferenceCounted(true);
		}
		return (lockStatic);
	}

	/**
	 * Obtain the LocationManager on startup
	 */
	@Override
	public void onCreate() {
		locManager = (LocationManager) getSystemService(LOCATION_SERVICE);
	}

	/**
	 * No-op implementation as required by superclass
	 */
	@Override
	public IBinder onBind(Intent i) {
		return (null);
	}

	/**
	 * Validates the required extra (EXTRA_INTENT). If valid, updates the Intent to be broadcast with the application's own
	 * package (required to keep the broadcast within this application, so we do not leak security information). Then, forks a 
	 * PollerThread to do the actual location lookup.
	 * 
	 * @return START_REDELIVER_INTENT to ensure we get the last request again
	 */
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		PowerManager.WakeLock sWakeLock = getLock(getApplicationContext());

		if (!Consts.LOCATION_RETRIEVING_ACTION.equals(intent.getAction())) {
			Util.LogWarn("Received wrong intent");
			stopSelf();
		}
		
		if (!sWakeLock.isHeld() || (flags & START_FLAG_REDELIVERY) != 0) {
			sWakeLock.acquire();
		}

		new PollerThread(sWakeLock, locManager).start();

		return (START_REDELIVER_INTENT);
	}

	/**
	 * A WakefulThread subclass that knows how to look up the current location, plus handle the timeout scenario.
	 */
	private class PollerThread extends WakefulThread {
		
		LocationManager locManager = null;
		Runnable onTimeout = null;
		Handler handler = new Handler();
		
		LocationListener listener = new LocationListener() {
			
			/**
			 * If the location changes, get rid of the timeout condition, save the location to {@link Prefs}, broadcast intent with
			 * {@link LocationBootReceiver#LOCATION_SENDING_ACTION} action, then exit the polling loop so the thread terminates.
			 */
			@Override
			public void onLocationChanged(Location location) {
				onLocationAcquired(location);
			}

			@Override
			public void onProviderDisabled(String provider) {
				
				if (!locManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && !locManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
					handler.removeCallbacks(onTimeout);
					
					Util.LogDebug("Quitting location updates, no location acquired.");
					quit();
				}
				
				if (LocationManager.GPS_PROVIDER.equals(provider)) {

					if (locManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
						locManager.removeUpdates(listener);
						
						Util.LogVerbose("GPS provider disabled, requesting updates with network provider");
						locManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, listener);
					}
				}	
			}

			@Override
			public void onProviderEnabled(String provider) {
				locManager.removeUpdates(listener);
				
				Util.LogVerbose("Requesting updates with " + provider + " provider");
				locManager.requestLocationUpdates(provider, 0, 0, listener);
			} 

			@Override
			public void onStatusChanged(String provider, int status, Bundle extras) { } // not used
		};

		/**
		 * Constructor.
		 * 
		 * @param lock Already-locked WakeLock
		 * @param locManager LocationManager for doing the location lookup
		 */
		PollerThread(PowerManager.WakeLock lock, LocationManager locManager) {
			super(lock, "LocationService-PollerThread");

			this.locManager = locManager;
		}

		/**
		 * Called before the Handler loop begins. When a timeout occurs, log the error, then terminate the thread. Also,
		 * requests a location update from the LocationManager.
		 */
		@Override
		protected void onPreExecute() {
			Util.LogVerbose("Getting last known location for available providers.");			
			List<String> matchingProviders = locManager.getProviders(false);
			
			for (String provider : matchingProviders) {
				Location location = locManager.getLastKnownLocation(provider);
				
				if (location != null) {
					long time = location.getTime();
					
					if (isLocationValid(time)) {
						onLocationAcquired(location);
						return;
					}
				}
			}

			onTimeout = new Runnable() {
				
				@Override
				public void run() {
					Util.LogDebug("No location retrieved: Timeout!");
					quit();
				}
			};

			handler.postDelayed(onTimeout, DEFAULT_TIMEOUT);


			try {
				Util.LogVerbose("Requesting location updates");
				
				if (locManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
					locManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, listener);
				}
				
				if (locManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
					locManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, listener);
				}
				
			} catch (IllegalArgumentException e) {
				// see http://code.google.com/p/android/issues/detail?id=21237
				Util.LogError("Exception requesting updates - may be emulator issue: " + e, LocationService.class);
				quit();
			}
		}

		private boolean isLocationValid(long time) {
			return System.currentTimeMillis() - time < Consts.VALID_LOCATION_MILLIS;
		}
		
		/**
		 * Called when the Handler loop ends. Removes the location listener.
		 */
		@Override
		protected void onPostExecute() {
			locManager.removeUpdates(listener);
			super.onPostExecute();
		}

		/**
		 * Called when the WakeLock is completely unlocked. Stops the service, so everything shuts down.
		 */
		@Override
		protected void onUnlocked() {
			stopSelf();
		}

		private void onLocationAcquired(Location location) {
			Util.LogDebug("Location acquired with provider: " + location.getProvider());
			
			handler.removeCallbacks(onTimeout);
			
	        Prefs prefs = new Prefs(getApplicationContext());
			saveLocationToPrefs(prefs, location);			
			
			Intent toBroadcast = new Intent(getApplicationContext(), InternalReceiver.class);
			toBroadcast.setAction(Consts.LOCATION_SENDING_ACTION);
			getApplicationContext().sendBroadcast(toBroadcast);
			quit();
		}
	}
	
	/**
	 * Saves obtained location to shared preferences with data about latitude, longitude and time saved.
	 * @param context
	 * @param location
	 */
	private void saveLocationToPrefs(Prefs prefs, Location location) {
        prefs.setLocationLatitude(String.valueOf(location.getLatitude()));
        prefs.setLocationLongitude(String.valueOf(location.getLongitude()));
        prefs.setLocationTimeSaved(location.getTime());
        prefs.save();
	}
	
}
