package com.infobip.push.lib;

import java.util.Locale;

import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.os.PowerManager;
import android.text.TextUtils;

import com.infobip.push.lib.model.LocationModel;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;

public class LocationUpdateService  extends IntentService {
	
	private static final Object LOCK = LocationUpdateService.class;
	private static PowerManager.WakeLock sWakeLock;
	
	public LocationUpdateService() {
		super("LocationUpdateService");
	}
	
	public static void runIntentInService(Context context, Intent intent) {
		acquireLock(context);
		intent.setClassName(context, LocationUpdateService.class.getName());
		context.startService(intent);
	}
	    
	private static void acquireLock(Context context) {		
		synchronized(LOCK) {
			if (sWakeLock == null) {
				PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
				sWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, Consts.WAKE_LOCK_LOCATION_INTENT_SERVICE);
			}
		}
		if (!sWakeLock.isHeld()) {
			sWakeLock.acquire();
			Util.LogVerbose("Aquiring wakelock");
		}
	}
	
	@Override
	public final void onHandleIntent(Intent intent) {
		try {
			String action = intent.getAction();
			
			if (Consts.LOCATION_SENDING_ACTION.equals(action)) {
				double latitude = intent.getDoubleExtra("lat", 0);
				double longitude = intent.getDoubleExtra("long", 0);
				String deviceId = intent.getStringExtra("deviceId");
				
				if (latitude != 0 && longitude != 0 && !TextUtils.isEmpty(deviceId)) {
					sendLocationUpdate(latitude, longitude, deviceId);
				}
			}
			
		} finally {
            // Release the power lock, so phone can get back to sleep. The lock is reference-counted by default, so multiple
            // messages are ok.
			synchronized(LOCK) {
				// sanity check for null as this is a public method
				if (sWakeLock != null && sWakeLock.isHeld()) {
                    Util.LogVerbose("Releasing wakelock");
                    sWakeLock.release();
                } else {
                    // should never happen during normal workflow
                    Util.LogError("Wakelock reference is null");
                }
			}
		}
	}
	
	private void sendLocationUpdate(double latitude, double longitude, String deviceId) {			
		Util.LogDebug(String.format(Locale.US, "Sending location (%s, %s) to the Infobip Push service", latitude, longitude));
		
		LocationModel locationModel = new LocationModel();
		locationModel.latitude = latitude;
		locationModel.longitude = longitude;
		locationModel.deviceId = deviceId;
		
		SendPostMessage message = new SendPostMessage(Consts.URL_LOCATION, locationModel.toJsonObject(), SendPostMessage.POST);
		int operationResponse = message.run(getApplicationContext());
		
		switch (operationResponse) {
		case SendPostMessage.OPERATION_COMPLETED:
			Prefs prefs = new Prefs(getApplicationContext());
			prefs.setLocationTimeSent(System.currentTimeMillis());
			prefs.save();
			
			Util.LogDebug("Location sent");	
			break;
		
		default:
			Util.LogError("Sending location failed.");
			break;
		}
	}

}
