package com.infobip.push.lib;

import org.json.JSONObject;

import android.content.Context;
import android.os.Handler;
import android.os.Message;
import android.util.Base64;

import com.infobip.push.PushNotificationManager;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;


public class SendPostMessage {
	
	public static final int GET 	= Consts.GET; 	// 1
	public static final int POST 	= Consts.POST; 	// 2
	public static final int PUT 	= Consts.PUT; 	// 3
	
	private static final int MAX_RETRY_COUNT = 3;
	private static final int SEND_REQUEST_WAIT_SECONDS = 1;
	
	public static final int OPERATION_COMPLETED = 11;
	public static final int OPERATION_FAILED 	= 12;
	public static final int OPERATION_RETRY 	= 13;
	public static final int ERROR_PUSH_SERVICE_NOT_AVAILABLE = PushNotificationManager.PUSH_SERVICE_NOT_AVAILABLE;
	public static final int ERROR_BAD_REQUEST 			= Consts.ERROR_BAD_REQUEST; 	// 400
	public static final int ERROR_UNAUTHORIZED 			= Consts.ERROR_UNAUTHORIZED; 	// 401
	public static final int ERROR_FORBIDDEN 			= Consts.ERROR_FORBIDDEN; 	// 403
	public static final int ERROR_NOT_FOUND 			= Consts.ERROR_NOT_FOUND; 		// 404
	public static final int ERROR_METHOD_NOT_ALLOWED 	= Consts.ERROR_METHOD_NOT_ALLOWED; 	// 405
	public static final int ERROR_GONE 					= Consts.ERROR_GONE; 	// 410
	public static final int ERROR_INTERNAL_SERVER_ERROR = Consts.ERROR_INTERNAL_SERVER_ERROR; 	// 500
	
	private InfobipRestClient rest;
	private JSONObject modelObject; 		// Registration, Unregistration, NotificationOpened, ChannelRegistration or Location model
	private int method = POST;
	
	private int backoff = SEND_REQUEST_WAIT_SECONDS * 1000;
	
	public SendPostMessage(String url, int method) {
		this.method = method;
		this.rest = new InfobipRestClient(url);
	}
	
	public SendPostMessage(String url, JSONObject modelObject) {
		this.modelObject = modelObject;
		this.rest = new InfobipRestClient(url);
	}
	
	public SendPostMessage(String url, JSONObject modelObject, int method) {
		this.modelObject = modelObject;
		this.method = method;
		this.rest = new InfobipRestClient(url);
	}
	
	/**
	 * Try to successfully send post request to Infobip's push server {@link #MAX_RETRY_COUNT} number of times with delay of
	 * {@link #SEND_REQUEST_WAIT_SECONDS}.
	 * 
	 */
	public int run(Context context) {
		
		addHeaders(context);
		
		for (int i = 0; i < MAX_RETRY_COUNT; ++i) {			
			Util.LogDebug("Performing request try #" + String.valueOf(i + 1) + " (of " + MAX_RETRY_COUNT + ")");
		
			try {
				int operationSuccess = OPERATION_RETRY;
				
				if (Util.isNetworkConnected(context)) {				
					operationSuccess = exec(modelObject);
				}

				// Retry operation or return message 
				if (operationSuccess != OPERATION_RETRY) {
					return operationSuccess;
				}
				
				if (MAX_RETRY_COUNT > (i + 1)) {
					// Wait, then try again
					try {
						Util.LogDebug("Waiting " + backoff / 1000 + " seconds until retry...");
						Thread.sleep(backoff);
					} catch (InterruptedException e) {
						Util.LogError("Thread interrupted. Received message 'OPERATION_FAILED'", SendPostMessage.this);
						return OPERATION_FAILED;
					}
					
					backoff *= 2;
				}
				
			} catch (Exception e) {
				Util.LogError(e.toString(), SendPostMessage.this);
				return OPERATION_FAILED;
			}

		}
		//TODO check whether PUSH_SERVICE_NOT_AVAILABLE error should be returned or OPERATION_FAILED
		Util.LogDebug("Received message 'PUSH_SERVICE_NOT_AVAILABLE' - Infobip Push Service is not available.", SendPostMessage.this);
		return ERROR_PUSH_SERVICE_NOT_AVAILABLE;
	}
	
	private void addHeaders(Context context) {
		Prefs prefs = new Prefs(context);
		String appIdAppSecret = prefs.getApplicationId() + ":" + prefs.getApplicationSecret();
		String appIdAppSecretBase64 = Base64.encodeToString(appIdAppSecret.getBytes(), Base64.NO_WRAP);

		// Adding header with hashed app id and secret
		rest.addHeader("Push-Authorization", "Basic " + appIdAppSecretBase64);
	}

	private int exec(JSONObject modelObject) throws Exception {

		switch (method) {
		case GET:
			rest.execute(GET);
			break;
	
		case POST:
			if (modelObject != null) {
				rest.addParams(modelObject);
			}
			rest.execute(POST);
			break;
			
		case PUT:
			if (modelObject != null) {
				rest.addParams(modelObject);
			}
			rest.execute(PUT);
			break;
		}
	
		// Check whether the operation succeeded by switching the response code
		int responseCode = getResponseCode();
		switch (responseCode) {
			case 200:
			case 201:
			case 202:
			case 204:
				Util.LogDebug("Received message 'OPERATION_COMPLETED'", SendPostMessage.this);
				return OPERATION_COMPLETED;
			case 400:
				Util.LogDebug("Received message 'BAD_REQUEST'", SendPostMessage.this);
				return ERROR_BAD_REQUEST;
			case 401:
				Util.LogDebug("Received message 'UNAUTHORIZED'", SendPostMessage.this);
				return ERROR_UNAUTHORIZED;
			case 403:
				Util.LogDebug("Received message 'FORBIDDEN'", SendPostMessage.this);
				return ERROR_FORBIDDEN;
			case 404:
				Util.LogDebug("Received message 'NOT_FOUND'", SendPostMessage.this);
				return ERROR_NOT_FOUND;
			case 405:
				Util.LogDebug("Received message 'METHOD_NOT_ALLOWED'", SendPostMessage.this);
				return ERROR_METHOD_NOT_ALLOWED;
			case 410:
				Util.LogDebug("Received message 'GONE'", SendPostMessage.this);
				return ERROR_GONE;
			case 500:
				Util.LogDebug("Received message 'INTERNAL_SERVER_ERROR'", SendPostMessage.this);
				return ERROR_INTERNAL_SERVER_ERROR;
			default:
				return OPERATION_RETRY;
		}

	}

	public void doInBackground(final Context context, final MessageSentListener messageSentListener) {

		final Handler networkOperationCompleteHandler = new Handler() {			
            @Override
            public void handleMessage(Message msg) {
            	
            	switch (msg.what) {
            	case OPERATION_COMPLETED:
            		if (messageSentListener != null)
            			messageSentListener.onSendSuccess();
            		break;
            	
            	case OPERATION_FAILED:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.OPERATION_FAILED);
            		break;
            	
            	case ERROR_PUSH_SERVICE_NOT_AVAILABLE:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_PUSH_SERVICE_NOT_AVAILABLE);
            		break;
            		
            	case ERROR_BAD_REQUEST:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_BAD_REQUEST);
            		break;
            		
            	case ERROR_UNAUTHORIZED:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_UNAUTHORIZED);
            		break;
            		
            	case ERROR_FORBIDDEN:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_FORBIDDEN);
            		break;
            		
            	case ERROR_NOT_FOUND:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_NOT_FOUND);
            		break;
            		
            	case ERROR_METHOD_NOT_ALLOWED:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_METHOD_NOT_ALLOWED);
            		break;
            		
            	case ERROR_GONE:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_GONE);
            		break;
            		
            	case ERROR_INTERNAL_SERVER_ERROR:
            		if (messageSentListener != null)
            			messageSentListener.onSendFailed(MessageSentListener.ERROR_INTERNAL_SERVER_ERROR);
            		break;
            	}
            }
		};
		
		Thread networkThread = new Thread(new Runnable() {
			
			@Override
			public void run() {				
				int operationResult = SendPostMessage.this.run(context);
				networkOperationCompleteHandler.sendEmptyMessage(operationResult);
			}
		});
		networkThread.start();
	}
	
	public int getResponseCode() {
		return rest.getResponseCode();
	}

	public String getMessage() {
		return rest.getMessage();
	}

	public String getResponse() {
		return rest.getResponse();
	}
	
	public void addParam(String name, String value) {
		rest.addParam(name, value);
	}

	public void addParams(JSONObject params) {
		rest.addParams(params);
	}
	
    public interface MessageSentListener {
    	
    	public static final int OPERATION_FAILED = SendPostMessage.OPERATION_FAILED;
    	public static final int ERROR_BAD_REQUEST = SendPostMessage.ERROR_BAD_REQUEST;
    	public static final int ERROR_UNAUTHORIZED = SendPostMessage.ERROR_UNAUTHORIZED;
    	public static final int ERROR_FORBIDDEN = SendPostMessage.ERROR_FORBIDDEN;
    	public static final int ERROR_NOT_FOUND = SendPostMessage.ERROR_NOT_FOUND;
    	public static final int ERROR_METHOD_NOT_ALLOWED = SendPostMessage.ERROR_METHOD_NOT_ALLOWED;
    	public static final int ERROR_GONE = SendPostMessage.ERROR_GONE;
    	public static final int ERROR_PUSH_SERVICE_NOT_AVAILABLE = SendPostMessage.ERROR_PUSH_SERVICE_NOT_AVAILABLE;
    	public static final int ERROR_INTERNAL_SERVER_ERROR = SendPostMessage.ERROR_INTERNAL_SERVER_ERROR;
    	
    	public abstract void onSendSuccess();
    	public abstract void onSendFailed(int reason);
    }
}
