package com.infobip.push.lib.livegeo;

import org.json.JSONException;
import org.json.JSONObject;

import com.google.android.gms.location.Geofence;
import com.infobip.push.lib.util.LiveGeoUtil;

/**
 * A single Geofence object, defined by its multiple attributes.
 */
public class LiveGeoArea {
	
	private String mId;
	private double mLatitude;
	private double mLongitude;
	private float mRadius;
	private long mExpirationDuration;
	private int mEvent;
	private long mTimeSaved;
	private int mTransitionType;

	/**
	 * @param Id
	 *            The id of the geofence
	 * @param latitude
	 *            Latitude of the Geofence's center.
	 * @param longitude
	 *            Longitude of the Geofence's center.
	 * @param radius
	 *            Radius of the geofence circle.
	 * @param expiration
	 *            Geofence expiration duration
	 * @param transition
	 *            Type of Geofence transition.
	 * @param placeId
	 *            id of the place is linked to
	 */
	public LiveGeoArea(String id, double latitude, double longitude, float radius, long expiration, int event, long timeSaved, int transitionType) {
		mId = id;
		mLatitude = latitude;
		mLongitude = longitude;
		mRadius = radius;
		mExpirationDuration = expiration;
		mEvent = event;
		mTimeSaved = timeSaved;
		mTransitionType = transitionType;
	}

	public String getId() {
		return mId;
	}

	public double getLatitude() {
		return mLatitude;
	}

	public double getLongitude() {
		return mLongitude;
	}

	public float getRadius() {
		return mRadius;
	}

	/**
	 * Gets live area's expiry time.
	 * @return Expiry in UNIX epoch time
	 */
	public long getExpirationDuration() {
		return mExpirationDuration;
	}

	public int getEvent() {
		return mEvent;
	}
	
	public long getTimeSaved() {
		return mTimeSaved;
	}
	
	public int getTransitionType() {
		return mTransitionType;
	}


	/**
	 * Creates a Location Services Geofence object from a LiveGeoArea.
	 * LiveGeoArea has expiry time saved in UNIX epoch time. Geofence object requires expiry defined in offset (geofence's active time).
	 * 
	 * Geofence's offset is calculated as a difference between UNIX epoch time milliseconds defined for LiveArea and current system's time. 
	 * 
	 * @return A Geofence object
	 */
	public Geofence toGeofence() {
		long expiryOffset = getExpirationDuration() - System.currentTimeMillis();
		return new Geofence.Builder().setRequestId(getId()).setTransitionTypes(getTransitionType()).setCircularRegion(getLatitude(), getLongitude(), getRadius())
				.setExpirationDuration(expiryOffset).build();
	}
	
	/**
	 * Creates a Location Services Geofence object from a LiveGeoArea.
	 * 
	 * @return A Geofence object
	 * @throws JSONException 
	 */
	public String toJsonString() throws JSONException {
		JSONObject liveGeoArea = new JSONObject();
		
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_ID, getId());
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_LATITUDE, getLatitude());
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_LONGITUDE, getLongitude());
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_RADIUS, getRadius());
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_EXPIRY, getExpirationDuration());
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_EVENT, getEvent());
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_TIME_SAVED, getTimeSaved());
		liveGeoArea.put(LiveGeoUtil.KEY_AREA_TRANSITION_TYPE, getTransitionType());
		
		return liveGeoArea.toString();
	}

}
