package com.infobip.push.lib.livegeo;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONException;
import org.json.JSONObject;

import com.infobip.push.lib.util.LiveGeoUtil;
import com.infobip.push.lib.util.Util;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.text.TextUtils;

/**
 * Storage for geofence values, implemented in SharedPreferences.
 */
public class LiveGeoAreaStorage {

	private static final String SHARED_PREFERENCES = "LiveGeoPreferences";
	public static final long INVALID_LONG_VALUE = -999l;
	public static final float INVALID_FLOAT_VALUE = -999.0f;
	public static final int INVALID_INT_VALUE = -999;
	public static final String INVALID_STRING_VALUE = "";

	private final SharedPreferences mPrefs;

	public LiveGeoAreaStorage(final Context context) {
		mPrefs = context.getSharedPreferences(SHARED_PREFERENCES, Context.MODE_PRIVATE);
	}
	
	/**
	 * Returns a stored geofence by its ID, or returns null if it's not found.
	 */
	public LiveGeoArea getLiveGeoArea(final String id) {
		String liveGeoAreaJsonStr = mPrefs.getString(id, INVALID_STRING_VALUE);
		
		if (!TextUtils.isEmpty(liveGeoAreaJsonStr)) {
			return deserializeLiveGeoArea(liveGeoAreaJsonStr);
			
		} else {
			return null;
		}
	}
	
	/**
	 * Creates a Location Services Geofence object from a LiveGeoArea.
	 * 
	 * @return A Geofence object
	 * @throws JSONException 
	 */
	private LiveGeoArea deserializeLiveGeoArea(String liveGeoAraJsonString) {
		String id = INVALID_STRING_VALUE;
		double lat = INVALID_FLOAT_VALUE;
		double lng = INVALID_FLOAT_VALUE;
		float radius= INVALID_FLOAT_VALUE;
		long expiry = INVALID_LONG_VALUE;
		int event = INVALID_INT_VALUE;
		long timeSaved = INVALID_LONG_VALUE;
		int transitionType = INVALID_INT_VALUE;
		
		try {
			JSONObject liveGeoArea = new JSONObject(liveGeoAraJsonString);
			
			id = liveGeoArea.optString(LiveGeoUtil.KEY_AREA_ID);
			lat = liveGeoArea.optDouble(LiveGeoUtil.KEY_AREA_LATITUDE);
			lng = liveGeoArea.optDouble(LiveGeoUtil.KEY_AREA_LONGITUDE);
			radius = (float) liveGeoArea.optDouble(LiveGeoUtil.KEY_AREA_RADIUS);
			expiry = liveGeoArea.optLong(LiveGeoUtil.KEY_AREA_EXPIRY);
			event = liveGeoArea.optInt(LiveGeoUtil.KEY_AREA_EVENT);
			timeSaved = liveGeoArea.optLong(LiveGeoUtil.KEY_AREA_TIME_SAVED);
			transitionType = liveGeoArea.optInt(LiveGeoUtil.KEY_AREA_TRANSITION_TYPE);
			
		} catch (JSONException e) {
			Util.LogError("Failed to parse JSON string for live geo area. Error: " + e.toString());
		}
		
		if (lat != INVALID_FLOAT_VALUE && lng != INVALID_FLOAT_VALUE && radius != INVALID_FLOAT_VALUE && expiry != INVALID_LONG_VALUE && event != INVALID_INT_VALUE
					&& !id.equals(INVALID_STRING_VALUE)) {
			return new LiveGeoArea(id, lat, lng, radius, expiry, event, timeSaved, transitionType);
			
		} else {
			return null;
		}
	}
	
	/**
	 * Save LiveGeoArea to shared prefs under its message ID
	 * @throws JSONException 
	 */
	public void setLiveGeoArea(final String id, final LiveGeoArea geofence) throws JSONException {
		String geofenceJsonString = geofence.toJsonString();
		Editor editor = mPrefs.edit();			
		editor.putString(id, geofenceJsonString);
		editor.commit();		
	}
	
	public void clearLiveGeoArea(String id) {
		Editor editor = mPrefs.edit();			
		editor.remove(id);
		editor.commit();
	}

	/**
	 * Get live geo area IDs
	 */
	public List<String> getLiveGeoAreaIds() {
		List<String> geofenceIds = new ArrayList<String>();
		geofenceIds.addAll(mPrefs.getAll().keySet());
		return geofenceIds;
	}

	/**
	 * Get all live geo areas from storage
	 */
	public List<LiveGeoArea> getAllLiveGeoAreas() {
		List<String> geofenceIds = getLiveGeoAreaIds();		
		List<LiveGeoArea> geofencesList = new ArrayList<LiveGeoArea>(geofenceIds.size());
		
		LiveGeoArea item = null;		
		for (String id : geofenceIds) {
			item = getLiveGeoArea(id);
			
			if (item != null) {
				geofencesList.add(item);
			}
		}
		return geofencesList;
	}

	/**
	 * Removes the given list of live geo areas by their ids
	 */
	public void clearLiveGeoAreas(final List<String> geofenceIds) {
		
		if (geofenceIds != null) {
			Editor editor = mPrefs.edit();
			
			for (String id : geofenceIds) {
				editor.remove(id);
			}
			
			editor.commit();
		}
	}

	/**
	 * Removes all stored live geo areas from shared prefs
	 */
	public void clearAllLiveGeoAreas() {
		Editor editor = mPrefs.edit();
		editor.clear();
		editor.commit();
	}

}
