package com.infobip.push.lib.livegeo;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.os.PowerManager;
import android.text.TextUtils;

import com.google.android.gms.location.Geofence;
import com.infobip.push.lib.SendPostMessage;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.LiveGeoUtil;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;

public class LiveGeoIntentService extends IntentService {
	
	private static final String JSON_KEY_LIVE_GEO_ARRAY = "liveLocations";
	private static final String JSON_KEY_LIVE_GEO_MSG_ID = "messageID";
	private static final String JSON_KEY_LIVE_GEO_EXPIRY = "expiry";
	private static final String JSON_KEY_LIVE_GEO_EVENT = "event";
	private static final String JSON_KEY_LIVE_GEO_AREA = "area";
	private static final String JSON_KEY_LIVE_GEO_LAT = "lat";
	private static final String JSON_KEY_LIVE_GEO_LONG = "long";
	private static final String JSON_KEY_LIVE_GEO_RADIUS = "radius";	
	
	private static final Object LOCK = LiveGeoIntentService.class;
	private static PowerManager.WakeLock sWakeLock;
	
	LiveGeoAreaStorage mLiveGeoAreaStorage;
	
	String mMessageId;
	double mLatitude;
	double mLongitude;
	long mExpiry;
	float mRadius;
	int mEvent;
	int mTransitionType;
	LiveGeoArea mLiveGeoArea;
	
	public LiveGeoIntentService() {
		super("LiveGeoIntentService");
	}	

	public static void runIntentInService(Context context, Intent intent) {
		acquireLock(context);
		intent.setClassName(context, LiveGeoIntentService.class.getName());
		context.startService(intent);
	}
	    
	private static void acquireLock(Context context) {		
		synchronized(LOCK) {
			if (sWakeLock == null) {
				PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
				sWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, Consts.WAKE_LOCK_LIVE_GEO_INTENT_SERVICE);
			}
		}
		if (!sWakeLock.isHeld()) {
			sWakeLock.acquire();
			Util.LogVerbose("Aquiring wakelock");
		}
	}
	
	
	@Override
	protected void onHandleIntent(Intent intent) {
		try {		
			mLiveGeoAreaStorage = new LiveGeoAreaStorage(getBaseContext());
			
			if (LiveGeoUtil.ADD_AREAS_ACTION.equals(intent.getAction())) {
				String liveGeosJsonString = intent.getStringExtra(LiveGeoUtil.KEY_LIVE_GEOS_EXTRA);
				handleAddingJsonOfAreas(liveGeosJsonString);
			}
			
			else if (LiveGeoUtil.ADD_AREA_ACTION.equals(intent.getAction())) {
				handleAddingSingleArea(intent);
			}
			
			else if (LiveGeoUtil.REMOVE_AREA_ACTION.equals(intent.getAction())) {
				handleRemovingSingleArea(intent);
			}
			
			else if (LiveGeoUtil.REQUEST_AREAS_ACTION.equals(intent.getAction())) {				
				handleRequestingAreas(intent);
			}
			
		} finally {
            // Release the power lock, so phone can get back to sleep. The lock is reference-counted by default, so multiple messages are ok.
			synchronized(LOCK) {
				// sanity check for null as this is a public method
				if (sWakeLock != null && sWakeLock.isHeld()) {
					Util.LogVerbose("Releasing wakelock");
                    sWakeLock.release();
                } else {
                    // should never happen during normal workflow
                    Util.LogError("Wakelock reference is null");
                }
			}
		}
	}	
	
	private void handleAddingJsonOfAreas(String liveGeosJsonString) {
		
		if (!TextUtils.isEmpty(liveGeosJsonString)) {			
			try {
				JSONObject liveGeosJson = new JSONObject(liveGeosJsonString);
				JSONArray areasJson = liveGeosJson.optJSONArray(JSON_KEY_LIVE_GEO_ARRAY);
				addLiveGeoArrayFromJson(areasJson);
			} catch (JSONException e) {
				Util.LogWarn("Unable to parse live geos JSON. Error: " + e.toString());
			}
		}
	}

	private void handleRequestingAreas(Intent intent) {
		String liveGeosJsonString = getActiveAreasFromPushServer();
		
		if (!TextUtils.isEmpty(liveGeosJsonString)) {
			try {
				JSONArray areasJson = new JSONArray(liveGeosJsonString);
				addLiveGeoArrayFromJson(areasJson);
			} catch (JSONException e) {
				Util.LogWarn("Unable to parse live geos JSON. Error: " + e.toString());
			}					
		}
	}

	private void handleRemovingSingleArea(Intent intent) {
		mMessageId = intent.getStringExtra(JSON_KEY_LIVE_GEO_MSG_ID);
		
		if (!TextUtils.isEmpty(mMessageId)) {
			Prefs prefs = new Prefs(getApplicationContext());
			List<String> mGeofencesToRemove = new ArrayList<String>(1);
			
			mGeofencesToRemove.add(mMessageId);

			LiveGeoUtil.setLiveGeoAreasAsVisited(getApplicationContext(), prefs, mGeofencesToRemove);
			LiveGeoUtil.removeLiveGeosFromPlayServices(getApplicationContext(), mGeofencesToRemove);
		}
	}

	private void handleAddingSingleArea(Intent intent) {
		mMessageId = intent.getStringExtra(JSON_KEY_LIVE_GEO_MSG_ID);
		String add_event = intent.getStringExtra(JSON_KEY_LIVE_GEO_EVENT);
		String add_expiry = intent.getStringExtra(JSON_KEY_LIVE_GEO_EXPIRY);
		String add_area = intent.getStringExtra(JSON_KEY_LIVE_GEO_AREA);
		
		if (!TextUtils.isEmpty(mMessageId) && !TextUtils.isEmpty(add_event) && !TextUtils.isEmpty(add_expiry) && !TextUtils.isEmpty(add_area)) {
			addSingleLiveGeoArea(add_event, add_expiry, add_area);
		}
	}

	private String getActiveAreasFromPushServer() {
		Prefs prefs = new Prefs(getApplicationContext());
		SendPostMessage sendPost = new SendPostMessage(String.format(Consts.URL_LIVE_GEO_AREAS, prefs.getDeviceId()), SendPostMessage.GET);
		int response = sendPost.run(getApplicationContext());
		
		String liveGeoAreasJsonString = null;
		
		switch (response) {
		case SendPostMessage.OPERATION_COMPLETED:
			Util.LogDebug("Confirmation sent");
			liveGeoAreasJsonString = sendPost.getResponse();
			break;
		
		default:
			Util.LogError("Sending confirmation and getting message failed.");
			break;
		}
		
		return liveGeoAreasJsonString;
	}

	private void addSingleLiveGeoArea(String add_event, String add_expiry, String add_area) {
		try {
			mEvent = Integer.valueOf(add_event);
			mExpiry = Long.valueOf(add_expiry);
			
			setupTransitionType(mEvent);
			
			JSONObject areaJson = new JSONObject(add_area);
			mLatitude = areaJson.optDouble(JSON_KEY_LIVE_GEO_LAT);
			mLongitude = areaJson.optDouble(JSON_KEY_LIVE_GEO_LONG);
			mRadius = (float) areaJson.optDouble(JSON_KEY_LIVE_GEO_RADIUS);
			
		} catch (NumberFormatException e) {
			Util.LogError("Received invalid event or expiry type. Error: " + e.toString());
			return;
			
		} catch (JSONException e) {
			Util.LogWarn("Unable to parse live geos JSON. Error: " + e.toString());
			return;
		}
		
		if (!LiveGeoUtil.isLiveGeoExpired(mExpiry)) {
			mLiveGeoArea = new LiveGeoArea(mMessageId, mLatitude, mLongitude, mRadius, mExpiry, mEvent, System.currentTimeMillis(), mTransitionType);	
			
			try {
				mLiveGeoAreaStorage.setLiveGeoArea(mMessageId, mLiveGeoArea);		
				Prefs prefs = new Prefs(getApplicationContext());
				prefs.appendActiveAreaIds(mMessageId);
				prefs.setLiveGeoRequestType(LiveGeoUtil.REQUEST_TYPE_ADD);
				prefs.save();
		
				LiveGeoUtil.addLiveGeosToPlayServices(getApplicationContext());
				
			} catch (JSONException e) {
				Util.LogError("Failed to construct JSON for live geo area. Error: " + e.toString());
			}
		}
	}

	private void addLiveGeoArrayFromJson(JSONArray areasJson) {
		
		try {
			JSONObject liveMessage = null;
			List<String> activeAreas = new ArrayList<String>();
			
			Prefs prefs = new Prefs(getApplicationContext());
			List<String> visitedAreaIds = prefs.getVisitedAreaIds();
			
			for (int i = 0; i < areasJson.length(); i++) {				
				liveMessage = areasJson.getJSONObject(i);
				
				mMessageId = liveMessage.optString(JSON_KEY_LIVE_GEO_MSG_ID);
				mExpiry = liveMessage.optLong(JSON_KEY_LIVE_GEO_EXPIRY);
				mEvent = liveMessage.optInt(JSON_KEY_LIVE_GEO_EVENT);
				
				setupTransitionType(mEvent);
				
				JSONObject area = liveMessage.getJSONObject(JSON_KEY_LIVE_GEO_AREA);						
				mLatitude = area.optDouble(JSON_KEY_LIVE_GEO_LAT);
				mLongitude = area.optDouble(JSON_KEY_LIVE_GEO_LONG);
				mRadius = (float) area.optDouble(JSON_KEY_LIVE_GEO_RADIUS);
				
				if (!LiveGeoUtil.isLiveGeoExpired(mExpiry)) {
					mLiveGeoArea = new LiveGeoArea(mMessageId, mLatitude, mLongitude, mRadius, mExpiry, mEvent, System.currentTimeMillis(),
							mTransitionType);
					mLiveGeoAreaStorage.setLiveGeoArea(mMessageId, mLiveGeoArea);
					
					activeAreas.add(mLiveGeoArea.getId());
				}
			}
			
			if (!activeAreas.isEmpty()) {				
				LiveGeoUtil.filterVisitedAndSaveNewActiveAreas(prefs, activeAreas, visitedAreaIds);
				
				LiveGeoUtil.addLiveGeosToPlayServices(getApplicationContext());
			}
			
		} catch (JSONException e) {
			Util.LogWarn("Unable to parse live geos JSON. Error: " + e.toString());
		}
	}
	
	private void setupTransitionType(int event) {
		switch (event) {		
		case 4:
			mTransitionType = Geofence.GEOFENCE_TRANSITION_ENTER;
			break;
			
		case 2:
			mTransitionType = Geofence.GEOFENCE_TRANSITION_EXIT;
			break;
			
		default:
			mTransitionType = Geofence.GEOFENCE_TRANSITION_ENTER|Geofence.GEOFENCE_TRANSITION_EXIT;
			break;
		}
	}

}
