package com.infobip.push.lib.livegeo;

import java.util.Arrays;
import java.util.List;

import android.app.PendingIntent;
import android.content.Context;
import android.os.Bundle;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient;
import com.google.android.gms.common.GooglePlayServicesClient.ConnectionCallbacks;
import com.google.android.gms.common.GooglePlayServicesClient.OnConnectionFailedListener;
import com.google.android.gms.location.LocationClient;
import com.google.android.gms.location.LocationClient.OnRemoveGeofencesResultListener;
import com.google.android.gms.location.LocationStatusCodes;
import com.infobip.push.lib.util.LiveGeoUtil;
import com.infobip.push.lib.util.Util;

public class LiveGeoRemover implements ConnectionCallbacks, OnConnectionFailedListener, OnRemoveGeofencesResultListener {

	// Storage for a context from the calling client
	private Context mContext;
	private LiveGeoCallbacks mListener;

	private String mPlaceId; 

	// Stores the current list of geofences
	private List<String> mCurrentGeofenceIds;

	// Stores the current instantiation of the location client
	private LocationClient mLocationClient;

	// The PendingIntent sent in removeGeofencesByIntent
	private PendingIntent mCurrentIntent;

	/*
	 * Record the type of removal. This allows continueRemoveGeofences to call the appropriate removal request method.
	 */
	private LiveGeoUtil.RemoveType mRequestType;

	/*
	 * Flag that indicates whether an add or remove request is underway. Check this flag before attempting to start a new request.
	 */
	private boolean mRemovingInProgress;

	/**
	 * Default constructor
	 * 
	 * @param context
	 */
	public LiveGeoRemover(final Context context, final LiveGeoCallbacks listener) {
		mContext = context;
		mListener = listener;
	}

	/**
	 * Set the "in progress" flag from a caller. This allows callers to re-set a request that failed but was later fixed.
	 * 
	 * @param flag
	 *            Turn the in progress flag on or off.
	 */
	public void setInProgressFlag(final boolean flag) {
		mRemovingInProgress = flag;
	}

	/**
	 * Get the current in progress status.
	 * 
	 * @return The current value of the in progress flag.
	 */
	public boolean getInProgressFlag() {
		return mRemovingInProgress;
	}

	/**
	 * Request a connection to Location Services. This call returns immediately, but the request is not complete until onConnected() or onConnectionFailure() is called.
	 */
	private void requestConnection() {
		getLocationClient().connect();
	}

	/**
	 * Get the current location client, or create a new one if necessary.
	 * 
	 * @return A LocationClient object
	 */
	private GooglePlayServicesClient getLocationClient() {
		if (mLocationClient == null) {
			mLocationClient = new LocationClient(mContext, this, this);
		}
		return mLocationClient;
	}

	/**
	 * Remove the geofences associated with a PendIntent. The PendingIntent is the one used in the request to add the geofences; all geofences in that request are removed. To remove a subset
	 * of those geofences, call removeGeofencesById().
	 * 
	 * @param requestIntent
	 *            The PendingIntent used to request the geofences
	 */
	public void removeGeofencesByIntent(final PendingIntent requestIntent) {

		if (mRemovingInProgress) {
			throw new UnsupportedOperationException();
		}
		
		mRequestType = LiveGeoUtil.RemoveType.INTENT;
		mCurrentIntent = requestIntent;
		requestConnection();
	}

	/**
	 * When the request to remove geofences by PendingIntent returns, handle the result.
	 * 
	 * @param statusCode
	 *            the code returned by Location Services
	 * @param requestIntent
	 *            The Intent used to request the removal.
	 */
	@Override
	public void onRemoveGeofencesByPendingIntentResult(final int statusCode, final PendingIntent requestIntent) {

		if (statusCode == LocationStatusCodes.SUCCESS) {
			Util.LogDebug("Live geo removed");
			mListener.removeGeofenceListener(mPlaceId);
		} else {
			Util.LogError("Removing live geos failed");
			mListener.errorGeofenceListener(mPlaceId);
		}

		requestDisconnection();
	}

	/**
	 * Remove the geofences in a list of geofence IDs. To remove all current geofences associated with a request, you can also call removeGeofencesByIntent.
	 * <b>Note: The List must contain at least one ID, otherwise an Exception is thrown</b>
	 * 
	 * @param geofenceIds
	 *            A List of geofence IDs
	 */
	public void removeGeofencesById(final List<String> geofenceIds, final String placeId) throws IllegalArgumentException, UnsupportedOperationException {

		if ((geofenceIds == null) || (geofenceIds.size() == 0)) {
			throw new IllegalArgumentException();
		}
		
		if (mRemovingInProgress) {
			throw new UnsupportedOperationException();
		}
			
		mPlaceId = placeId;
		mRequestType = LiveGeoUtil.RemoveType.LIST;
		mCurrentGeofenceIds = geofenceIds;
		
		requestConnection();
	}

	/**
	 * When the request to remove geofences by IDs returns, handle the result.
	 * 
	 * @param statusCode
	 *            The code returned by Location Services
	 * @param geofenceRequestIds
	 *            The IDs removed
	 */
	@Override
	public void onRemoveGeofencesByRequestIdsResult(final int statusCode, final String[] geofenceRequestIds) {

		if (LocationStatusCodes.SUCCESS == statusCode) {
			Util.LogDebug("Removed live geos with following message IDs: " + Arrays.toString(geofenceRequestIds));
			mListener.removeGeofenceListener(mPlaceId);
		} else {
			Util.LogError("Removing live geos for following message IDs failed: " + Arrays.toString(geofenceRequestIds));
			mListener.errorGeofenceListener(mPlaceId);
		}

		requestDisconnection();
	}

	/**
	 * Get a location client and disconnect from Location Services
	 */
	private void requestDisconnection() {
		mRemovingInProgress = false;

		getLocationClient().disconnect();
		/*
		 * If the request was done by PendingIntent, cancel the Intent. This prevents problems if the client gets disconnected before the disconnection request finishes; the location updates
		 * will still be cancelled.
		 */
		if (mRequestType == LiveGeoUtil.RemoveType.INTENT) {
			mCurrentIntent.cancel();
		}
	}

	/*
	 * Implementation of OnConnectionFailedListener.onConnectionFailed If a connection or disconnection request fails, report the error connectionResult is passed in from Location Services
	 */
	@Override
	public void onConnectionFailed(final ConnectionResult connectionResult) {
		mRemovingInProgress = false;
		mListener.errorGeofenceListener(mPlaceId);
	}

	/*
	 * Called by Location Services once the location client is connected.
	 * 
	 * Continue by removing the requested geofences.
	 */
	@Override
	public void onConnected(final Bundle arg0) {
		continueRemoveGeofences();
	}

	/**
	 * Once the connection is available, send a request to remove the Geofences. The method signature used depends on which type of remove request was originally received.
	 */
	private void continueRemoveGeofences() {
		
		switch (mRequestType) {
		case INTENT:
			mLocationClient.removeGeofences(mCurrentIntent, this);
			break;

		case LIST:
			mLocationClient.removeGeofences(mCurrentGeofenceIds, this);
			break;
		}
	}

	/*
	 * Called by Location Services if the connection is lost. Destroys LocationClient instance.
	 */
	@Override
	public void onDisconnected() {
		mRemovingInProgress = false;
		mLocationClient = null;
	}

}
