package com.infobip.push.lib.livegeo;

import java.util.Arrays;
import java.util.List;

import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient;
import com.google.android.gms.common.GooglePlayServicesClient.ConnectionCallbacks;
import com.google.android.gms.common.GooglePlayServicesClient.OnConnectionFailedListener;
import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.LocationClient;
import com.google.android.gms.location.LocationClient.OnAddGeofencesResultListener;
import com.google.android.gms.location.LocationStatusCodes;
import com.infobip.push.lib.InternalReceiver;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.Util;

/**
 * <p>Class for connecting to Location Services and requesting geofences. <b> Note: Clients must ensure that Google Play services is available before requesting geofences.</b> Use
 * GooglePlayServicesUtil.isGooglePlayServicesAvailable() to check.</p>
 * <p>To use a LiveGeoRequester, instantiate it and call addGeofences(). Everything else is done automatically.</p>
 */
public class LiveGeoRequester implements OnAddGeofencesResultListener, ConnectionCallbacks, OnConnectionFailedListener {

	private String mPlaceId;

	private final Context mContext;
	private LiveGeoCallbacks mListener;

	// Stores the PendingIntent used to send geofence transitions back to the app
	private PendingIntent mGeofencePendingIntent;

	// Stores the current list of geofences
	private List<Geofence> mCurrentGeofences;

	// Stores the current instantiation of the location client
	private LocationClient mLocationClient;

	/*
	 * Flag that indicates whether an add or remove request is underway. Check this flag before attempting to start a new request.
	 */
	private boolean mAddingInProgress;

	/**
	 * Default private constructor
	 * @param context
	 */
	public LiveGeoRequester(final Context context, final LiveGeoCallbacks listener) {
		mContext = context;
		mListener = listener;
	}

	/**
	 * Set the "in progress" flag from a caller. This allows callers to re-set a request that failed but was later fixed.
	 * @param flag Turn the in progress flag on or off.
	 */
	public void setInProgressFlag(final boolean flag) {
		mAddingInProgress = flag;
	}

	/**
	 * Get the current in progress status.
	 * @return The current value of the in progress flag.
	 */
	public boolean getInProgressFlag() {
		return mAddingInProgress;
	}

	/**
	 * Returns the current PendingIntent to the caller.
	 * @return The PendingIntent used to create the current set of geofences
	 */
	public PendingIntent getRequestPendingIntent() {
		return createRequestPendingIntent();
	}

	/**
	 * Request a connection to Location Services. This call returns immediately, but the request is not complete until onConnected() or onConnectionFailure() is called.
	 */
	private void requestConnection() {
		getLocationClient().connect();
	}

	/**
	 * Get the current location client, or create a new one if necessary.
	 * @return A LocationClient object
	 */
	private GooglePlayServicesClient getLocationClient() {
		if (mLocationClient == null) {
			mLocationClient = new LocationClient(mContext, this, this);
		}
		return mLocationClient;

	}

	/**
	 * Start adding geofences. Save the geofences, then start adding them by requesting a connection
	 * @param geofences A List of one or more geofences to add.
	 */
	public void addGeofences(final List<Geofence> geofences, final String placeId) throws UnsupportedOperationException {

		mCurrentGeofences = geofences;
		mPlaceId = placeId;

		if (mAddingInProgress) {
			throw new UnsupportedOperationException();
		}
		
		mAddingInProgress = true;
		requestConnection();
	}

	/**
	 * Once the connection is available, send a request to add the Geofences
	 */
	private void continueAddGeofences() {
		mGeofencePendingIntent = createRequestPendingIntent();
		mLocationClient.addGeofences(mCurrentGeofences, mGeofencePendingIntent, this);
	}

	/*
	 * Implementation of OnConnectionFailedListener.onConnectionFailed If a connection or disconnection request fails, report the error connectionResult is passed in from Location Services
	 */
	@Override
	public void onConnectionFailed(final ConnectionResult connectionResult) {
		mAddingInProgress = false;
		mListener.errorGeofenceListener(mPlaceId);
	}

	/*
	 * Called by Location Services once the location client is connected.
	 * Continue by adding the requested geofences.
	 */
	@Override
	public void onConnected(final Bundle arg0) {
		continueAddGeofences();
	}

	/*
	 * Called by Location Services once the location client is disconnected.
	 */
	@Override
	public void onDisconnected() {
		mAddingInProgress = false;
		mLocationClient = null;
	}

	/**
	 * Get a location client and disconnect from Location Services
	 */
	private void requestDisconnection() {
		mAddingInProgress = false;
		getLocationClient().disconnect();
	}

	/*
	 * Handle the result of adding the geofences
	 */
	@Override
	public void onAddGeofencesResult(final int statusCode, final String[] geofenceRequestIds) {

		if (mListener != null) {
			
			if (LocationStatusCodes.SUCCESS == statusCode) {
				Util.LogDebug("Geofences ids added: " + Arrays.toString(geofenceRequestIds));
				mListener.addGeofenceListener(mPlaceId);
				
			} else {
				Util.LogError("Add geofences failed. Ids: " + Arrays.toString(geofenceRequestIds));
				mListener.errorGeofenceListener(mPlaceId);
			}
		}
		requestDisconnection();
	}

	/**
	 * Get a PendingIntent to send with the request to add Geofences. Location Services issues the Intent inside this PendingIntent whenever a geofence transition occurs for the current list
	 * of geofences.
	 * 
	 * @return A PendingIntent for the IntentService that handles geofence transitions.
	 */
	private PendingIntent createRequestPendingIntent() {

		if (mGeofencePendingIntent == null) {
			Intent intent = new Intent(mContext, InternalReceiver.class);
			intent.setAction(Consts.LIVE_GEO_TRANSITION_ACTION);
			mGeofencePendingIntent = PendingIntent.getBroadcast(mContext, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
		}
		return mGeofencePendingIntent;
	}
	
}
