package com.infobip.push.lib.livegeo;

import java.util.ArrayList;
import java.util.List;

import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.os.PowerManager;
import android.text.TextUtils;

import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.LocationClient;
import com.infobip.push.lib.InfobipNotificationHandlingUtil;
import com.infobip.push.lib.SendPostMessage;
import com.infobip.push.lib.util.Consts;
import com.infobip.push.lib.util.LiveGeoUtil;
import com.infobip.push.lib.util.Prefs;
import com.infobip.push.lib.util.Util;

public class ReceiveTransitionsIntentService extends IntentService {

	private static final Object LOCK = ReceiveTransitionsIntentService.class;
	private static PowerManager.WakeLock sWakeLock;

	LiveGeoRemover mGeofenceRemover;
	List<String> mGeofencesToRemove;
	LiveGeoAreaStorage mGeofenceStorage;
	
	Prefs prefs;

	
	public ReceiveTransitionsIntentService() {
		super("ReceiveTransitionsIntentService");
	}

	public static void runIntentInService(Context context, Intent intent) {
		acquireLock(context);
		intent.setClassName(context, ReceiveTransitionsIntentService.class.getName());
		context.startService(intent);
	}
	    
	private static void acquireLock(Context context) {		
		synchronized(LOCK) {
			if (sWakeLock == null) {
				PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
				sWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, Consts.WAKE_LOCK_LIVE_GEO_INTENT_SERVICE);
			}
		}
		if (!sWakeLock.isHeld()) {
			sWakeLock.acquire();
			Util.LogVerbose("Aquiring wakelock");
		}
	}

	/**
	 * Handles incoming intents
	 * 
	 * @param intent
	 *            The Intent sent by Location Services. This Intent is provided to Location Services (inside a PendingIntent) when you call addGeofences()
	 */
	@Override
	protected void onHandleIntent(final Intent intent) {
		
		
		try {
			int transition = LocationClient.getGeofenceTransition(intent);
			if ((transition == Geofence.GEOFENCE_TRANSITION_ENTER) || (transition == Geofence.GEOFENCE_TRANSITION_EXIT)) {
				prefs = new Prefs(getApplicationContext());
				
				List<Geofence> geofenceList = LocationClient.getTriggeringGeofences(intent);
				onGeofencesTriggered(geofenceList, transition);
			
			} else {
				Util.LogError("Geofence transition error: " + Integer.toString(transition));
			}

		} finally {
	        // Release the power lock, so phone can get back to sleep. The lock is reference-counted by default, so multiple messages are ok.
			synchronized(LOCK) {
				// sanity check for null as this is a public method
				if (sWakeLock != null && sWakeLock.isHeld()) {
					Util.LogVerbose("Releasing wakelock");
	                sWakeLock.release();
	            } else {
	                // should never happen during normal workflow
	                Util.LogError("Wakelock reference is null");
	            }
			}
		}
		
	}
	
	/**
	 * If the notification is received for the triggered geofence, it will be managed like other push notifications. All triggered notifications, 
	 * expired ones and gone from server are being removed from the library.
	 */
	private void onGeofencesTriggered(List<Geofence> geofenceList, int transition) {
		String[] geofenceIds = new String[geofenceList.size()];
		mGeofencesToRemove = new ArrayList<String>(geofenceList.size());
		
		for (int i = 0; i < geofenceIds.length; i++) {	
			// "triggered geofence with messageID: " + placeId
			String placeId = geofenceList.get(i).getRequestId();
			
			geofenceIds[i] = placeId;
			
			LiveGeoArea geofence = new LiveGeoAreaStorage(this).getLiveGeoArea(geofenceIds[i]);

			if (geofence != null) {
				
				if (LiveGeoUtil.isLiveGeoExpired(geofence.getExpirationDuration())) {
					mGeofencesToRemove.add(placeId);
				
				} else if (LiveGeoUtil.isLiveGeoEventActive(geofence.getEvent(), geofence.getTimeSaved(), transition)) {
					String liveGeoMessage = getLiveGeoMessage(placeId);
					
					if (!TextUtils.isEmpty(liveGeoMessage)) {
						InfobipNotificationHandlingUtil.handleInfobipMessage(getApplicationContext(), placeId, liveGeoMessage);
						mGeofencesToRemove.add(placeId);
					}
					
					else if ("".equals(liveGeoMessage)) {							
						mGeofencesToRemove.add(placeId);
					}
				}
			}
		}
		
		if (!mGeofencesToRemove.isEmpty()) {
			LiveGeoUtil.setLiveGeoAreasAsVisited(getApplicationContext(), prefs, mGeofencesToRemove);			
			LiveGeoUtil.removeLiveGeosFromPlayServices(getApplicationContext(), mGeofencesToRemove);
		}
	}

	private String getLiveGeoMessage(String placeId) {
		Util.LogDebug("Obtaining live geo notification...");
		
		String deviceId = prefs.getDeviceId();
		
		SendPostMessage sendPost = new SendPostMessage(String.format(Consts.URL_LIVE_GEO_MESSAGE, placeId, deviceId), SendPostMessage.GET);
		int response = sendPost.run(getApplicationContext());
		
		switch (response) {
		case SendPostMessage.OPERATION_COMPLETED:
			Util.LogDebug("Live geo message received.");
			return sendPost.getResponse();
			
		case SendPostMessage.ERROR_GONE:
			Util.LogDebug("Live geo message isn't active anymore, removing it...");
			return "";
			
		default:
			Util.LogError("Receiving live geo message failed.");
			return null;
		}
	}

}
