package com.infobip.push.lib.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import android.app.Service;
import android.content.Context;
import android.location.LocationManager;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.location.Geofence;
import com.infobip.push.lib.livegeo.LiveGeoArea;
import com.infobip.push.lib.livegeo.LiveGeoAreaStorage;
import com.infobip.push.lib.livegeo.LiveGeoCallbacks;
import com.infobip.push.lib.livegeo.LiveGeoRemover;
import com.infobip.push.lib.livegeo.LiveGeoRequester;

/**
 * This class defines constants used by location sample apps.
 */
public final class LiveGeoUtil {

	// Used to track what type of geofence removal request was made.
	public enum RemoveType {
		INTENT, LIST
	}

	public static final String REQUEST_TYPE_ADD				= "ADD";
	
	public static final String ADD_AREA_ACTION				= "com.push.ADD_AREA";
	public static final String ADD_AREAS_ACTION				= "com.push.ADD_AREAS";
	public static final String REQUEST_AREAS_ACTION			= "com.push.REQUEST_AREAS";
	public static final String REMOVE_AREA_ACTION			= "com.push.REMOVE_AREA";
	public static final String NETWORK_ENABLED_ACTION 		= "com.push.NETWORK_ENABLED";
	
	public static final String KEY_LIVE_GEOS_EXTRA			= "liveGeosJson";	
	
	public static final String KEY_AREA 					= "area";	
	public static final String KEY_AREA_ID 					= "id";
	public static final String KEY_AREA_LATITUDE 			= "latitude";
	public static final String KEY_AREA_LONGITUDE 			= "longitude";
	public static final String KEY_AREA_RADIUS 				= "radius";
	public static final String KEY_AREA_EXPIRY 				= "expiry";
	public static final String KEY_AREA_EVENT 				= "event";
	public static final String KEY_AREA_TIME_SAVED 			= "timeSaved";
	public static final String KEY_AREA_TRANSITION_TYPE 	= "transitionType";
	
	/**
	 * Returns a {@link LiveGeoArea} if there is one associated to the given placeId. 
	 */
	public static LiveGeoArea isLiveGeoAssociated(final Context context, final String placeId) {
		LiveGeoAreaStorage store = new LiveGeoAreaStorage(context);
		return store.getLiveGeoArea(placeId);
	}
	
	/**
	 * Checks whether live geo area expired.
	 */
	public static boolean isLiveGeoExpired(long expiryMillis) {
		return expiryMillis < System.currentTimeMillis();
	}
	
	/**
	 * Checks if the triggered event (entry, in, exit) is active for a given transition.
	 */
	public static boolean isLiveGeoEventActive(int event, long timeSaved, int transition) {
		boolean exit = ((event & 2) == 2) && (transition == Geofence.GEOFENCE_TRANSITION_EXIT);
		
		boolean eventNow = ((event & 1) == 1) && (transition == Geofence.GEOFENCE_TRANSITION_ENTER);
		boolean liveGeoRecentlyAdded = (System.currentTimeMillis() - timeSaved) < Consts.VALID_LOCATION_MILLIS; //ONE MINUTE
		boolean now = eventNow && liveGeoRecentlyAdded;
		
		boolean enter = ((event & 4) == 4) && (transition == Geofence.GEOFENCE_TRANSITION_ENTER) && !liveGeoRecentlyAdded;
		
		return enter|exit|now;
	}
	
	/**
	 * Adds live geo areas for monitoring if areas exist. ADD TO STORAGE / ADD TO GOOGLE PLAY SERVICES / ADD TO PREFS ACTIVE IDs
	 */
	public static void addLiveGeosToPlayServices(Context context) {
		final Prefs prefs = new Prefs(context);
		LocationManager lm = (LocationManager) context.getSystemService(Service.LOCATION_SERVICE);
			
		if (!lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			prefs.setLiveGeoRequestType(REQUEST_TYPE_ADD);
			prefs.save();
			return;
		}

		List<String> activeLiveGeoIds = prefs.getActiveAreaIds();		
		if (activeLiveGeoIds.isEmpty()) {
			return;
		}
		
		if (!REQUEST_TYPE_ADD.equals(prefs.getLiveGeoRequestType())) {
			return;
		}
		
		LiveGeoRequester liveGeoRequester = new LiveGeoRequester(context, new LiveGeoCallbacks() {
			
			@Override
			public void removeGeofenceListener(String placeId) { }
			
			@Override
			public void errorGeofenceListener(String placeId) {
				prefs.setLiveGeoRequestType(REQUEST_TYPE_ADD);
				prefs.save();
			}
			
			@Override
			public void addGeofenceListener(String placeId) {				
				prefs.setLiveGeoRequestType("");
				prefs.save();
			}
		});
		
		if (activeLiveGeoIds.isEmpty() || !LiveGeoUtil.gPlayServicesConnected(context)) {
			return;
		}
		
		try {
			List<Geofence> activeGeofences = getActiveGeofencesFromStorage(context, prefs, activeLiveGeoIds);			
			if (!activeGeofences.isEmpty()) {
				liveGeoRequester.addGeofences(activeGeofences, activeGeofences.get(0).getRequestId());
				prefs.setLiveGeoRequestType("");
				prefs.save();
			}
		} catch (UnsupportedOperationException e) {
			//previous request is in progress
			prefs.setLiveGeoRequestType(REQUEST_TYPE_ADD);
			prefs.save();
		}
	}
	
	public static void setLiveGeoAreasAsVisited(Context context, Prefs prefs, List<String> liveGeosToRemove) {
		prefs.appendVisitedAreaIds(new HashSet<String>(liveGeosToRemove));
		prefs.removeFromActiveAreaIds(liveGeosToRemove);
		prefs.setLiveGeoRequestType("");
		prefs.save();
	}
	
	/**
	 * Adds live geo areas for monitoring if areas exist.
	 */
	public static void removeLiveGeosFromPlayServices(final Context context, final List<String> liveGeosToRemove) {
		LocationManager lm = (LocationManager) context.getSystemService(Service.LOCATION_SERVICE);
		
		if (!lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			return;
		}
		
		LiveGeoRemover liveGeoRemover = new LiveGeoRemover(context, new LiveGeoCallbacks() {
			
			@Override
			public void removeGeofenceListener(String placeId) {
				LiveGeoAreaStorage storage = new LiveGeoAreaStorage(context);
				storage.clearLiveGeoAreas(liveGeosToRemove);
			}
			
			@Override
			public void errorGeofenceListener(String placeId) {	} //do nothing, areas will be removed from Play services when triggered or expired
			
			@Override
			public void addGeofenceListener(String placeId) { }
		});
		
		if (liveGeosToRemove.isEmpty() || !LiveGeoUtil.gPlayServicesConnected(context)) {
			return;
		}
		
		try {
			liveGeoRemover.removeGeofencesById(liveGeosToRemove, liveGeosToRemove.get(0));
		} catch (UnsupportedOperationException e) {
			//previous request is in progress
		}
	}
	
	/**
	 * Adds live geo areas for monitoring if areas exist.
	 */
	public static void removeAllActiveLiveGeosFromPlayServices(final Context context, Prefs prefs) {
		LocationManager lm = (LocationManager) context.getSystemService(Service.LOCATION_SERVICE);
		
		if (!lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			return;
		}
		
		final List<String> liveGeosToRemove = prefs.getActiveAreaIds();
		
		LiveGeoRemover liveGeoRemover = new LiveGeoRemover(context, new LiveGeoCallbacks() {
			
			@Override
			public void removeGeofenceListener(String placeId) {
				LiveGeoAreaStorage storage = new LiveGeoAreaStorage(context);
				storage.clearLiveGeoAreas(liveGeosToRemove);
			}
			
			@Override
			public void errorGeofenceListener(String placeId) {	} //do nothing, areas will be removed from Play services when triggered or expired
			
			@Override
			public void addGeofenceListener(String placeId) { }
		});
		
		if (liveGeosToRemove.isEmpty() || !LiveGeoUtil.gPlayServicesConnected(context)) {
			return;
		}
			
		try {	
			liveGeoRemover.removeGeofencesById(liveGeosToRemove, liveGeosToRemove.get(0));
		} catch (UnsupportedOperationException e) {
			//previous request is in progress
		}
	}
	
	/**
	 * Get active and remove expired live geo areas from geofence storage.
	 * @return List of geofences
	 */
	private static List<Geofence> getActiveGeofencesFromStorage(Context context, Prefs prefs, List<String> activeAreaIds) {
		List<Geofence> activeGeofences = new ArrayList<Geofence>();
		LiveGeoAreaStorage storage = new LiveGeoAreaStorage(context);
		LiveGeoArea area;
		
		List<String> tempExpiredGeofences = new ArrayList<String>();
		
		for (String id : activeAreaIds) {
			area = storage.getLiveGeoArea(id);
			
			if (area != null && !LiveGeoUtil.isLiveGeoExpired(area.getExpirationDuration())) {
				activeGeofences.add(area.toGeofence()); 
			} else {
				storage.clearLiveGeoArea(id);
				tempExpiredGeofences.add(id);
			}
		}
		
		if (!tempExpiredGeofences.isEmpty()) {
			prefs.removeFromActiveAreaIds(tempExpiredGeofences);
			prefs.save();
		}
		
		return activeGeofences;
	}
	
	public static void filterVisitedAndSaveNewActiveAreas(Prefs prefs, List<String> activeAreas, List<String> visitedAreaIds) {		
		visitedAreaIds.retainAll(activeAreas);
		prefs.setVisitedAreaIds(visitedAreaIds);
		
		activeAreas.removeAll(visitedAreaIds);		
		prefs.setActiveAreaIds(activeAreas);
		
		prefs.setLiveGeoRequestType(LiveGeoUtil.REQUEST_TYPE_ADD);
		prefs.save();
	}
	
	public static void removeAllLiveGeosFromPrefs(Context context, Prefs prefs) {
		LiveGeoAreaStorage lgas = new LiveGeoAreaStorage(context);
		lgas.clearAllLiveGeoAreas();
		prefs.removeAllActiveAreaIds();
		prefs.setLiveGeoRequestType("");
		prefs.save();
	}
	
	public static void removeLiveGeosFromPrefs(Prefs prefs, List<String> geofencesToRemove) {
		prefs.appendVisitedAreaIds(new HashSet<String>(geofencesToRemove));
		prefs.removeAllActiveAreaIds();
		prefs.setLiveGeoRequestType("");
		prefs.save();
	}
	
	/**
	 * Get active and remove expired live geo areas from geofence storage.
	 * @return List of geofences
	 */
	public static void filterExpiredGeofencesFromStorage(Context context, Prefs prefs) {
		LiveGeoAreaStorage storage = new LiveGeoAreaStorage(context);
		List<LiveGeoArea> geofences = storage.getAllLiveGeoAreas();
		
		if (geofences.isEmpty()) {
			return;
		}
		
		List<String> expiredAreaIds = new ArrayList<String>();
		
		for (LiveGeoArea simpleGeofence : geofences) {
			
			if (LiveGeoUtil.isLiveGeoExpired(simpleGeofence.getExpirationDuration())) {//					
				expiredAreaIds.add(simpleGeofence.getId());					
				storage.clearLiveGeoArea(simpleGeofence.getId());
			}
		}
		
		List<String> activeAreaIds = prefs.getActiveAreaIds();
		List<String> visitedAreaIds = prefs.getVisitedAreaIds();
		
		if (!expiredAreaIds.isEmpty()) {
			activeAreaIds.removeAll(expiredAreaIds);
			visitedAreaIds.removeAll(expiredAreaIds);
			
			prefs.setActiveAreaIds(activeAreaIds);
			prefs.setVisitedAreaIds(visitedAreaIds);			
			prefs.save();
		}
	}
	
	/**
	 * Verify that Google Play services is available before making a request.
	 * 
	 * @return true if Google Play services is available, otherwise false
	 */
	public static boolean gPlayServicesConnected(Context context) {
		int resultCode = GooglePlayServicesUtil.isGooglePlayServicesAvailable(context);
		return ConnectionResult.SUCCESS == resultCode;
	}
}
