package com.infobip.push.lib.util;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.SystemClock;
import android.text.TextUtils;

import com.infobip.push.lib.InternalReceiver;
import com.infobip.push.lib.LocationService;
import com.infobip.push.lib.LocationUpdateService;
import com.infobip.push.lib.livegeo.LiveGeoReceiver;

public class LocationManagerUtil {
	
	private static LocationManagerUtil locUtil;	
	private static Context context;
	
	private LocationManagerUtil(Context context) {
		LocationManagerUtil.context = context;
	}
	
	public static LocationManagerUtil getInstance(Context context) {
		
		if (locUtil == null) {
			locUtil = new LocationManagerUtil(context.getApplicationContext());
		}
		return locUtil;
	}

	/**
	 * <p>Starts location sending alarm with {@link Consts#LOCATION_SENDING_ACTION} action.</p>
	 */
	public void startUpdateAlarm(Prefs prefs) {
		AlarmManager mgr = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
		
		long updateTimeInterval = prefs.getLocationUpdateTimeInterval() * 60 * 1000;
		
		if (prefs.isUsingCustomLocationService()) {
			PendingIntent pi = setupLocationSendingPendingIntent();
			mgr.setRepeating(AlarmManager.ELAPSED_REALTIME_WAKEUP, SystemClock.elapsedRealtime() + 60 * 1000, updateTimeInterval, pi);
		} else {
			PendingIntent pi = setupLocationRetrievingPendingIntent();
			mgr.setRepeating(AlarmManager.ELAPSED_REALTIME_WAKEUP, SystemClock.elapsedRealtime(), updateTimeInterval, pi);
		}	
	}
	
	/**
	 * <p>Cancels alarm set to send (and retrieve, in case of using Infobip) location every {@link #locationUpdateInterval}.</p>
	 */
	public void cancelUpdateAlarm(Prefs prefs) {		
		AlarmManager mgr = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
		PendingIntent pi;
		
		if (prefs.isUsingCustomLocationService()) {
			pi = setupLocationSendingPendingIntent();
		} else {
			pi = setupLocationRetrievingPendingIntent();
		}
		
		if (pi != null) {
			mgr.cancel(pi);
			pi.cancel();
		}
	}
	
	/**
	 * It should run if the user is registered and location is enabled.
	 */
	public boolean shouldLocationUpdatesRun(Prefs prefs) {
		
		if (!prefs.isLocationEnabled()) {
			return false;
		}
		
		if (TextUtils.isEmpty(prefs.getRegistrationId())) {
			return false;
		}
		return true;
	}
	
	/**
	 * <p>Reschedules location sending or retrieving update alarm if they aren't up.</p>
	 */
	public void rescheduleUpdateAlarmIfNecessary(Prefs prefs) {
		
		if (isMigrateLocationRetrievingAlarmUp() || isMigrateLocationSendingAlarmUp()) {
			cancelMigrateUpdateAlarm(prefs);
		}
		
		if (!isLocationSendingAlarmUp() && !isLocationRetrievingAlarmUp()) {
		    startUpdateAlarm(prefs);
		}
	}

	/**
	 * <p>Cancels alarm set to send (and retrieve, in case of using Infobip) location every {@link #locationUpdateInterval}.</p>
	 */
	public void cancelMigrateUpdateAlarm(Prefs prefs) {		
		AlarmManager mgr = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
		
		PendingIntent pi = setupMigrateLocationPendingIntent(prefs);
		if (pi != null) {
			mgr.cancel(pi);
			pi.cancel();
		}
	}

	private PendingIntent setupMigrateLocationPendingIntent(Prefs prefs) {
		
		if (prefs.isUsingCustomLocationService()) {
			Intent locationSendingIntent = new Intent(context, LocationUpdateService.class);
			locationSendingIntent.setAction(Consts.LOCATION_SENDING_ACTION);
			return PendingIntent.getService(context, 11, locationSendingIntent, 0);
			
		} else {
			Intent locationRetrievingIntent = new Intent(context, LocationService.class);
			locationRetrievingIntent.setAction(Consts.LOCATION_RETRIEVING_ACTION);
			return PendingIntent.getService(context, 22, locationRetrievingIntent, 0);
		}
	}

	private boolean isMigrateLocationSendingAlarmUp() {
		Intent locationSendingIntent = new Intent(context, LocationUpdateService.class);
		locationSendingIntent.setAction(Consts.LOCATION_SENDING_ACTION);		
		boolean locationSendingAlarmUp = (PendingIntent.getService(context, 11, locationSendingIntent, PendingIntent.FLAG_NO_CREATE) != null);
		
		return locationSendingAlarmUp;
	}

	private boolean isMigrateLocationRetrievingAlarmUp() {
		Intent locationRetrievingIntent = new Intent(context, LocationService.class);
		locationRetrievingIntent.setAction(Consts.LOCATION_RETRIEVING_ACTION);		
		boolean locationRetrievingAlarmUp = (PendingIntent.getService(context, 22, locationRetrievingIntent, PendingIntent.FLAG_NO_CREATE) != null);
		
		return locationRetrievingAlarmUp;
	}

	private boolean isLocationSendingAlarmUp() {
		Intent locationSendingIntent = new Intent(context, InternalReceiver.class);
		locationSendingIntent.setAction(Consts.LOCATION_SENDING_ACTION);		
		boolean locationSendingAlarmUp = (PendingIntent.getBroadcast(context, 11, locationSendingIntent, PendingIntent.FLAG_NO_CREATE) != null);
		
		return locationSendingAlarmUp;
	}

	private boolean isLocationRetrievingAlarmUp() {
		Intent locationRetrievingIntent = new Intent(context, InternalReceiver.class);
		locationRetrievingIntent.setAction(Consts.LOCATION_RETRIEVING_ACTION);		
		boolean locationRetrievingAlarmUp = (PendingIntent.getBroadcast(context, 22, locationRetrievingIntent, PendingIntent.FLAG_NO_CREATE) != null);
		
		return locationRetrievingAlarmUp;
	}

	private PendingIntent setupLocationSendingPendingIntent() {		
		Intent locationSendingIntent = new Intent(context, InternalReceiver.class);
		locationSendingIntent.setAction(Consts.LOCATION_SENDING_ACTION);
		return PendingIntent.getBroadcast(context, 11, locationSendingIntent, PendingIntent.FLAG_UPDATE_CURRENT);
	}

	private PendingIntent setupLocationRetrievingPendingIntent() {
		Intent locationRetrievingIntent = new Intent(context, InternalReceiver.class);
		locationRetrievingIntent.setAction(Consts.LOCATION_RETRIEVING_ACTION);
		return PendingIntent.getBroadcast(context, 22, locationRetrievingIntent, PendingIntent.FLAG_UPDATE_CURRENT);
	}
	
	/**
	 * <p>Starts inexact repeating alarm for requesting live geo areas.</p>
	 */
	public void startRequestingLiveGeosAlarm() {
		AlarmManager mgr = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
		PendingIntent pi = setupLiveGeoPendingIntent();
    	
    	mgr.setInexactRepeating(AlarmManager.ELAPSED_REALTIME, SystemClock.elapsedRealtime() + AlarmManager.INTERVAL_DAY,  AlarmManager.INTERVAL_DAY, pi);
	}
	
	/**
	 * <p>Cancels live geo requesting alarm.</p>
	 */
	public void cancelRequestingLiveGeosAlarm() {
		AlarmManager mgr = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
		
		PendingIntent pi = setupLiveGeoPendingIntent();
		if (pi != null) {
			mgr.cancel(pi);
			pi.cancel();
		}
	}
	
	private PendingIntent setupLiveGeoPendingIntent() {
		Intent requestingAreasIntent = new Intent(context, LiveGeoReceiver.class);
		requestingAreasIntent.setAction(LiveGeoUtil.REQUEST_AREAS_ACTION);
    	return PendingIntent.getBroadcast(context, 0, requestingAreasIntent, PendingIntent.FLAG_UPDATE_CURRENT);
	}
	
}
