package com.infobip.push.lib.util;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Random;
import java.util.TimeZone;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.Log;

import com.infobip.push.AbstractPushReceiver;
import com.infobip.push.LocationConfigurationException;
import com.infobip.push.PushConfigurationException;

/**
 * Singleton class containing push library utilities.
 *
 */
public class Util {

	private static final String DEBUG_TAG = "Push library";
	private static boolean debugMode = false;
	
	/**
	 * Check if Push library is in debug mode.
	 * @return <code>true</code> if Push library is in debug mode, or <code>false</code> otherwise. By default, Push library is not in debug mode.
	 * @see #setDebugModeEnabled(boolean)
	 */
	public static boolean isDebugModeEnabled() {
		return debugMode;
	}
	
	/**
	 * Enable or disable Push library debug mode. <br />
	 * Debug mode should be used during debugging of the application. When it is enabled, library will print a lot of debug info to the log output. By default, debug mode is disabled.
	 * @param debugMode Set to <code>true</code> to enable debug mode, or set to <code>false</code> to disable debug mode.
	 */
	public static void setDebugModeEnabled(boolean debugMode) {
		Util.debugMode = debugMode;
	}

	/**
	 * Performs logging in debug mode.
	 * @param message Message that will be logged.
	 */
	public static void LogDebug(String message) {
		if (Util.isDebugModeEnabled()) {
			Log.d(Util.DEBUG_TAG, message);
		}
	}
	
	/**
	 * Performs logging in debug mode by concatenating message and class name in the following form: <br />
	 * <strong>&lt;class name&gt;: &lt;message&gt;</strong>
	 * @param message Message that will be logged.
	 * @param sourceClass Class which name will be reflected and logged.
	 */
	public static void LogDebug(String message, Object sourceClass) {
		if (Util.isDebugModeEnabled()) {		
			Log.d(Util.DEBUG_TAG, sourceClass.getClass().getSimpleName() + ": " + message);
		}
	}
	
	/**
	 * Performs logging in error mode.
	 * @param message Message that will be logged.
	 */
	public static void LogError(String message) {
		if (Util.isDebugModeEnabled()) {
			Log.e(Util.DEBUG_TAG, message);
		}
	}
	
	/**
	 * Performs logging in error mode by concatenating message and class name in the following form: <br />
	 * <strong>&lt;class name&gt;: &lt;message&gt;</strong>
	 * @param message Message that will be logged.
	 * @param sourceClass Class which name will be reflected and logged.
	 */
	public static void LogError(String message, Object sourceClass) {
		if (Util.isDebugModeEnabled()) {		
			Log.e(Util.DEBUG_TAG, sourceClass.getClass().getSimpleName() + ": " + message);
		}
	}
	
	/**
	 * Performs logging in info mode.
	 * @param message Message that will be logged.
	 */
	public static void LogInfo(String message) {
		if (Util.isDebugModeEnabled()) {
			Log.i(Util.DEBUG_TAG, message);
		}
	}
	
	/**
	 * Performs logging in info mode by concatenating message and class name in the following form: <br />
	 * <strong>&lt;class name&gt;: &lt;message&gt;</strong>
	 * @param message Message that will be logged.
	 * @param sourceClass Class which name will be reflected and logged.
	 */
	public static void LogInfo(String message, Object sourceClass) {
		if (Util.isDebugModeEnabled()) {		
			Log.i(Util.DEBUG_TAG, sourceClass.getClass().getSimpleName() + ": " + message);
		}
	}
	
	/**
	 * Performs logging in verbose mode.
	 * @param message Message that will be logged.
	 */
	public static void LogVerbose(String message) {
		if (Util.isDebugModeEnabled()) {
			Log.v(Util.DEBUG_TAG, message);
		}
	}
	
	/**
	 * Performs logging in verbose mode by concatenating message and class name in the following form: <br />
	 * <strong>&lt;class name&gt;: &lt;message&gt;</strong>
	 * @param message Message that will be logged.
	 * @param sourceClass Class which name will be reflected and logged.
	 */
	public static void LogVerbose(String message, Object sourceClass) {
		if (Util.isDebugModeEnabled()) {		
			Log.v(Util.DEBUG_TAG, sourceClass.getClass().getSimpleName() + ": " + message);
		}
	}
	
	/**
	 * Performs logging in warning mode.
	 * @param message Message that will be logged.
	 */
	public static void LogWarn(String message) {
		if (Util.isDebugModeEnabled()) {
			Log.w(Util.DEBUG_TAG, message);
		}
	}
	
	/**
	 * Performs logging in warning mode by concatenating message and class name in the following form: <br />
	 * <strong>&lt;class name&gt;: &lt;message&gt;</strong>
	 * @param message Message that will be logged.
	 * @param sourceClass Class which name will be reflected and logged.
	 */
	public static void LogWarn(String message, Object sourceClass) {
		if (Util.isDebugModeEnabled()) {		
			Log.w(Util.DEBUG_TAG, sourceClass.getClass().getSimpleName() + ": " + message);
		}
	}
	
	/**
	 * Checks for available network connection.
	 * @return <code>true</code> if network connection is available, <code>false</code> otherwise.
	 */
	public static boolean isNetworkConnected(Context context) {		
		ConnectivityManager connMgr = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
	    NetworkInfo networkInfo = connMgr.getActiveNetworkInfo();
	    return (networkInfo != null) && (networkInfo.isConnected()); 
	}
	
	/**
	 * Checks if required permissions are defined.
	 * @param permissionName Name of the permission to be checked.
	 * @param context Context.
	 * @return <code>true</code> if permission is added, <code>false</code> otherwise.
	 * @throws PushConfigurationException 
	 */
	public static void checkPushRequiredPermission(String permissionName, Context context)  {
		if (!Util.isPermissionGranted(permissionName, context)) {
			throw new PushConfigurationException("Missing required permission in AndroidManifest.xml, add: " + permissionName);
		}
	}

	/**
	 * Checks if required location permissions are defined.
	 * @param permissionName Name of the permission to be checked.
	 * @param context Context.
	 * @return <code>true</code> if permission is added, <code>false</code> otherwise.
	 * @throws LocationConfigurationException 
	 */
	public static void checkLocationRequiredPermission(String permissionName, Context context)  {
		if (!Util.isPermissionGranted(permissionName, context)) {
			throw new LocationConfigurationException("Missing required permission in AndroidManifest.xml, add: " + permissionName);
		}
	}
	
	/**
	 * Checks if permission is defined in the AndroidManifest.xml.
	 * @param permissionName Name of the permission to be checked.
	 * @param context Context.
	 * @return <code>true</code> if permission is added, <code>false</code> otherwise.
	 */
	public static boolean isPermissionGranted(String permissionName, Context context) {
		PackageManager packageManager = context.getPackageManager();		
		return (packageManager.checkPermission(permissionName, context.getPackageName())) == (PackageManager.PERMISSION_GRANTED);
	}
	
	/**
	 * Checks if required services are defined.
	 * @param serviceName Name of the service to be checked.
	 * @param context Context.
	 */
	public static void checkPushRequiredService(String serviceName, Context context) {
		ComponentName componentName = new ComponentName(context.getPackageName(), serviceName);		
		try {
			context.getPackageManager().getServiceInfo(componentName, PackageManager.GET_META_DATA);
			
		} catch (PackageManager.NameNotFoundException localNameNotFoundException) {
			throw new PushConfigurationException("Missing required service in AndroidManifest.xml, add: " + serviceName);
		}
	}

	/**
	 * Checks if required services are defined.
	 * @param serviceName Name of the service to be checked.
	 * @param context Context.
	 */
	public static void checkLocationRequiredService(String serviceName, Context context) {
		ComponentName componentName = new ComponentName(context.getPackageName(), serviceName);		
		try {
			context.getPackageManager().getServiceInfo(componentName, PackageManager.GET_META_DATA);
			
		} catch (PackageManager.NameNotFoundException localNameNotFoundException) {
			throw new LocationConfigurationException("Missing required service in AndroidManifest.xml, add: " + serviceName);
		}
	}
	
	/**
	 * Checks if required receivers are defined.
	 * @param receiverName Name of the receiver to be checked.
	 * @param context Context.
	 */
	public static void checkPushRequiredReceiver(String receiverName, Context context) {
		ComponentName componentName = new ComponentName(context.getPackageName(), receiverName);
		try {
			context.getPackageManager().getReceiverInfo(componentName, PackageManager.GET_META_DATA);
			
		} catch (PackageManager.NameNotFoundException localNameNotFoundException) {
			throw new PushConfigurationException("Missing required receiver in AndroidManifest.xml, add: " + receiverName);
		}
	}

	/**
	 * Checks if required receivers are defined.
	 * @param receiverName Name of the receiver to be checked.
	 * @param context Context.
	 */
	public static void checkLocationRequiredReceiver(String receiverName, Context context) {
		ComponentName componentName = new ComponentName(context.getPackageName(), receiverName);
		try {
			context.getPackageManager().getReceiverInfo(componentName, PackageManager.GET_META_DATA);
			
		} catch (PackageManager.NameNotFoundException localNameNotFoundException) {
			throw new LocationConfigurationException("Missing required receiver in AndroidManifest.xml, add: " + receiverName);
		}
	}
	
	/**
	 * Gets offset from UTC for this time zone at the current time defined in minutes. The offset includes daylight 
	 * savings time if the specified date is within the daylight savings time period.
	 * @param context
	 * @return the date in minutes since January 1, 1970 00:00:00 UTC 
	 */
	public static int getTimeZoneOffset(Context context) {
		TimeZone zone = TimeZone.getDefault();
		int offsetMinutes = zone.getOffset(System.currentTimeMillis()) / 1000 / 60;
		return offsetMinutes;
	}
	
	/**
	 * Gets the name of the network operator.
	 * @param context
	 * @return
	 */
	public static String getNetworkOperatorName(Context context) {
		TelephonyManager manager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
		String operatorName = manager.getNetworkOperatorName();
		return operatorName;
	}
	
	/**
	 * Gets mobile country code (MCC).
	 * @return MCC
	 */
	public static String getMcc(Context context) {
		TelephonyManager manager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
		String simOperator = manager.getSimOperator();
		String mcc = null;

		if (!TextUtils.isEmpty(simOperator) && (simOperator.length() > 4) && (simOperator.length() < 7)) {
			mcc = simOperator.substring(0, 3);
		}		
		return mcc;
	}
	
	/**
	 * Gets mobile network code (MNC).
	 * @return MNC
	 */
	public static String getMnc(Context context) {
		TelephonyManager manager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
		String simOperator = manager.getSimOperator();
		String mnc = null;
		
		if (!TextUtils.isEmpty(simOperator) && (simOperator.length() > 4) && (simOperator.length() < 7)) {
			mnc = simOperator.substring(3, simOperator.length());
		}		
		return mnc;
	}
	
	/**
	 * Gets the application version.
	 * @return Application version.
	 */
    public static int getAppVersion(Context context) {
        try {
            PackageInfo packageInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
            return packageInfo.versionCode;
        } catch (NameNotFoundException e) {
            // should never happen
            throw new RuntimeException("Could not get package name: " + e);
        }
    }
    
    /**
	 * <p>Gets application name.</p>
	 * @return Application name.
	 */
	public static String getAppName(Context context) {
		PackageManager pm = context.getPackageManager();
		ApplicationInfo ai;
		try {
		    ai = pm.getApplicationInfo(context.getPackageName(), 0);
		} catch (NameNotFoundException e) {
		    ai = null;
		}
		
		String appName = "";
		if (ai != null) {
			appName = pm.getApplicationLabel(ai).toString();
		}

		return appName;
	}
	
	/**
	 * Invoked when an intent is going to be sent to the {@link AbstractPushReceiver} class instance. If an error occurs reason is proceeded. 
	 */
	public static void sendCustomBroadcast(Context context, String receiverAction, int reason) {		
		Intent intent = new Intent(receiverAction);
		
		if (reason != -1) {
			intent.putExtra(Consts.EXTRA_REASON, reason);
		}
		intent.addCategory(context.getPackageName());
		context.sendBroadcast(intent, context.getPackageName() + Consts.GCM_PERMISSION_C2D_MESSAGE);
	}
	
	/**
	 * Generates unique device ID.
	 * @return Unique device ID.
	 */
	public static String generateDeviceId() {
		Date currentDate = new Date();
		SimpleDateFormat currentDateFormat = new SimpleDateFormat("yyMMddHHmmssSSS", Locale.getDefault());
		currentDateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
		String utcTime = currentDateFormat.format(currentDate);
		
		String randomChars = "";
		for (int i = 0; i < 4; i++) {
			randomChars += randomChar();
		}
		
		String deviceId = "A" + utcTime + randomChars;
		return deviceId;
	}
	
	private static char randomChar() {
	    int randomNo = new Random().nextInt(52);
	    char base = (randomNo < 26) ? 'A' : 'a';
	    return (char) (base + randomNo % 26);
	}
	
}
