package com.infobip.push.media;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.Window;
import android.webkit.WebChromeClient;
import android.webkit.WebView;
import android.widget.RelativeLayout;

import com.infobip.push.AbstractPushReceiver;
import com.infobip.push.PushNotification;
import com.infobip.push.lib.util.Util;

/**
 * <p>Media activity is intended for displaying the content of the media push. Media content refers to the multimedia content 
 * (text, audio, images, video) wrapped inside HTML tags.</p>
 * 
 * <p><b>Default behavior of this activity:</b>
 * <ul>
 * <li>activity is loaded without title (reference: {@link Window#FEATURE_NO_TITLE})</li>
 * <li>clicked links from HTML data are loaded inside this activity (not the web browser)</li>
 * <li>activity is able to load media data or URL provided in Intents extras</li>
 * <li>pressing back button goes back in the {@link MediaWebView} history</li>
 * <li>destroying activity removes all views and clears browsing history</li>
 * </ul>
 * </p>
 * 
 * <p><b>Customizing activity behavior:</b>
 * <ul>
 * <li>for showing progress (reference: {@link Window#FEATURE_PROGRESS}) indicating page loading, instead of showing activity without title,
 * 		use {@link Intent#setAction(String)} where String value is {@link #START_WITH_PROGRESS} action that starts this activity</li>
 * <li>for displaying media data, put {@link #EXTRA_DATA} with media data as a value on Intent that starts this activity</li>
 * <li>for loading an URL, put {@link #EXTRA_URL} with URL as a value on Intent that starts this activity</li>
 * </ul>
 * </p>
 * 
 * <p><b>Examples:</b>
 * <ol> 
 * <li>Default notification handling - setup starting activity intent and start activity from 
 * {@link AbstractPushReceiver#onNotificationOpened(PushNotification, Context)} method</li>
 * <li>Overridden default message handling - put the intent below on the {@link PendingIntent#getActivity(Context, int, Intent, int)}</li>
 * </ol>
 * </p>
 * 
 * Setting up starting activity intent:
 * <pre>
 * Intent intent = new Intent(context, MediaActivity.class);
 * intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
 * intent.setAction(MediaActivity.START_WITH_PROGRESS);
 * intent.putExtra(MediaActivity.EXTRA_DATA, notification.getMediaData());
 * </pre>
 * Starting activity:
 * <pre>
 * context.startActivity(intent);
 * </pre>
 * 
 */
public class MediaActivity extends Activity {

	/**
	 * <p>Put this value as an action of an intent for starting this activity if you want to show progress (reference: {@link Window#FEATURE_PROGRESS}) indicating 
	 * page loading, instead of showing an activity without a title.</p>
	 * Constant value: {@value}
	 */
	public static final String START_WITH_PROGRESS 	= "progress";
	
	/**
	 * <p>Put this value as an extra key to an intent for starting this activity if you want to load media data received in a push notification.</p>
	 * Constant value: {@value}
	 */
	public static final String EXTRA_DATA 			= "mediaData";
	
	/**
	 * <p>Put this value as an extra key to an intent for starting this activity if you want to load URL.</p>
	 * Constant value: {@value}
	 */
	public static final String EXTRA_URL 			= "url";
	
	RelativeLayout relativeLayout;
	MediaWebView webView;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		if (isWithProgress(getIntent().getAction())) {
			requestWindowFeature(Window.FEATURE_PROGRESS);			
		} else {
			requestWindowFeature(Window.FEATURE_NO_TITLE);
		}
			
		relativeLayout = new RelativeLayout(this);
        RelativeLayout.LayoutParams rlp = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT,
                RelativeLayout.LayoutParams.MATCH_PARENT);

        webView = new MediaWebView(this);
        if (isWithProgress(getIntent().getAction())) {
        	webView.setWebChromeClient(new MediaWebChromeClient());
        }
        
        webView.setLayoutParams(rlp);
        relativeLayout.addView(webView);
        
        setContentView(relativeLayout, rlp);

		String mediaData = getIntent().getStringExtra("mediaData");
		String url = getIntent().getStringExtra("url");
		
		if (!TextUtils.isEmpty(mediaData)) {
			webView.loadMediaData(mediaData);
			
		} else if (!TextUtils.isEmpty(url)) {
			webView.loadMediaUrl(url);;
		
		} else {
			Util.LogWarn("Media content is not provided. Make sure that you've put in Intent's extras mediaData or url.");
		}
		
	}
	
	@Override
	public void onBackPressed()	{
		// Check if there's history on the WebView
	    if(webView.canGoBack()) {
	        webView.goBack();
	        return;
	    }  
	    super.onBackPressed();
	}
	
	@Override
	protected void onDestroy() {
		super.onDestroy();
		relativeLayout.removeView(webView);
		webView.clearHistory();
		webView.removeAllViews();
	    webView.destroy();
	}
	
	private class MediaWebChromeClient extends WebChromeClient {
		
		@Override
		public void onProgressChanged(WebView view, int newProgress) {
			super.onProgressChanged(view, newProgress);
			setProgress(newProgress * 100);
		}
	}
	
	private boolean isWithProgress(String action) {
		return START_WITH_PROGRESS.equals(action);
	}
	
}
