package com.infobip.push.media;

import org.apache.http.protocol.HTTP;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebSettings.PluginState;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.infobip.push.lib.util.Util;

/**
 * <p>MediaWebView is a subclass of the  {@link WebView} and thereby inherits its methods, fields and default configuration. 
 * Additionally, default MediaWebView configuration has zoom controls, Javascript, plugins, file and content access enabled. 
 * Links which are clicked inside this view will be opened in the same view.</p>
 * 
 * <p>Usage of the MediaWebView is the same as the usage of the Android's default WebView. You can add it as an element 
 * to activity's layout or set it directly as an activity's content view in activity's onCreate() method:
 * <pre>
 * protected void onCreate(Bundle savedInstanceState) {
 * 	super.onCreate(savedInstanceState);
 * 
 * 	MediaWebView mediaView = new MediaWebView(this);
 * 	setContentView(mediaView);
 * }
 * </pre>
 * <p>For loading media data use {@link #loadMediaData(String)} or for loading URL call {@link #loadMediaUrl(String)}. These methods 
 * can be called from within activity's onCreate() method.</p>
 *
 */
public class MediaWebView extends WebView {
	
	public MediaWebView(Context context) {
		super(context);
		setupMediaWebView();
	}
	
	public MediaWebView(Context context, AttributeSet attrs) {
		super(context, attrs);
		setupMediaWebView();
	}

	public MediaWebView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		setupMediaWebView();
	}

	@SuppressWarnings("deprecation")
	@SuppressLint({ "NewApi", "SetJavaScriptEnabled" }) 
	protected void setupMediaWebView() {
		WebSettings rvSettings = getSettings();
	  
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
			rvSettings.setAllowContentAccess(true);
			rvSettings.setDisplayZoomControls(false);
		}
		
		rvSettings.setPluginState(PluginState.ON);
		rvSettings.setJavaScriptEnabled(true);
		rvSettings.setAllowFileAccess(true);
		rvSettings.setUseWideViewPort(true);
		rvSettings.setLoadWithOverviewMode(true);
		rvSettings.setBuiltInZoomControls(true);
		rvSettings.setDefaultTextEncodingName(HTTP.UTF_8);
		
		setWebChromeClient(new WebChromeClient());
		setWebViewClient(new WebViewClient());
	}

	/**
	 * <p>Loads the given media data into this view. Data is encoded in UTF-8. To use another encoding, rather use 
	 * {@link #loadData(String, String, String)} method.</p>
	 * @param mediaData Multimedia content (text, images, video, audio) wrapped inside HTML tags.
	 */
	public void loadMediaData(String mediaData) {
	    
		if (TextUtils.isEmpty(mediaData)) {
			Util.LogWarn("Provided media content is null or empty. Unable to load it into the MediaWebView.");
			return;
		}
		
		loadDataWithBaseURL(null, mediaData, "text/html", HTTP.UTF_8, null);
	}
	
	/**
	 * <p>Loads the given URL if the URL is provided.</p>
	 * @param mediaUrl URL of the content to display.
	 */
	public void loadMediaUrl(String mediaUrl) {
		
		if (TextUtils.isEmpty(mediaUrl)) {
			Util.LogWarn("Provided media url is null or empty. Unable to load it into the MediaWebView.");
			return;
		}
		
		loadUrl(mediaUrl);
	}

}
